@@ -65,6 +65,7 @@ PyObject* exterior_electron_density_region(PyObject *self, PyObject *args);
 PyObject* plane_wave_grid(PyObject *self, PyObject *args);
 PyObject* tci_overlap(PyObject *self, PyObject *args);
 PyObject *pwlfc_expand(PyObject *self, PyObject *args);
+PyObject *pwlfc_expand_old(PyObject *self, PyObject *args);
 PyObject *pw_insert(PyObject *self, PyObject *args);
 PyObject *pw_precond(PyObject *self, PyObject *args);
 PyObject *fd_precond(PyObject *self, PyObject *args);
@@ -192,13 +193,27 @@ PyObject* axpbz_gpu(PyObject *self, PyObject *args);
 PyObject* fill_gpu(PyObject *self, PyObject *args);
 PyObject* pwlfc_expand_gpu(PyObject *self, PyObject *args);
 PyObject* pw_insert_gpu(PyObject *self, PyObject *args);
+PyObject* pw_norm_kinetic_gpu(PyObject *self, PyObject *args);
+PyObject* pw_norm_gpu(PyObject *self, PyObject *args);
+
 PyObject* pw_amend_insert_realwf_gpu(PyObject *self, PyObject *args);
 PyObject* add_to_density_gpu(PyObject* self, PyObject* args);
 PyObject* dH_aii_times_P_ani_gpu(PyObject* self, PyObject* args);
 PyObject* evaluate_lda_gpu(PyObject* self, PyObject* args);
 PyObject* evaluate_pbe_gpu(PyObject* self, PyObject* args);
 PyObject* calculate_residual_gpu(PyObject* self, PyObject* args);
+
+#endif // GPAW_GPU
+
+#ifdef GPAW_WITH_MAGMA
+#include "magma_gpaw.h"
+PyObject* eigh_magma_dsyevd(PyObject* self, PyObject* args);
+PyObject* eigh_magma_zheevd(PyObject* self, PyObject* args);
+#ifdef GPAW_GPU
+PyObject* eigh_magma_dsyevd_gpu(PyObject* self, PyObject* args);
+PyObject* eigh_magma_zheevd_gpu(PyObject* self, PyObject* args);
 #endif
+#endif // GPAW_WITH_MAGMA
 
 static PyMethodDef functions[] = {
     {"pawexxvv", pawexxvv, METH_VARARGS, 0},
@@ -225,6 +240,7 @@ static PyMethodDef functions[] = {
     {"eed_region", exterior_electron_density_region, METH_VARARGS, 0},
     {"plane_wave_grid", plane_wave_grid, METH_VARARGS, 0},
     {"pwlfc_expand", pwlfc_expand, METH_VARARGS, 0},
+    {"pwlfc_expand_old", pwlfc_expand_old, METH_VARARGS, 0},
     {"pw_insert", pw_insert, METH_VARARGS, 0},
     {"pw_precond", pw_precond, METH_VARARGS, 0},
     {"fd_precond", fd_precond, METH_VARARGS, 0},
@@ -361,13 +377,27 @@ static PyMethodDef functions[] = {
     {"fill_gpu", fill_gpu, METH_VARARGS, 0},
     {"pwlfc_expand_gpu", pwlfc_expand_gpu, METH_VARARGS, 0},
     {"pw_insert_gpu", pw_insert_gpu, METH_VARARGS, 0},
+    {"pw_norm_kinetic_gpu", pw_norm_kinetic_gpu, METH_VARARGS, 0},
+    {"pw_norm_gpu", pw_norm_gpu, METH_VARARGS, 0},
     {"pw_amend_insert_realwf_gpu", pw_amend_insert_realwf_gpu, METH_VARARGS, 0},
     {"add_to_density_gpu", add_to_density_gpu, METH_VARARGS, 0},
     {"dH_aii_times_P_ani_gpu", dH_aii_times_P_ani_gpu, METH_VARARGS, 0},
     {"evaluate_lda_gpu", evaluate_lda_gpu, METH_VARARGS, 0},
     {"evaluate_pbe_gpu", evaluate_pbe_gpu, METH_VARARGS, 0},
     {"calculate_residuals_gpu", calculate_residual_gpu, METH_VARARGS, 0},
+
 #endif // GPAW_GPU
+
+#ifdef GPAW_WITH_MAGMA
+{"eigh_magma_dsyevd", eigh_magma_dsyevd, METH_VARARGS, 0},
+{"eigh_magma_zheevd", eigh_magma_zheevd, METH_VARARGS, 0},
+#ifdef GPAW_GPU
+{"eigh_magma_dsyevd_gpu", eigh_magma_dsyevd_gpu, METH_VARARGS, 0},
+{"eigh_magma_zheevd_gpu", eigh_magma_zheevd_gpu, METH_VARARGS, 0},
+#endif
+#endif // GPAW_WITH_MAGMA
+
+
     {0, 0, 0, 0}
 };
 
@@ -387,6 +417,14 @@ extern PyTypeObject lxcXCFunctionalType;
 #endif
 
 
+static void gpaw_module_cleanup(void *m)
+{
+#ifdef GPAW_WITH_MAGMA
+    // Assuming GPAW calls magma_init() during module startup
+    magma_finalize();
+#endif
+}
+
 static struct PyModuleDef moduledef = {
     PyModuleDef_HEAD_INIT,
     "_gpaw",
@@ -396,7 +434,7 @@ static struct PyModuleDef moduledef = {
     NULL,
     NULL,
     NULL,
-    NULL
+    gpaw_module_cleanup
 };
 
 static PyObject* moduleinit(void)
@@ -462,8 +500,25 @@ static PyObject* moduleinit(void)
 #else
     PyObject_SetAttrString(m, "have_openmp", Py_False);
 #endif
+
+#ifdef GPAW_WITH_MAGMA
+    PyObject_SetAttrString(m, "have_magma", Py_True);
+
+    // MAGMA needs to be globally initialized, but keeps track of accumulated
+    // magma_init() calls. So it's safe to call it inside GPAW, even if other
+    // libs are also doing it.
+
+    // FIXME: Where should GPAW call magma_init()?
+    // Should not be in GPU-specific init because magma can work without GPU too.
+    // However it needs to come AFTER cudaSetValidDevices and cudaSetDeviceFlags.
+    // Calling it here could become a problem if Python-side GPU init does more than setDevice(...)
+    magma_init();
+
+#else
+    PyObject_SetAttrString(m, "have_magma", Py_False);
+#endif
     // Version number of C-code.  Keep in sync with gpaw/_broadcast_imports.py
-    PyObject_SetAttrString(m, "version", PyLong_FromLong(9));
+    PyObject_SetAttrString(m, "version", PyLong_FromLong(10));
 
     Py_INCREF(&LFCType);
     Py_INCREF(&OperatorType);
