@@ -77,8 +77,12 @@ module load libvdwxc/0.4.0-{fullchain}
 }
 
 module_cmds_arch_dependent = """\
-if [ "$CPU_ARCH" == "icelake" ] && [ {fullchain} == "foss-2023a" ];\
+if ( [ "$CPU_ARCH" == "icelake" ] || [ "$CPU_ARCH" == "skylake_el8" ] )\
+ && [ {fullchain} == "foss-2023a" ];\
 then module load CuPy/12.3.0-{fullchain}-CUDA-12.1.1;fi
+if [ "$SLURM_JOB_PARTITION" == "a100" ] \
+ || [ "$SLURM_JOB_PARTITION" == "sm3090el8" ];\
+then export GPAW_USE_GPUS=1;export GPAW_NEW=1;fi
 """
 
 
@@ -189,6 +193,11 @@ def main():
                         help='Default is foss.')
     parser.add_argument('--dftd3', action='store_true',
                         help='Also build DFTD3.')
+    parser.add_argument('--gpaw-branch', default='master',
+                        help='Check out a particular gpaw branch')
+    parser.add_argument('--run-benchmarks', action='store_true',
+                        help='Also submit the GPAW benchmark suite'
+                             ' with the checked out branch.')
     parser.add_argument('--recompile', action='store_true',
                         help='Recompile the GPAW C-extensions in an '
                         'exising venv.')
@@ -204,16 +213,23 @@ def main():
     activate = venv / 'bin/activate'
     gpaw = venv / 'gpaw'
 
+    intel_only = args.toolchain == 'intel'
+
+    def run_benchmarks():
+        benchmarkworkflow = gpaw / 'gpaw/benchmark/niflheim-myqueue.py'
+        run(f'. {activate} && mkdir benchmarks-{args.gpaw_branch} && cd benchmarks-{args.gpaw_branch} && mq workflow {benchmarkworkflow}')
+
     if args.recompile:
-        compile_gpaw_c_code(gpaw, activate)
+        compile_gpaw_c_code(gpaw, activate, intel_only)
+        if args.run_benchmarks:
+            run_benchmarks()
+
         return 0
 
     # Sanity checks
     if args.toolchain not in ('foss', 'intel'):
         raise ValueError(f'Unsupported toolchain "{args.toolchain}"')
 
-    intel_only = args.toolchain == 'intel'
-
     module_cmds = module_cmds_all.format(**toolchains[args.toolchain])
     if not args.piponly:
         module_cmds += module_cmds_easybuild.format(
@@ -264,8 +280,9 @@ def main():
                      'scikit-learn']
     run(f'. {activate} && pip install -q -U ' + ' '.join(packages))
 
-    for name in ['ase', 'gpaw']:
-        run(f'git clone -q https://gitlab.com/{name}/{name}.git')
+    run('git clone -q https://gitlab.com/ase/ase.git')
+    branch = '' if args.gpaw_branch == 'master' else f'-b {args.gpaw_branch} '
+    run(f'git clone -q {branch}https://gitlab.com/gpaw/gpaw.git')
 
     run(f'. {activate} && pip install -q -e ase/')
 
@@ -303,7 +320,7 @@ def main():
     Path(f'lib/python{version}/site-packages/niflheim.pth').write_text(pth)
 
     # Install extra basis-functions:
-    run(f'. {activate} && gpaw install-data --basis --version=20000 '
+    run(f'. {activate} && gpaw install-data --basis --version=0.9.20000 '
         f'{venv} --no-register')
 
     extra = activate_extra.format(venv=venv)
@@ -329,6 +346,9 @@ def main():
     # Run tests:
     run(f'. {activate} && ase info && gpaw test')
 
+    if args.run_benchmarks:
+        run_benchmarks()
+
     return 0
 
 
