@@ -12,7 +12,7 @@ fxc0 = FXCCorrelation('CO.ralda.pbe_wfcs_C.gpw',
 E0_i = fxc0.calculate()
 
 f = paropen('CO.ralda_rapbe_C.dat', 'w')
-for ecut, E0 in zip(fxc0.rpa.ecut_i, E0_i):
+for ecut, E0 in zip(fxc0.rpa.integral.ecut_i, E0_i):
     print(ecut * Hartree, E0, file=f)
 f.close()
 
@@ -24,6 +24,6 @@ rpa0 = RPACorrelation('CO.ralda.pbe_wfcs_C.gpw',
 E0_i = rpa0.calculate()
 
 f = paropen('CO.ralda_rpa_C.dat', 'w')
-for ecut, E0 in zip(rpa0.ecut_i, E0_i):
+for ecut, E0 in zip(rpa0.integral.ecut_i, E0_i):
     print(ecut * Hartree, E0, file=f)
 f.close()
