@@ -9,7 +9,7 @@ E1_i = rpa.calculate()
 rpa = RPACorrelation('N2.gpw', nblocks=8,
                      txt='rpa_N2.txt', ecut=400)
 E2_i = rpa.calculate()
-ecut_i = rpa.ecut_i
+ecut_i = rpa.integral.ecut_i
 
 f = paropen('rpa_N2.dat', 'w')
 for ecut, E1, E2 in zip(ecut_i, E1_i, E2_i):
