@@ -7,30 +7,31 @@ from gpaw.lcaotddft.magneticmomentwriter import MagneticMomentWriter
 setup_paths.insert(0, '.')
 
 
-def main(kick):
+def main(kick, gauge):
     kick_strength = [0., 0., 0.]
     kick_strength['xyz'.index(kick)] = 1e-5
 
     td_calc = LCAOTDDFT('gs.gpw', txt=f'td-{kick}.out')
 
-    DipoleMomentWriter(td_calc, f'dm-{kick}.dat')
+    DipoleMomentWriter(td_calc, f'dm-{gauge}_{kick}.dat')
 
     # Origin: center of mass
-    MagneticMomentWriter(td_calc, f'mm-COM-{kick}.dat',
+    MagneticMomentWriter(td_calc, f'mm-COM-{gauge}_{kick}.dat',
                          origin='COM')
 
     # Origin: center of mass + 5 Å shift
     for shift_axis in 'xyz':
         origin_shift = [0, 0, 0]
         origin_shift['xyz'.index(shift_axis)] = 5
-        MagneticMomentWriter(td_calc, f'mm-COM+{shift_axis}-{kick}.dat',
+        MagneticMomentWriter(td_calc,
+                             f'mm-COM+{shift_axis}-{gauge}_{kick}.dat',
                              origin='COM', origin_shift=origin_shift)
 
     # Origin: arbitrary coordinate
-    MagneticMomentWriter(td_calc, f'mm-123-{kick}.dat',
+    MagneticMomentWriter(td_calc, f'mm-123-{gauge}_{kick}.dat',
                          origin='zero', origin_shift=[1, 2, 3])
 
-    td_calc.absorption_kick(kick_strength)
+    td_calc.absorption_kick(kick_strength, gauge=gauge)
     td_calc.propagate(10, 2000)
 
     td_calc.write(f'td-{kick}.gpw', mode='all')
@@ -40,5 +41,6 @@ if __name__ == '__main__':
     import argparse
     parser = argparse.ArgumentParser()
     parser.add_argument('--kick', default='x')
+    parser.add_argument('--gauge', default='length')
     kwargs = vars(parser.parse_args())
     main(**kwargs)
