@@ -13,11 +13,11 @@ args.update(dict(name=name, exx=True))
 
 # Generate setup
 generator = Generator(atom, xc, scalarrel=True)
-generator.run(write_xml=True, **args)
+setup = generator.run(write_xml=True, **args)
 
 # Generate basis
-bm = BasisMaker(atom, name=f'{name}.{xc}', xc=xc, run=False)
-bm.generator.run(write_xml=False, **args)
+bm = BasisMaker.from_setup_and_generator(
+    setup, generator, name=f'{name}.{xc}')
 basis = bm.generate(zetacount=2, polarizationcount=0,
                     jvalues=[0, 1, 2])  # include d, s and p
 basis.write_xml()
