@@ -6,10 +6,12 @@ import os
 import sys
 import contextlib
 from pathlib import Path
-from typing import List, Union, Any, TYPE_CHECKING
+from typing import List, Any, TYPE_CHECKING
+import warnings
 
-__version__ = '25.1.0'
-__ase_version_required__ = '3.23.0'
+
+__version__ = '25.7.0'
+__ase_version_required__ = '3.25.0'
 
 __all__ = ['GPAW',
            'Mixer', 'MixerSum', 'MixerDif', 'MixerSum2',
@@ -20,8 +22,19 @@ __all__ = ['GPAW',
            'PW', 'LCAO', 'FD',
            'restart']
 
-GPAW_NEW = bool(int(os.environ.get('GPAW_NEW') or 0))
-setup_paths: List[Union[str, Path]] = []
+boolean_envvars = {
+    'GPAW_NEW',
+    'GPAW_CPUPY',
+    'GPAW_USE_GPUS',
+    'GPAW_TRACE',
+    'GPAW_NO_C_EXTENSION',
+    'GPAW_MPI4PY'}
+allowed_envvars = {
+    *boolean_envvars,
+    'GPAW_MPI_OPTIONS',
+    'GPAW_MPI',
+    'GPAW_SETUP_PATH'}
+
 is_gpaw_python = '_gpaw' in sys.builtin_module_names
 dry_run = 0
 
@@ -30,6 +43,11 @@ debug: bool = (TYPE_CHECKING or
                'pytest' in sys.modules or
                bool(sys.flags.debug))
 
+if debug:
+    for var in os.environ:
+        if var.startswith('GPAW') and var not in allowed_envvars:
+            warnings.warn(f'Unknown GPAW environment varable: {var}')
+
 
 @contextlib.contextmanager
 def disable_dry_run():
@@ -89,8 +107,6 @@ def get_libraries() -> dict[str, str]:
 def parse_arguments(argv):
     from argparse import (ArgumentParser, REMAINDER,
                           RawDescriptionHelpFormatter)
-    import warnings
-
     # With gpaw-python BLAS symbols are in global scope and we need to
     # ensure that NumPy and SciPy use symbols from their own dependencies
     if is_gpaw_python:
@@ -147,17 +163,41 @@ def parse_arguments(argv):
 
 
 def __getattr__(attr: str) -> Any:
+    for attr_getter in _lazy_import, _get_gpaw_env_vars:
+        try:
+            result = attr_getter(attr)
+        except AttributeError:
+            continue
+        return globals().setdefault(attr, result)
+    raise _module_attr_error(attr)
+
+
+def __dir__() -> List[str]:
+    """
+    Get the (1) normally-present module attributes, (2) lazily-imported
+    objects, and (3) envrionmental variables starting with `GPAW_`.
+    """
+    return list({*globals(),
+                 *all_lazy_imports,  # From `_lazy_import()`
+                 *{*boolean_envvars,  # From `_get_gpaw_env_vars()`
+                   *(var for var in os.environ if var.startswith('GPAW_'))}})
+
+
+def _module_attr_error(attr: str, *args, **kwargs) -> AttributeError:
+    return AttributeError(f'{__getattr__.__module__}: '
+                          f'no attribute named `.{attr}`',
+                          *args, **kwargs)
+
+
+def _lazy_import(attr: str) -> Any:
     """
     Implement the lazy importing of classes in submodules."""
-
     import importlib
 
     try:
         import_target = all_lazy_imports[attr]
     except KeyError:
-        raise AttributeError(
-            f'{__getattr__.__module__}: no attribute named `.{attr!r}`'
-        ) from None
+        raise _module_attr_error(attr) from None
 
     module, sep, target = import_target.rpartition('.')
     assert module and all(chunk.isidentifier() for chunk in module.split('.'))
@@ -166,11 +206,12 @@ def __getattr__(attr: str) -> Any:
     return getattr(importlib.import_module(module), target)
 
 
-def __dir__() -> List[str]:
-    """
-    Get the normally-present module attributes and the lazily-imported objects.
-    """
-    return [*globals(), *all_lazy_imports]
+def _get_gpaw_env_vars(attr: str) -> bool | str:
+    if attr in boolean_envvars:
+        return bool(int(os.environ.get(attr) or 0))
+    if attr in allowed_envvars and attr in os.environ:
+        return os.environ[attr]
+    raise _module_attr_error(attr)
 
 
 all_lazy_imports = dict(
@@ -194,6 +235,16 @@ all_lazy_imports = dict(
     PW='gpaw.wavefunctions.pw.PW')
 
 
+# Make sure e.g. GPAW_NEW=0 will set GPAW_NEW=False
+# (`__getattr__()` magic handles the other boolean environment
+# variables, but GPAW_NEW is used within the same script, so it needs to
+# concretely exist in the namespace)
+GPAW_NEW = _get_gpaw_env_vars('GPAW_NEW')
+
+if os.uname().machine == 'wasm32':
+    GPAW_NO_C_EXTENSION = True
+
+
 class BroadcastImports:
     def __enter__(self):
         from gpaw._broadcast_imports import broadcast_imports
@@ -255,7 +306,7 @@ if debug:
             a.fill(-42)
         return a
 
-    np.empty = empty
+    np.empty = empty  # type: ignore[misc]
     np.empty_like = empty_like
 
 if TYPE_CHECKING:
@@ -265,6 +316,8 @@ elif GPAW_NEW:
 else:
     all_lazy_imports['GPAW'] = 'gpaw.calculator.GPAW'
 
+all_lazy_imports['get_calculation_info'] = 'gpaw.calcinfo.get_calculation_info'
+
 
 def restart(filename, Class=None, **kwargs):
     if Class is None:
@@ -288,14 +341,19 @@ def initialize_data_paths():
     try:
         setup_paths[:0] = os.environ['GPAW_SETUP_PATH'].split(os.pathsep)
     except KeyError:
-        if len(setup_paths) == 0:
-            if os.pathsep == ';':
-                setup_paths[:] = [r'C:\gpaw-setups']
-            else:
-                setup_paths[:] = ['/usr/local/share/gpaw-setups',
-                                  '/usr/share/gpaw-setups']
+        pass
 
 
+def standard_setup_paths() -> list[str | Path]:
+    try:
+        import gpaw_data
+    except ModuleNotFoundError:
+        return []
+    else:
+        return [gpaw_data.datapath()]
+
+
+setup_paths = standard_setup_paths()
 read_rc_file()
 initialize_data_paths()
 
