@@ -24,13 +24,24 @@ import sys
 import marshal
 from importlib.machinery import PathFinder, ModuleSpec
 
+from gpaw import GPAW_NO_C_EXTENSION, GPAW_MPI4PY
 import gpaw.cgpaw as cgpaw
 
-if getattr(cgpaw, 'version', 0) != 9:
-    raise ImportError('Please recompile GPAW''s C-extensions!')
+cgpaw_version = getattr(cgpaw, 'version', 0)
+if not GPAW_NO_C_EXTENSION and cgpaw_version != 10:
+    improvement = ''
+    if cgpaw_version == 9:
+        improvement = ('GPAW has now much reduced memory consumption due to '
+                       'optimized pwlfc_expand function in new GPAW. Enjoy. ')
+
+    raise ImportError(improvement + 'Please recompile GPAW''s C-extensions!')
 
 
-if hasattr(cgpaw, 'Communicator'):
+if GPAW_MPI4PY:
+    from gpaw.mpi4pywrapper import MPI4PYWrapper
+    from mpi4py.MPI import COMM_WORLD
+    world = MPI4PYWrapper(COMM_WORLD)
+elif hasattr(cgpaw, 'Communicator'):
     if '_gpaw' not in sys.builtin_module_names:
         libmpi = os.environ.get('GPAW_MPI', 'libmpi.so')
         import ctypes
