@@ -214,7 +214,7 @@ class AtomGridDescriptor(EquidistantRadialGridDescriptor):
     def __init__(self, h, rcut):
         ng = int(float(rcut) / h + 0.5) - 1
         rcut = ng * h
-        EquidistantRadialGridDescriptor.__init__(self, h, ng, h0=h)
+        super().__init__(h, ng, h0=h)
         self.sdisp_cd = np.empty((3, 2))
         self.comm = mpi.serial_comm
         self.pbc_c = np.zeros(3, bool)
@@ -265,7 +265,7 @@ class AtomOccupations(OccupationNumberCalculator):
 
     def __init__(self, f_sln):
         self.f_sln = f_sln
-        OccupationNumberCalculator.__init__(self)
+        super().__init__()
 
     def _calculate(self,
                    nelectrons,
@@ -291,16 +291,15 @@ class AtomPAW(GPAW):
         self.symbol = symbol
 
         gd = AtomGridDescriptor(h, rcut)
-        GPAW.__init__(self,
-                      mode=MakeWaveFunctions(gd),
-                      eigensolver=AtomEigensolver(gd, f_sln),
-                      poissonsolver=AtomPoissonSolver(),
-                      nbands=sum([(2 * l + 1) * len(f_n)
-                                  for l, f_n in enumerate(f_sln[0])]),
-                      communicator=mpi.serial_comm,
-                      parallel=dict(augment_grids=False),
-                      occupations=AtomOccupations(f_sln),
-                      **kwargs)
+        super().__init__(mode=MakeWaveFunctions(gd),
+                         eigensolver=AtomEigensolver(gd, f_sln),
+                         poissonsolver=AtomPoissonSolver(),
+                         nbands=sum([(2 * l + 1) * len(f_n)
+                                     for l, f_n in enumerate(f_sln[0])]),
+                         communicator=mpi.serial_comm,
+                         parallel=dict(augment_grids=False),
+                         occupations=AtomOccupations(f_sln),
+                         **kwargs)
         # Initialize function will raise an error unless we set a (bogus) cell
         self.initialize(Atoms(symbol, calculator=self,
                               cell=np.eye(3)))
@@ -331,15 +330,11 @@ class AtomPAW(GPAW):
         """Create BasisFunctions object with pseudo wave functions."""
         from gpaw.basis_data import Basis, BasisFunction
         assert self.wfs.nspins == 1
-
         d = self.wfs.gd.r_g[0]
         ng = self.wfs.gd.N + 1
         rgd = EquidistantRadialGridDescriptor(d, ng)
-        basis = Basis(self.symbol, basis_name, readxml=False, rgd=rgd)
-        basis.generatorattrs = {}  # attrs of the setup maybe
-        basis.generatordata = 'AtomPAW'  # version info too?
 
-        bf_j = basis.bf_j
+        bf_j = []
         for l, n, f, eps, psit_G in self.state_iter():
             n = [N for N, L in zip(n_j, l_j) if L == l][n]
             bf_g = rgd.empty()
@@ -354,4 +349,6 @@ class AtomPAW(GPAW):
             bf = BasisFunction(n, l, self.wfs.gd.r_g[-1], bf_g,
                                f'{n}{"spdfgh"[l]} e={eps:.3f} f={f:.3f}')
             bf_j.append(bf)
-        return basis
+
+        return Basis(self.symbol, basis_name, rgd=rgd,
+                     generatordata='AtomPAW', bf_j=bf_j)
