@@ -10,6 +10,7 @@ from gpaw.mpi import MPIComm, serial_comm
 from gpaw.new import prod, zips
 from gpaw.typing import Array1D, ArrayLike1D
 from gpaw.new.c import dH_aii_times_P_ani_gpu
+from gpaw.utilities import as_real_dtype
 
 
 class AtomArraysLayout(XP):
@@ -262,11 +263,11 @@ class AtomArrays:
         data:
             Array to use for storage.
         """
-        if xp is np:
+        if xp is not None:
             assert layout is None
             assert data is None
-            assert self.layout.xp is cp
-            layout = self.layout.new(xp=np)
+            if self.layout.xp is not xp:
+                layout = self.layout.new(xp=xp)
         return AtomArrays(layout or self.layout,
                           self.dims,
                           self.comm,
@@ -288,15 +289,26 @@ class AtomArrays:
         return self.new(layout=self.layout.new(xp=cp),
                         data=cp.asarray(self.data))
 
+    @overload
+    def __getitem__(self, a: int) -> np.ndarray:
+        ...
+
+    @overload
+    def __getitem__(self, a: tuple) -> AtomArrays:
+        ...
+
     def __getitem__(self, a):
         if isinstance(a, numbers.Integral):
             return self._arrays[a]
-        if len(self.dims) == 1:
-            a0, a1 = a
-            assert a0 == slice(None)
-            a_ai = AtomArrays(self.layout, data=self.data[a1].copy())
-            return a_ai
-        1 / 0
+        assert len(self.dims) >= 1
+        a0, a1 = a
+        assert a0 == slice(None)
+        data = self.data[a1]
+        a_ai = AtomArrays(self.layout, dims=data.shape[:-1], data=data)
+        return a_ai
+
+    def copy(self):
+        return self.new(data=self.data.copy())
 
     def get(self, a):
         return self._arrays.get(a)
@@ -373,6 +385,7 @@ class AtomArrays:
         comm = self.layout.atomdist.comm
         xp = self.layout.xp
         if comm.size == 1:
+            assert data is not None
             self.data[:] = data
             return
 
@@ -535,5 +548,6 @@ class AtomArrays:
         if index is not None:
             data = data[index]
         if self.data.size > 0:
-            dH_aii_times_P_ani_gpu(data, ni_a,
-                                   self.data, out_ani.data)
+            realdtype = as_real_dtype(self.data.dtype)
+            dH_aii_times_P_ani_gpu(xp.asarray(data, dtype=realdtype),
+                                   ni_a, self.data, out_ani.data)
