@@ -9,7 +9,7 @@ import scipy.linalg as sla
 
 import gpaw.utilities.blas as blas
 from gpaw import debug, get_scipy_version
-from gpaw.gpu import cupy as cp, cupy_eigh, XP
+from gpaw.gpu import cupy as cp, cupy_eigh, XP, gpu_gemm
 from gpaw.mpi import MPIComm, _Communicator, serial_comm
 from gpaw.typing import Array1D, ArrayLike1D, ArrayLike2D, Array2D
 
@@ -52,10 +52,32 @@ def suggest_blocking(N: int, ncpus: int) -> tuple[int, int, int]:
     return nprow, npcol, blocksize
 
 
+class MatrixWithNoData:
+    def __init__(self,
+                 M: int,
+                 N: int,
+                 *,
+                 dtype=None,
+                 dist: MatrixDistribution | tuple | None = None):
+        self.shape = (M, N)
+        self.dtype = dtype
+        self.data = np.empty((0, 0), dtype)
+        dist = dist or ()
+        if isinstance(dist, tuple):
+            kwargs = {key: val for key, val in zip(['comm', 'r', 'c', 'b'],
+                                                   dist)}
+            dist = create_distribution(M, N, **kwargs)
+        self.dist = dist
+
+    def create(self) -> Matrix:
+        return Matrix(*self.shape, dtype=self.dtype, dist=self.dist)
+
+
 class Matrix(XP):
     def __init__(self,
                  M: int,
                  N: int,
+                 *,
                  dtype=None,
                  data: ArrayLike2D | None = None,
                  dist: MatrixDistribution | tuple | None = None,
@@ -91,7 +113,8 @@ class Matrix(XP):
             else:
                 dtype = data.dtype
         self.dtype = np.dtype(dtype)
-        assert dtype == float or dtype == complex, dtype
+        assert np.dtype(self.dtype) in \
+            [np.float32, np.float64, np.complex64, np.complex128], dtype
 
         self.xp: ModuleType
         if xp is None:
@@ -160,6 +183,7 @@ class Matrix(XP):
                  opa='N',
                  opb='N',
                  out=None,
+                 data_buffer=None,
                  beta=0.0,
                  symmetric=False) -> Matrix:
         """BLAS matrix-multiplication with other matrix."""
@@ -172,9 +196,71 @@ class Matrix(XP):
             assert beta == 0.0
             M = A.shape[0] if opa == 'N' else A.shape[1]
             N = B.shape[1] if opb == 'N' else B.shape[0]
-            out = Matrix(M, N, A.dtype, dist=dist.new(M, N))
+            out = Matrix(M, N, dtype=A.dtype, dist=dist.new(M, N))
         elif not isinstance(out, Matrix):
             out = out.matrix
+        if out.data is other.data:
+            # Repeatably call multiply using data_buffer
+            assert opa == 'N', 'Not implemented'
+            assert opb == 'N', 'Not implemented'
+            assert not beta, 'Not implemented'
+            assert other.shape[0] == self.shape[0]
+
+            # Assert simple (only row distributed) distributions:
+            assert self.shape[1] == self.data.shape[1]
+            assert other.shape[1] == other.data.shape[1]
+            assert out.shape[1] == out.data.shape[1]
+
+            if data_buffer is None:
+                raise ValueError('other is out, and data_buffer is None')
+
+            assert isinstance(data_buffer, other.xp.ndarray)
+            dtype = other.data.dtype
+            data_buffer = data_buffer.view(dtype)
+            if other.data.shape[0] > 0:
+                # Obtain buffer size s.t. the maximum number of
+                # columns in other.data fits into data_buffer
+                buffer_size = min(
+                    data_buffer.size // other.data.shape[0],
+                    other.data.shape[1])
+            else:
+                # There is no data in other. Thus buffer_size
+                # fits all.
+                buffer_size = other.data.shape[1]
+            buffer_size = dist.comm.min_scalar(buffer_size)
+            max_B = other.data.shape[1]
+
+            if buffer_size >= max_B:
+                # No need for sliced multiply
+                other_buffer = other.new(
+                    data=data_buffer[:other.data.size].reshape(
+                        other.data.shape))
+                other_buffer.data[:] = other.data
+                dist.multiply(alpha, A, opa, other_buffer, opb, beta, out,
+                              symmetric=symmetric)
+                return out
+
+            # Sliced multiply
+            for i in range(0, max_B, buffer_size):
+                r_buffer_size = min(max(other.data.shape[1] - i, 0),
+                                    buffer_size)
+                l_buffer_size = r_buffer_size * other.data.shape[0]
+                buffer = Matrix(
+                    M=other.shape[0],
+                    N=r_buffer_size,
+                    data=data_buffer[
+                        :l_buffer_size].reshape(
+                        (other.data.shape[0], r_buffer_size)
+                    ),
+                    dist=dist.new(M=other.shape[0], N=r_buffer_size),
+                    xp=other.xp)
+                buffer.data[:] \
+                    = other.data[:, i:i + buffer_size]
+                out_view = buffer.new(
+                    data=out.data[:, i:i + buffer_size])
+                dist.multiply(alpha, A, opa, buffer,
+                              opb, beta, out_view, symmetric=False)
+            return out
 
         dist.multiply(alpha, A, opa, B, opb, beta, out, symmetric=symmetric)
         return out
@@ -362,7 +448,7 @@ class Matrix(XP):
 
         if rows * columns == 1:
             if self.dist.comm.rank == 0:
-                if cc and H.dtype == complex:
+                if cc and np.issubdtype(H.dtype, np.complexfloating):
                     np.negative(H.data.imag, H.data.imag)
                 if debug:
                     H.data[np.triu_indices(H.shape[0], 1)] = 42.0
@@ -489,7 +575,7 @@ class Matrix(XP):
 
     def complex_conjugate(self) -> None:
         """Inplace complex conjugation."""
-        if self.dtype == complex:
+        if np.issubdtype(self.dtype, np.complexfloating):
             self.xp.negative(self.data.imag, self.data.imag)
 
     def add_hermitian_conjugate(self, scale: float = 1.0) -> None:
@@ -541,10 +627,24 @@ class Matrix(XP):
         assert M == N
         self.data.ravel()[n1::N + 1] += d
 
-    def to_cpu(self):
-        if isinstance(self.data, np.ndarray):
+    def to_cpu(self) -> Matrix:
+        """Create new matrix object with values transfered from GPU to CPU."""
+        return self.to_xp(np)
+
+    def to_xp(self, xp) -> Matrix:
+        """Create new matrix object with data on GPU or CPU."""
+        if xp is self.xp:
+            assert xp is np, 'cp -> cp should not be needed!'
             return self
-        return Matrix(*self.shape, data=cp.asnumpy(self.data))
+        if xp is np:
+            return self.dist.matrix(data=cp.asnumpy(self.data))
+        return self.dist.matrix(data=cp.asarray(self.data))
+
+    def to_dtype(self, dtype) -> Matrix:
+        """Convert to new data type."""
+        if dtype == self.dtype:
+            return self
+        return self.dist.matrix(data=self.data.astype(dtype))
 
 
 def _matrix(M):
@@ -773,6 +873,7 @@ class BLACSDistribution(MatrixDistribution):
                              beta, c.data,
                              b.dist.desc, a.dist.desc, c.dist.desc,
                              opb, opa)
+        return c
 
 
 def cublas_mmm(alpha, a, opa, b, opb, beta, c):
@@ -780,8 +881,8 @@ def cublas_mmm(alpha, a, opa, b, opb, beta, c):
         return
     if a.size == 0 and beta == 1.0:
         return
-    cp.cublas.gemm(opa.replace('C', 'H'), opb.replace('C', 'H'),
-                   a, b, c, alpha, beta)
+    gpu_gemm(opa.replace('C', 'H'), opb.replace('C', 'H'),
+             a, b, c, alpha, beta)
 
 
 class CuPyDistribution(MatrixDistribution):
@@ -828,9 +929,9 @@ class CuPyDistribution(MatrixDistribution):
             if opa == 'N':
                 assert opb == 'C' or opb == 'T' and a.dtype == float
                 if a is b:
-                    cp.cublas.gemm('N', 'H',
-                                   a.data, a.data, c.data,
-                                   alpha, beta)
+                    gpu_gemm('N', 'H',
+                             a.data, a.data, c.data,
+                             alpha, beta)
                     # cp.cublas.syrk('N', a.data, c.data, alpha, beta, True)
                 else:
                     if beta == 1.0 and a.shape[1] == 0:
@@ -838,12 +939,12 @@ class CuPyDistribution(MatrixDistribution):
                     if c.data.size > 0:
                         assert beta in [0.0, 1.0]
                         # CuPy doesn't have dsyrk, so we roll our own:
-                        cp.cublas.gemm('N', 'H',
-                                       a.data, b.data, c.data,
-                                       0.5 * alpha, beta)
-                        cp.cublas.gemm('N', 'H',
-                                       b.data, a.data, c.data,
-                                       0.5 * alpha, 1.0)
+                        gpu_gemm('N', 'H',
+                                 a.data, b.data, c.data,
+                                 0.5 * alpha, beta)
+                        gpu_gemm('N', 'H',
+                                 b.data, a.data, c.data,
+                                 0.5 * alpha, 1.0)
             else:
                 1 / 0
                 assert opa == 'C' and opb == 'N'
@@ -885,11 +986,13 @@ def mmm_nn(m1, m2, m3, alpha, beta, mmm):
     buf1 = m2.data
     xp = m1.xp
 
-    N = m1.shape[0]
+    N = m1.shape[1]
+    assert N == m2.shape[0], f'{N}, {m2.shape[0]}'
     n = (N + comm.size - 1) // comm.size
 
     for r in range(comm.size):
         if r == 0:
+            # Buffers...
             buf2 = xp.empty((n, buf1.shape[1]), dtype=buf1.dtype)
 
         rrequest = None
@@ -907,11 +1010,13 @@ def mmm_nn(m1, m2, m3, alpha, beta, mmm):
         r0 = (comm.rank + r) % comm.size
         n1 = min(r0 * n, N)
         n2 = min(n1 + n, N)
+        # Contiguity...
         mmm(alpha, m1.data[:, n1:n2], 'N', buf1[:n2 - n1], 'N', beta, m3.data)
 
         beta = 1.0
 
         if r == 0:
+            # Buffers...
             buf1 = xp.empty_like(buf2)
 
         buf1, buf2 = buf2, buf1
@@ -932,14 +1037,18 @@ def mmm_nc_sym(a, b, out, alpha, mmm):
                 †
         c <- αab + c
 
+    This function utilizes the fact that c is symmetric, s.t.:
+                       †     †
+        c <- 0.5 * (αab + αba) + c
     Only lower half of c is updated.
     """
     comm = a.dist.comm
-    M, N = a.shape
+    M, N = b.shape
     m = (M + comm.size - 1) // comm.size
-    mym = len(a.data)
+    mym = len(b.data)
     xp = a.xp
 
+    # Buffers...
     buf1 = xp.empty((m, N), dtype=a.dtype)
     buf2 = xp.empty((m, N), dtype=a.dtype)
     half = comm.size // 2
@@ -966,6 +1075,7 @@ def mmm_nc_sym(a, b, out, alpha, mmm):
             m2 = min(m1 + m, M)
             if r == 0:
                 # symmmmmmmmmmmmmmmmmmmmmmetricccccccccccccccc
+                # Contiguity...
                 mmm(alpha, aa, 'N', bb, 'C', 1.0, out.data[:, m1:m2])
             else:
                 beta = 1.0 if r <= comm.rank else 0.0
@@ -1009,7 +1119,7 @@ def mmm_nc_sym(a, b, out, alpha, mmm):
 
 
 def mmm_nc(a, b, out, alpha, beta, mmm):
-    """Symmetric parallel matrix-matrix multiplication.
+    """Parallel matrix-matrix multiplication.
 
     :::
 
@@ -1017,11 +1127,12 @@ def mmm_nc(a, b, out, alpha, beta, mmm):
         c <- αab  + βc
     """
     comm = a.dist.comm
-    M, N = a.shape
+    M, N = b.shape
     m = (M + comm.size - 1) // comm.size
-    mym = len(a.data)
+    mym = len(b.data)
     xp = a.xp
 
+    # Nasty buffers
     buf1 = xp.empty((m, N), dtype=a.dtype)
     buf2 = xp.empty((m, N), dtype=a.dtype)
     aa = a.data
