@@ -22,7 +22,7 @@ if TYPE_CHECKING:
     import plotly.graph_objects as go
 
 
-class UGDesc(Domain):
+class UGDesc(Domain['UGArray']):
     def __init__(self,
                  *,
                  cell: ArrayLike1D | ArrayLike2D,  # bohr
@@ -158,7 +158,7 @@ class UGDesc(Domain):
         """
         return UGArray(self, dims, comm, xp=xp)
 
-    def from_data(self, data):
+    def from_data(self, data: np.ndarray) -> UGArray:
         return UGArray(self, data.shape[:-3], data=data)
 
     def blocks(self, data: np.ndarray):
@@ -256,7 +256,7 @@ class UGDesc(Domain):
                                    comm: MPIComm = serial_comm,
                                    dtype=None) -> UGDesc:
         """Create UGDesc from grid-spacing."""
-        domain = Domain(cell, pbc, kpt, comm, dtype)
+        domain: Domain = Domain(cell, pbc, kpt, comm, dtype)
         return domain.uniform_grid_with_grid_spacing(grid_spacing)
 
     def fft_plans(self,
@@ -321,17 +321,27 @@ class UGArray(DistributedArrays[UGDesc]):
             txt += ', xp=cp'
         return txt + ')'
 
-    def new(self, data=None, zeroed=False):
+    def new(self, data=None, zeroed=False, dims=None):
         """Create new UniforGridFunctions object of same kind.
 
         Parameters
         ----------
         data:
             Array to use for storage.
+        zeroed:
+            If True, set data to zero.
+        dims:
+            Extra dimensions (bands, spin, etc.), required if
+            data does not fit the full array.
         """
+        if dims:
+            assert data is not None
+        else:
+            dims = self.dims
         if data is None:
             data = self.xp.empty_like(self.data)
-        f_xR = UGArray(self.desc, self.dims, self.comm, data)
+
+        f_xR = UGArray(self.desc, dims, self.comm, data)
         if zeroed:
             f_xR.data[:] = 0.0
         return f_xR
@@ -388,10 +398,11 @@ class UGArray(DistributedArrays[UGDesc]):
         c.data[:] = self.data
         return c
 
-    def scatter_from(self, data=None):
+    def scatter_from(self, data: np.ndarray | UGArray | None = None) -> None:
         """Scatter data from rank-0 to all ranks."""
         if isinstance(data, UGArray):
             data = data.data
+
         comm = self.desc.comm
         if comm.size == 1:
             self.data[:] = data
@@ -402,6 +413,7 @@ class UGArray(DistributedArrays[UGDesc]):
             return
 
         requests = []
+        assert isinstance(data, self.xp.ndarray)
         for rank, block in enumerate(self.desc.blocks(data)):
             if rank != 0:
                 block = block.copy()
@@ -460,9 +472,9 @@ class UGArray(DistributedArrays[UGDesc]):
                           _ _
            _    1  / _  -iG.r   _
          C(G) = -- |dr e      f(r),
-                V  /
+                Ω  /
 
-        where `C(\bG)` are the plane wave coefficients and V is the cell
+        where `C(\bG)` are the plane wave coefficients and Ω is the cell
         volume.
 
         Parameters
@@ -474,10 +486,10 @@ class UGArray(DistributedArrays[UGDesc]):
         out:
             Target PWArray object.
         """
-        assert self.dims == ()
+        assert not self.desc.zerobc_c.any()
         if out is None:
             assert pw is not None
-            out = pw.empty(xp=self.xp)
+            out = pw.empty(dims=self.dims, xp=self.xp)
         if pw is None:
             pw = out.desc
         if pw.dtype != self.desc.dtype:
@@ -488,11 +500,12 @@ class UGArray(DistributedArrays[UGDesc]):
             input = input.gather()
         if self.desc.comm.rank == 0:
             plan = plan or self.desc.fft_plans(xp=self.xp)
-            coefs = plan.fft_sphere(input.data, pw)
+            for i, o in zip(input.flat(), out.flat()):
+                coefs = plan.fft_sphere(i.data, pw)
+                o.scatter_from(coefs)
         else:
-            coefs = None
-
-        out.scatter_from(coefs)
+            for o in out.flat():
+                o.scatter_from(None)
 
         return out
 
