@@ -128,7 +128,6 @@ class _Communicator:
         else:
             # assert a.ndim != 0
             tc = a.dtype
-            assert tc == int or tc == float or tc == complex
             assert is_contiguous(a, tc)
             assert root == -1 or 0 <= root < self.size
             self.comm.sum(a, root)
@@ -196,7 +195,7 @@ class _Communicator:
             self.comm.max(a, root)
 
     def max_scalar(self, a, root=-1):
-        assert isinstance(a, (int, float))
+        assert isinstance(a, (int, float, np.int64))
         return self.comm.max_scalar(a, root)
 
     def min(self, a, root=-1):
