@@ -8,7 +8,7 @@ if TYPE_CHECKING:
     from gpaw.core import UGArray
 
 
-from gpaw.new.timer import trace  # noqa
+from gpaw.new.timer import trace, tracectx  # noqa
 
 
 def prod(iterable: Iterable[int]) -> int:
