@@ -74,8 +74,9 @@ class FakeWFS:
         # self.setups.set_symmetry(ibzwfs.ibz.symmetries.symmetry)
         self.occ_calc = occ_calc
         self.occupations = occ_calc.occ
-        self.nvalence = int(round(ibzwfs.nelectrons))
-        assert self.nvalence == ibzwfs.nelectrons
+        self.nvalence = int(round(density.nvalence))
+        self.nvalence = density.nvalence
+        # assert self.nvalence == density.nvalence
         self.world = comm
         if ibzwfs.fermi_levels is not None:
             self.fermi_levels = ibzwfs.fermi_levels
@@ -147,7 +148,7 @@ class FakeWFS:
         from gpaw.matrix import Matrix
         return Matrix(
             self.bd.nbands, self.bd.nbands,
-            self.dtype,
+            dtype=self.dtype,
             dist=(self.bd.comm, self.bd.comm.size))
 
     @property
@@ -239,21 +240,25 @@ class KPT:
         self.pd = pd
         self.gd = gd
 
-        I1 = 0
-        nproj_a = []
-        for a, shape in enumerate(wfs.P_ani.layout.shape_a):
-            I2 = I1 + prod(shape)
-            nproj_a.append(I2 - I1)
-            I1 = I2
+        try:
+            I1 = 0
+            nproj_a = []
+            for a, shape in enumerate(wfs.P_ani.layout.shape_a):
+                I2 = I1 + prod(shape)
+                nproj_a.append(I2 - I1)
+                I1 = I2
+        except RuntimeError:
+            pass
+        else:
+            self.projections = Projections(
+                wfs.nbands,
+                nproj_a,
+                atom_partition,
+                wfs.P_ani.comm,
+                wfs.ncomponents < 4,
+                wfs.spin,
+                data=wfs.P_ani.data)
 
-        self.projections = Projections(
-            wfs.nbands,
-            nproj_a,
-            atom_partition,
-            wfs.P_ani.comm,
-            wfs.ncomponents < 4,
-            wfs.spin,
-            data=wfs.P_ani.data)
         self.s = wfs.spin if wfs.ncomponents < 4 else None
         self.k = wfs.k
         self.q = wfs.q
@@ -288,6 +293,8 @@ class KPT:
 
     @property
     def psit_nG(self):
+        if not hasattr(self, 'psit_nX'):
+            return None
         data = self.psit_nX.data
         if self.scale == 1:
             return data
@@ -357,7 +364,8 @@ class FakeDensity:
 
 class FakeHamiltonian:
     def __init__(self, ibzwfs, density, potential, pot_calc,
-                 e_total_free=np.nan):
+                 e_total_free=np.nan,
+                 e_xc=np.nan):
         self.pot_calc = pot_calc
         self.ibzwfs = ibzwfs
         self.density = density
@@ -368,12 +376,11 @@ class FakeHamiltonian:
             pass
         self.grid = potential.vt_sR.desc
         self.e_total_free = e_total_free
-        self.e_xc = potential.energies['xc']
+        self.e_xc = e_xc
 
     def update(self, dens, wfs, kin_en_using_band=True):
-        potential, _ = self.pot_calc.calculate(
+        self.potential, _ = self.pot_calc.calculate(
             self.density, self.ibzwfs, self.potential.vHt_x)
-        self.potential.update_from(potential)
 
         energies = self.potential.energies
         self.e_xc = energies['xc']
