@@ -2,12 +2,11 @@ from __future__ import annotations
 
 import warnings
 from functools import cached_property
-from typing import Any, Union
+from typing import Any, TYPE_CHECKING
 
 import numpy as np
 from ase import Atoms
 from ase.units import Bohr, Ha
-
 from gpaw.core import UGArray, UGDesc
 from gpaw.core.atom_arrays import AtomDistribution
 from gpaw.densities import Densities
@@ -17,8 +16,8 @@ from gpaw.mpi import broadcast as bcast
 from gpaw.mpi import broadcast_float, world
 from gpaw.new import trace, zips
 from gpaw.new.density import Density
+from gpaw.new.energies import DFTEnergies
 from gpaw.new.ibzwfs import IBZWaveFunctions
-from gpaw.new.input_parameters import InputParameters
 from gpaw.new.logger import Logger
 from gpaw.new.potential import Potential
 from gpaw.new.scf import SCFLoop
@@ -27,6 +26,8 @@ from gpaw.typing import Array1D, Array2D
 from gpaw.utilities import (check_atoms_too_close,
                             check_atoms_too_close_to_boundary)
 from gpaw.utilities.partition import AtomPartition
+if TYPE_CHECKING:
+    from gpaw.dft import Parameters
 
 
 class ReuseWaveFunctionsError(Exception):
@@ -59,26 +60,19 @@ units = {'energy': Ha,
          'non_collinear_magmoms': 1.0}
 
 
-class DFTState:
-    def __init__(self,
-                 ibzwfs: IBZWaveFunctions,
-                 density: Density,
-                 potential: Potential):
-        """State of a Kohn-Sham calculation."""
-        self.ibzwfs = ibzwfs
-        self.density = density
-        self.potential = potential
-
-
 class DFTCalculation:
     def __init__(self,
+                 atoms: Atoms,
                  ibzwfs: IBZWaveFunctions,
                  density: Density,
                  potential: Potential,
                  setups: Setups,
                  scf_loop: SCFLoop,
                  pot_calc,
-                 log: Logger):
+                 log: Logger,
+                 params: Parameters,
+                 energies: DFTEnergies | None = None):
+        self.atoms = atoms
         self.ibzwfs = ibzwfs
         self.density = density
         self.potential = potential
@@ -87,60 +81,59 @@ class DFTCalculation:
         self.pot_calc = pot_calc
         self.log = log
         self.comm = log.comm
+        self.params = params
 
         self.results: dict[str, Any] = {}
         self.relpos_ac = self.pot_calc.relpos_ac
+        self.energies = energies or DFTEnergies()
+        self.forces_have_been_printed = False
 
-    def get_state(self):
-        return DFTState(self.ibzwfs, self.density, self.potential)
-
-    @property
-    def state(self):
-        warnings.warn('Use of deprecated DFTCalculation.state attribute. '
-                      'Use ibzwfs, density and potential attributes instead.')
-        return self.get_state()
+    def __getattr__(self, name):
+        matches = [ext
+                   for ext in self.pot_calc.extensions
+                   if ext.name == name]
+        if len(matches) != 1:
+            raise AttributeError
+        return matches[0]
 
     @classmethod
     def from_parameters(cls,
                         atoms: Atoms,
-                        params: Union[dict, InputParameters],
+                        params: Parameters,
                         comm=None,
-                        log=None,
-                        builder=None) -> DFTCalculation:
+                        log=None) -> DFTCalculation:
         """Create DFTCalculation object from parameters and atoms."""
-        from gpaw.new.builder import builder as create_builder
-
         check_atoms_too_close(atoms)
         check_atoms_too_close_to_boundary(atoms)
 
-        if params is None:
-            params = {}
-        if isinstance(params, dict):
-            params = InputParameters(params)
-
         if not isinstance(log, Logger):
             log = Logger(log, comm or world)
 
-        builder = builder or create_builder(atoms, params, log.comm)
+        builder = params.dft_component_builder(atoms, log=log)
 
         basis_set = builder.create_basis_set()
 
-        density = builder.density_from_superposition(basis_set)
-        density.normalize()
-
         # The SCF-loop has a Hamiltonian that has an fft-plan that is
         # cached for later use, so best to create the SCF-loop first
         # FIX this!
         scf_loop = builder.create_scf_loop()
 
         pot_calc = builder.create_potential_calculator()
-        potential, _ = pot_calc.calculate_without_orbitals(
+
+        density = builder.density_from_superposition(basis_set)
+        if len(atoms) == 0:
+            density.nt_sR.data[:] = 1.0
+        density.normalize(pot_calc.environment.charge)
+
+        potential, energies, _ = pot_calc.calculate_without_orbitals(
             density, kpt_band_comm=builder.communicators['D'])
         ibzwfs = builder.create_ibz_wave_functions(
-            basis_set, potential, log=log)
+            basis_set, potential)
 
         if ibzwfs.wfs_qs[0][0]._eig_n is not None:
-            ibzwfs.calculate_occs(scf_loop.occ_calc)
+            nelectrons = (density.nvalence - density.charge +
+                          pot_calc.environment.charge)
+            ibzwfs.calculate_occs(scf_loop.occ_calc, nelectrons)
 
         write_atoms(atoms, builder.initial_magmom_av, builder.grid, log)
         log(ibzwfs)
@@ -150,12 +143,23 @@ class DFTCalculation:
         log(scf_loop)
         log(pot_calc)
 
-        return cls(ibzwfs, density, potential,
-                   builder.setups, scf_loop, pot_calc, log)
+        return cls(atoms, ibzwfs, density, potential,
+                   builder.setups, scf_loop, pot_calc, log,
+                   params=params, energies=energies)
+
+    def ase_calculator(self):
+        """Create ASE-compatible GPAW calculator.
+        """
+        from gpaw.new.ase_interface import ASECalculator
+        return ASECalculator(params=self.params,
+                             log=self.log,
+                             dft=self,
+                             atoms=self.atoms)
 
     def move_atoms(self, atoms) -> DFTCalculation:
         check_atoms_too_close(atoms)
 
+        self.atoms = atoms
         self.relpos_ac = np.ascontiguousarray(atoms.get_scaled_positions())
         self.comm.broadcast(self.relpos_ac, 0)
 
@@ -171,26 +175,30 @@ class DFTCalculation:
             self.density.update(self.ibzwfs)
         self.potential.move(atomdist)
 
-        new_potential, _ = self.pot_calc.calculate(
+        self.potential, self.energies, _ = self.pot_calc.calculate(
             self.density, self.ibzwfs, self.potential.vHt_x)
-        self.potential.update_from(new_potential)
 
         mm_av = self.results['non_collinear_magmoms']
         write_atoms(atoms, mm_av, self.density.nt_sR.desc, self.log)
 
         self.results = {}
+        self.forces_have_been_printed = False
 
         return self
 
     def iconverge(self, maxiter=None, calculate_forces=None):
         self.ibzwfs.make_sure_wfs_are_read_from_gpw_file()
-        yield from self.scf_loop.iterate(self.ibzwfs,
+        for ctx in self.scf_loop.iterate(self.ibzwfs,
                                          self.density,
                                          self.potential,
+                                         self.energies,
                                          self.pot_calc,
                                          maxiter=maxiter,
                                          calculate_forces=calculate_forces,
-                                         log=self.log)
+                                         log=self.log):
+            self.energies = ctx.energies
+            self.potential = ctx.potential
+            yield ctx
 
     @trace
     def converge(self,
@@ -206,26 +214,15 @@ class DFTCalculation:
         else:  # no break
             self.log('SCF steps:', step)
 
-    def energies(self):
-        energies = combine_energies(self.potential, self.ibzwfs)
+    def energy(self):
+        self.results['free_energy'] = broadcast_float(
+            self.energies.total_free, self.comm)
+        self.results['energy'] = broadcast_float(
+            self.energies.total_extrapolated, self.comm)
 
         self.log('Energy contributions relative to reference atoms:',
                  f'(reference = {self.setups.Eref * Ha:.6f})\n')
-
-        for name, e in energies.items():
-            if not name.startswith('total') and name != 'stress':
-                self.log(f'{name + ":":10}   {e * Ha:14.6f}')
-        total_free = energies['total_free']
-        total_extrapolated = energies['total_extrapolated']
-        self.log('----------------------------')
-        self.log(f'Free energy: {total_free * Ha:14.6f}')
-        self.log(f'Extrapolated:{total_extrapolated * Ha:14.6f}\n')
-
-        total_free = broadcast_float(total_free, self.comm)
-        total_extrapolated = broadcast_float(total_extrapolated, self.comm)
-
-        self.results['free_energy'] = total_free
-        self.results['energy'] = total_extrapolated
+        self.energies.summary(self.log)
 
     def dipole(self):
         if 'dipole' in self.results:
@@ -254,13 +251,15 @@ class DFTCalculation:
             self.log()
         return mm_v, mm_av
 
-    def forces(self, silent=False):
+    def forces(self):
         """Calculate atomic forces."""
-        if 'forces' not in self.results or silent:
+        if 'forces' not in self.results:
             self._calculate_forces()
 
-        if silent:
+        if self.forces_have_been_printed:
             return
+
+        self.forces_have_been_printed = True
         self.log('\nForces: [  # eV/Ang')
         F_av = self.results['forces'] * (Ha / Bohr)
         for a, setup in enumerate(self.setups):
@@ -278,8 +277,10 @@ class DFTCalculation:
 
         pot_calc = self.pot_calc
         Q_aL = self.density.calculate_compensation_charge_coefficients()
-        Fcc_av, Fnct_av, Ftauct_av, Fvbar_av = pot_calc.force_contributions(
-            Q_aL, self.density, self.potential)
+        Fcc_av, Fnct_av, Ftauct_av, Fvbar_av, Fext_av = \
+            pot_calc.force_contributions(Q_aL,
+                                         self.density,
+                                         self.potential)
 
         # Force from compensation charges:
         F_av += Fcc_av
@@ -302,6 +303,9 @@ class DFTCalculation:
         domain_comm = Q_aL.layout.atomdist.comm
         domain_comm.sum(F_av)
 
+        # Force from extensions (only from rank 0)
+        F_av += Fext_av
+
         F_av = self.ibzwfs.ibz.symmetries.symmetrize_forces(F_av)
         self.comm.broadcast(F_av, 0)
         self.results['forces'] = F_av
@@ -322,20 +326,19 @@ class DFTCalculation:
         if not np.isnan(vacuum_level):
             self.log(f'vacuum-level: {vacuum_level:.3f}  # V')
             try:
-                wf1, wf2 = self.workfunctions(vacuum_level=vacuum_level)
+                wf1, wf2 = self.workfunctions(_vacuum_level=vacuum_level)
             except NonsenseError:
                 pass
             else:
-                self.log(f'Workfunctions: {wf1:.3f}, {wf2:.3f}  # eV')
+                self.log(
+                    f'Workfunctions: {wf1:.3f}, {wf2:.3f} eV (top, bottom)')
         self.log.fd.flush()
 
     def workfunctions(self,
-                      *,
-                      vacuum_level: float | None = None
-                      ) -> tuple[float, float]:
-        if vacuum_level is None:
-            vacuum_level = self.potential.get_vacuum_level()
-        if np.isnan(vacuum_level):
+                      *, _vacuum_level=None) -> tuple[float, float]:
+        if _vacuum_level is None:
+            _vacuum_level = self.potential.get_vacuum_level()
+        if np.isnan(_vacuum_level):
             raise NonsenseError('No vacuum')
         try:
             correction = self.pot_calc.poisson_solver.dipole_layer_correction()
@@ -343,9 +346,12 @@ class DFTCalculation:
             raise NonsenseError('No dipole layer')
         correction *= Ha
         fermi_level = self.ibzwfs.fermi_level * Ha
-        wf = vacuum_level - fermi_level
+        wf = _vacuum_level - fermi_level
         return wf - correction, wf + correction
 
+    def vacuum_level(self) -> float:
+        return self.potential.get_vacuum_level()
+
     def electrostatic_potential(self) -> ElectrostaticPotential:
         return ElectrostaticPotential.from_calculation(self)
 
@@ -402,25 +408,21 @@ class DFTCalculation:
 
     def new(self,
             atoms: Atoms,
-            params: InputParameters,
+            params: Parameters,
             log=None) -> DFTCalculation:
         """Create new DFTCalculation object."""
-        from gpaw.new.builder import builder as create_builder
 
-        if params.mode['name'] != 'pw':
+        if params.mode.name != 'pw':
             raise ReuseWaveFunctionsError
 
         ibzwfs = self.ibzwfs
-        if ibzwfs.domain_comm.size != 1:
-            raise ReuseWaveFunctionsError
-
-        if not self.density.nt_sR.desc.pbc_c.all():
-            raise ReuseWaveFunctionsError
+        # if ibzwfs.domain_comm.size != 1:
+        #     raise ReuseWaveFunctionsError
 
         check_atoms_too_close(atoms)
         check_atoms_too_close_to_boundary(atoms)
 
-        builder = create_builder(atoms, params, self.comm)
+        builder = params.dft_component_builder(atoms, log=log)
 
         kpt_kc = builder.ibz.kpt_kc
         old_kpt_kc = ibzwfs.ibz.kpt_kc
@@ -429,24 +431,25 @@ class DFTCalculation:
         if abs(kpt_kc - old_kpt_kc).max() > 1e-9:
             raise ReuseWaveFunctionsError
 
-        log('# Interpolating wave functions to new cell')
+        log('Interpolating wave functions to new cell')
+
+        scf_loop = builder.create_scf_loop()
+        pot_calc = builder.create_potential_calculator()
 
         density = self.density.new(builder.grid,
                                    builder.interpolation_desc,
                                    builder.relpos_ac,
                                    builder.atomdist)
-        density.normalize()
+        density.normalize(pot_calc.environment.charge)
 
         # Make sure all have exactly the same density.
         # Not quite sure it is needed???
         # At the moment we skip it on GPU's because it doesn't
         # work!
         if density.nt_sR.xp is np:
-            self.comm.broadcast(density.nt_sR.data, 0)
+            ibzwfs.kpt_band_comm.broadcast(density.nt_sR.data, 0)
 
-        scf_loop = builder.create_scf_loop()
-        pot_calc = builder.create_potential_calculator()
-        potential, _ = pot_calc.calculate(density)
+        potential, energies, _ = pot_calc.calculate(density)
 
         old_ibzwfs = ibzwfs
 
@@ -459,7 +462,6 @@ class DFTCalculation:
 
         ibzwfs = ibzwfs.create(
             ibz=builder.ibz,
-            nelectrons=old_ibzwfs.nelectrons,
             ncomponents=old_ibzwfs.ncomponents,
             create_wfs_func=create_wfs,
             kpt_comm=old_ibzwfs.kpt_comm,
@@ -475,25 +477,18 @@ class DFTCalculation:
         log(pot_calc)
 
         return DFTCalculation(
-            ibzwfs, density, potential,
-            builder.setups, scf_loop, pot_calc, log)
+            atoms, ibzwfs, density, potential,
+            builder.setups, scf_loop, pot_calc, log,
+            params=params, energies=energies)
 
+    def get_state(self):
+        return DFTState(self.ibzwfs, self.density, self.potential)
 
-def combine_energies(potential: Potential,
-                     ibzwfs: IBZWaveFunctions) -> dict[str, float]:
-    """Add up energy contributions."""
-    energies = potential.energies.copy()
-    energies.pop('stress', 0.0)
-    energies['kinetic'] += ibzwfs.energies['band']
-    energies['kinetic'] += ibzwfs.energies.get('exx_kinetic', 0.0)
-    energies['xc'] += (ibzwfs.energies.get('exx_vv', 0.0) +
-                       ibzwfs.energies.get('exx_vc', 0.0) +
-                       ibzwfs.energies.get('exx_cc', 0.0))
-    energies['entropy'] = ibzwfs.energies['entropy']
-    energies['total_free'] = sum(energies.values())
-    energies['total_extrapolated'] = (energies['total_free'] +
-                                      ibzwfs.energies['extrapolation'])
-    return energies
+    @property
+    def state(self):
+        warnings.warn('Use of deprecated DFTCalculation.state attribute. '
+                      'Use ibzwfs, density and potential attributes instead.')
+        return self.get_state()
 
 
 def write_atoms(atoms: Atoms,
@@ -503,3 +498,16 @@ def write_atoms(atoms: Atoms,
     from gpaw.output import print_cell, print_positions
     print_positions(atoms, log, magmom_av)
     print_cell(grid._gd, grid.pbc, log)
+
+
+class DFTState:
+    def __init__(self,
+                 ibzwfs: IBZWaveFunctions,
+                 density: Density,
+                 potential: Potential,
+                 energies):
+        """State of a Kohn-Sham calculation."""
+        self.ibzwfs = ibzwfs
+        self.density = density
+        self.potential = potential
+        self.energies = energies
