@@ -16,11 +16,14 @@ Versions:
 5) Bug-fix: wave_functions.kpts.rotations are now U_scc
    as in version 3 (instead of U_svv).
 
+6) Write energy contributions to "energy_contributions".
+
 """
 from __future__ import annotations
 
+from dataclasses import dataclass
 from pathlib import Path
-from typing import IO, Any, Union
+from typing import IO, Any, Union, Callable
 
 import ase.io.ulm as ulm
 import gpaw
@@ -31,18 +34,13 @@ from ase.io.trajectory import read_atoms, write_atoms
 from ase.units import Bohr, Ha
 from gpaw.core.atom_arrays import AtomArraysLayout
 from gpaw.new.builder import DFTComponentsBuilder
-from gpaw.new.builder import builder as create_builder
 from gpaw.new.calculation import DFTCalculation, units
 from gpaw.new.density import Density
-from gpaw.new.input_parameters import InputParameters
 from gpaw.new.logger import Logger
 from gpaw.new.potential import Potential
-from gpaw.typing import DTypeLike
 from gpaw.utilities import unpack_hermitian, unpack_density
-
-ENERGY_NAMES = ['kinetic', 'coulomb', 'zero', 'external', 'xc', 'entropy',
-                'total_free', 'total_extrapolated',
-                'band', 'stress', 'spinorbit']
+from gpaw.new.energies import DFTEnergies
+from gpaw.dft import Parameters
 
 
 def as_single_precision(array):
@@ -72,17 +70,40 @@ def as_double_precision(array):
     return np.array(array, dtype=dtype)
 
 
+@dataclass
+class GPWFlags:
+    include_wfs: bool
+    include_projections: bool
+    precision: str
+
+    def __post_init__(self) -> None:
+        if self.precision not in ['single', 'double']:
+            raise ValueError('precision must be either "single" or "double"')
+
+    def storage_dtype(self, dtype):
+        dtype = np.dtype(dtype)
+        if self.precision == 'double':
+            return dtype
+
+        if dtype == float:
+            return np.dtype(np.float32)
+
+        if dtype == complex:
+            return np.dtype(np.complex64)
+
+        raise ValueError(f'Unexpected dtype: {dtype}')
+
+    def to_storage_dtype(self, array: np.ndarray) -> np.ndarray:
+        if self.precision == 'double':
+            return array
+        return array.astype(self.storage_dtype(array.dtype))
+
+
 def write_gpw(filename: str | Path,
-              atoms,
-              params,
               dft: DFTCalculation,
-              skip_wfs: bool = True,
-              precision: str = 'double',
-              include_projections=True) -> None:
+              flags: GPWFlags) -> None:
 
     comm = dft.comm
-    if precision not in ['single', 'double']:
-        raise ValueError('precision must be either "single" or "double"')
 
     writer: ulm.Writer | ulm.DummyWriter
     if comm.rank == 0:
@@ -91,33 +112,33 @@ def write_gpw(filename: str | Path,
         writer = ulm.DummyWriter()
 
     with writer:
-        writer.write(version=5,
+        writer.write(version=6,
                      gpaw_version=gpaw.__version__,
                      ha=Ha,
                      bohr=Bohr,
-                     precision=precision)
+                     precision=flags.precision)
 
-        write_atoms(writer.child('atoms'), atoms)
+        write_atoms(writer.child('atoms'), dft.atoms)
 
         results = {key: value * units[key]
                    for key, value in dft.results.items()}
         writer.child('results').write(**results)
 
-        p = {k: v for k, v in params.items() if k not in ['parallel']}
+        p = dft.params.todict()
+        p.pop('parallel', None)
         # ULM does not know about numpy dtypes:
         if 'dtype' in p:
             p['dtype'] = np.dtype(p['dtype']).name
         writer.child('parameters').write(**p)
 
-        dft.density.write(writer.child('density'), precision=precision)
-        dft.potential._write_gpw(writer.child('hamiltonian'),
-                                 dft.ibzwfs, precision=precision)
+        dft.density.write_to_gpw(writer.child('density'), flags)
+        dft.potential.write_to_gpw(writer.child('hamiltonian'), flags)
+        writer.write(e_stress=dft.potential.e_stress * Ha)
+        dft.energies.write_to_gpw(writer.child('energy_contributions'))
         wf_writer = writer.child('wave_functions')
-        dft.ibzwfs.write(wf_writer, skip_wfs,
-                         include_projections=include_projections,
-                         precision=precision)
+        dft.ibzwfs.write(wf_writer, flags=flags)
 
-        if not skip_wfs and params.mode['name'] == 'pw':
+        if flags.include_wfs and dft.params.mode.name == 'pw':
             write_wave_function_indices(wf_writer,
                                         dft.ibzwfs,
                                         dft.density.nt_sR.desc)
@@ -162,10 +183,12 @@ def read_gpw(filename: Union[str, Path, IO[str]],
              log: Union[Logger, str, Path, IO[str]] = None,
              comm=None,
              parallel: dict[str, Any] = None,
-             dtype: DTypeLike = None) -> tuple[Atoms,
-                                               DFTCalculation,
-                                               InputParameters,
-                                               DFTComponentsBuilder]:
+             dtype=None,
+             object_hooks: dict[str, Callable[[dict], Any]] | None = None
+             ) -> tuple[Atoms,
+                        DFTCalculation,
+                        Parameters,
+                        DFTComponentsBuilder]:
     """
     Read gpw file
 
@@ -194,13 +217,21 @@ def read_gpw(filename: Union[str, Path, IO[str]],
     if 'dtype' in kwargs:
         kwargs['dtype'] = np.dtype(kwargs['dtype'])
 
+    if dtype is not None:
+        kwargs['dtype'] = dtype
+
     # kwargs['nbands'] = reader.wave_functions.eigenvalues.shape[-1]
 
     for old_keyword in ['fixdensity', 'txt']:
         kwargs.pop(old_keyword, None)
 
-    params = InputParameters(kwargs, warn=False)
-    builder = create_builder(atoms, params, comm)
+    if object_hooks:
+        for key, hook in object_hooks.items():
+            if key in kwargs:
+                kwargs[key] = hook(kwargs[key])
+
+    params = Parameters(**kwargs)
+    builder = params.dft_component_builder(atoms, log=log)
 
     if comm.rank == 0:
         nt_sR_array = reader.density.density * bohr**3
@@ -230,8 +261,8 @@ def read_gpw(filename: Union[str, Path, IO[str]],
         # old gpw-file:
         kwargs.pop('h', None)
         kwargs['gpts'] = nt_sR_array.shape[1:]
-        params = InputParameters(kwargs, warn=False)
-        builder = create_builder(atoms, params, comm)
+        params = Parameters(**kwargs)
+        builder = params.dft_component_builder(atoms, log=log)
 
     kpts = reader.wave_functions.kpts
     rotation_scc = kpts.rotations
@@ -246,11 +277,8 @@ def read_gpw(filename: Union[str, Path, IO[str]],
         kwargs['symmetry'] = {'rotations': rotation_scc,
                               'translations': kpts.translations,
                               'atommaps': kpts.atommap}
-        params = InputParameters(kwargs, warn=False)
-        builder = create_builder(atoms, params, comm)
-
-    if dtype is not None:
-        params.mode['dtype'] = dtype
+        params = Parameters(**kwargs)
+        builder = params.dft_component_builder(atoms, log=log)
 
     (kpt_comm, band_comm, domain_comm, kpt_band_comm) = (
         builder.communicators[x] for x in 'kbdD')
@@ -292,7 +320,8 @@ def read_gpw(filename: Union[str, Path, IO[str]],
     kpt_band_comm.broadcast(D_asp.data, 0)
     kpt_band_comm.broadcast(dH_asp.data, 0)
 
-    if reader.version >= 4:
+    # if reader.version >= 4:
+    if 'electrostatic_potential' in reader.hamiltonian:
         if comm.rank == 0:
             vHt_x_array = reader.hamiltonian.electrostatic_potential / ha
             if singlep:
@@ -312,29 +341,36 @@ def read_gpw(filename: Union[str, Path, IO[str]],
         builder.setups,
         builder.get_pseudo_core_densities(),
         builder.get_pseudo_core_ked())
-    energies = {name: reader.hamiltonian.get(f'e_{name}', np.nan) / ha
-                for name in ENERGY_NAMES}
-    penergies = {key: e for key, e in energies.items()
-                 if not key.startswith('total')}
-    e_band = penergies.pop('band', np.nan)
-    e_entropy = penergies.pop('entropy')
-    penergies['kinetic'] -= e_band
 
-    potential = Potential(vt_sR, dH_asp.to_full(), dedtaut_sR, penergies,
-                          vHt_x)
+    if reader.version >= 6:
+        ec = {name: e / ha
+              for name, e in reader.energy_contributions.asdict().items()}
+        e_stress = reader.e_stress / ha
+    else:
+        NAMES = ['kinetic', 'coulomb', 'zero', 'external',
+                 'xc', 'entropy',
+                 'total_free', 'total_extrapolated',
+                 'band', 'stress', 'spinorbit']
+        ec = {name: reader.hamiltonian.get(f'e_{name}', np.nan) / ha
+              for name in NAMES}
+        ec['kinetic_correction'] = ec['kinetic'] - ec['band']
+        ec['extrapolation'] = (ec.pop('total_extrapolated') -
+                               ec.pop('total_free'))
+        e_stress = ec.pop('stress', np.nan) / ha
+
+    energies = DFTEnergies(**ec)
+
+    potential = Potential(vt_sR, dH_asp.to_full(), dedtaut_sR, vHt_x, e_stress)
 
     ibzwfs = builder.read_ibz_wave_functions(reader)
-    ibzwfs.energies = {
-        'band': e_band,
-        'entropy': e_entropy,
-        'extrapolation': (energies['total_extrapolated'] -
-                          energies['total_free'])}
 
     dft = DFTCalculation(
-        ibzwfs, density, potential,
+        atoms, ibzwfs, density, potential,
         builder.setups,
         builder.create_scf_loop(),
         pot_calc=builder.create_potential_calculator(),
+        params=params,
+        energies=energies,
         log=log)
 
     results = {key: value / units[key]
@@ -343,6 +379,11 @@ def read_gpw(filename: Union[str, Path, IO[str]],
     if results:
         log(f'Read {", ".join(sorted(results))}')
 
+    if reader.version < 4 and 'magmoms' in results:
+        magmom_a = results['magmoms']
+        magmom_av = np.pad(magmom_a[:, np.newaxis], [(0, 0), (2, 0)])
+        results['non_collinear_magmoms'] = magmom_av
+
     dft.results = results
 
     if builder.mode in ['pw', 'fd']:  # fd = finite-difference
