@@ -10,6 +10,7 @@ from gpaw.gpu import as_np, synchronize
 from gpaw.gpu.mpi import CuPyMPI
 from gpaw.mpi import MPIComm, serial_comm
 from gpaw.new import zips
+from gpaw.new.timer import trace
 from gpaw.new.brillouin import IBZ
 from gpaw.new.c import GPU_AWARE_MPI
 from gpaw.new.potential import Potential
@@ -28,7 +29,6 @@ class IBZWaveFunctions(Generic[WFT]):
     def __init__(self,
                  ibz: IBZ,
                  *,
-                 nelectrons: float,
                  ncomponents: int,
                  wfs_qs: list[list[WFT]],
                  kpt_comm: MPIComm = serial_comm,
@@ -39,7 +39,6 @@ class IBZWaveFunctions(Generic[WFT]):
         self.kpt_comm = kpt_comm
         self.kpt_band_comm = kpt_band_comm
         self.comm = comm
-        self.nelectrons = nelectrons
         self.ncomponents = ncomponents
         self.collinear = (ncomponents != 4)
         self.spin_degeneracy = ncomponents % 2 + 1
@@ -60,8 +59,6 @@ class IBZWaveFunctions(Generic[WFT]):
 
         self.fermi_levels: Array1D | None = None  # hartree
 
-        self.energies: dict[str, float] = {}  # hartree
-
         self.xp = self.wfs_qs[0][0].xp
         if self.xp is not np:
             if not GPU_AWARE_MPI:
@@ -75,14 +72,12 @@ class IBZWaveFunctions(Generic[WFT]):
     def create(cls,
                *,
                ibz: IBZ,
-               nelectrons: float,
                ncomponents: int,
                create_wfs_func,
                kpt_comm: MPIComm = serial_comm,
                kpt_band_comm: MPIComm = serial_comm,
                comm: MPIComm = serial_comm,
                ) -> Self:
-        """Collection of wave function objects for k-points in the IBZ."""
         rank_k = ibz.ranks(kpt_comm)
         mask_k = (rank_k == kpt_comm.rank)
         k_q = np.arange(len(ibz))[mask_k]
@@ -99,7 +94,6 @@ class IBZWaveFunctions(Generic[WFT]):
             wfs_qs.append(wfs_s)
 
         return cls(ibz,
-                   nelectrons=nelectrons,
                    ncomponents=ncomponents,
                    wfs_qs=wfs_qs,
                    kpt_comm=kpt_comm,
@@ -115,8 +109,8 @@ class IBZWaveFunctions(Generic[WFT]):
             return 'fd'
         return 'lcao'
 
-    def has_wave_functions(self):
-        return True
+    def has_wave_functions(self) -> bool:
+        raise NotImplementedError
 
     def get_max_shape(self, global_shape: bool = False) -> tuple[int, ...]:
         """Find the largest wave function array shape.
@@ -153,10 +147,10 @@ class IBZWaveFunctions(Generic[WFT]):
                 '  # (' +
                 ('' if self.collinear else 'non-') + 'collinear spins)\n'
                 f'bands: {self.nbands}\n'
-                f'valence electrons: {self.nelectrons}\n'
                 f'spin-degeneracy: {self.spin_degeneracy}\n'
                 f'dtype: {self.dtype}\n\n'
                 'memory:\n'
+                f'    storage: {"CPU" if self.xp is np else "GPU"}\n'
                 f'    wave functions: {nbytes:_}  # bytes '
                 f' ({nbytes // ncores:_} per core)\n\n'
                 'parallelization:\n'
@@ -170,7 +164,6 @@ class IBZWaveFunctions(Generic[WFT]):
 
     def move(self, relpos_ac, atomdist):
         self.ibz.symmetries.check_positions(relpos_ac)
-        self.energies.clear()
         self.make_sure_wfs_are_read_from_gpw_file()
         for wfs in self:
             wfs.move(relpos_ac, atomdist, self.move_wave_functions)
@@ -179,12 +172,16 @@ class IBZWaveFunctions(Generic[WFT]):
         for wfs in self:
             wfs.orthonormalize(work_array_nX)
 
-    def calculate_occs(self, occ_calc, fix_fermi_level=False):
+    @trace
+    def calculate_occs(self,
+                       occ_calc,
+                       nelectrons: float,
+                       fix_fermi_level=False) -> tuple[float, float, float]:
         degeneracy = self.spin_degeneracy
 
         # u index is q and s combined
         occ_un, fermi_levels, e_entropy = occ_calc.calculate(
-            nelectrons=self.nelectrons / degeneracy,
+            nelectrons=nelectrons / degeneracy,
             eigenvalues=[wfs.eig_n * Ha for wfs in self],
             weights=[wfs.weight for wfs in self],
             fermi_levels_guess=(None
@@ -206,10 +203,7 @@ class IBZWaveFunctions(Generic[WFT]):
             e_band += wfs.occ_n @ wfs.eig_n * wfs.weight * degeneracy
         e_band = self.kpt_comm.sum_scalar(float(e_band))  # XXX CPU float?
 
-        self.energies.update(
-            band=e_band,
-            entropy=e_entropy,
-            extrapolation=e_entropy * occ_calc.extrapolate_factor)
+        return e_band, e_entropy, e_entropy * occ_calc.extrapolate_factor
 
     def add_to_density(self, nt_sR, D_asii) -> None:
         """Compute density and add to ``nt_sR`` and ``D_asii``."""
@@ -294,20 +288,20 @@ class IBZWaveFunctions(Generic[WFT]):
                 return eig_n, occ_n
         return np.zeros(0), np.zeros(0)
 
-    def get_all_eigs_and_occs(self):
+    def get_all_eigs_and_occs(self, broadcast=False):
         nkpts = len(self.ibz)
-        if self.comm.rank == 0:
-            eig_skn = np.empty((self.nspins, nkpts, self.nbands))
-            occ_skn = np.empty((self.nspins, nkpts, self.nbands))
-        else:
-            eig_skn = np.empty((self.nspins, nkpts, 0))
-            occ_skn = np.empty((self.nspins, nkpts, 0))
+        mynbands = self.nbands if self.comm.rank == 0 or broadcast else 0
+        eig_skn = np.empty((self.nspins, nkpts, mynbands))
+        occ_skn = np.empty((self.nspins, nkpts, mynbands))
         for k in range(nkpts):
             for s in range(self.nspins):
                 eig_n, occ_n = self.get_eigs_and_occs(k, s)
                 if self.comm.rank == 0:
                     eig_skn[s, k, :] = eig_n
                     occ_skn[s, k, :] = occ_n
+        if broadcast:
+            self.comm.broadcast(eig_skn, 0)
+            self.comm.broadcast(occ_skn, 0)
         return eig_skn, occ_skn
 
     def forces(self, potential: Potential) -> Array2D:
@@ -320,11 +314,7 @@ class IBZWaveFunctions(Generic[WFT]):
         self.kpt_band_comm.sum(F_av)
         return F_av
 
-    def write(self,
-              writer: Writer,
-              skip_wfs: bool,
-              precision: str = 'double',
-              include_projections=True) -> None:
+    def write(self, writer: Writer, flags) -> None:
         """Write fermi-level(s), eigenvalues, occupation numbers, ...
 
         ... k-points, symmetry information, projections and possibly
@@ -360,9 +350,10 @@ class IBZWaveFunctions(Generic[WFT]):
             spin_k_shape = (len(ibz),)
             proj_shape = (self.nbands, 2, nproj)
 
-        if include_projections:
+        if flags.include_projections:
+            proj_dtype = flags.storage_dtype(self.dtype)
             writer.add_array('projections', spin_k_shape + proj_shape,
-                             self.dtype)
+                             proj_dtype)
             for spin in range(self.nspins):
                 for k, rank in enumerate(self.rank_k):
                     if rank == self.kpt_comm.rank:
@@ -372,31 +363,25 @@ class IBZWaveFunctions(Generic[WFT]):
                             P_nI = P_ani.matrix.gather()  # gather bands
                             if P_nI.dist.comm.rank == 0:
                                 if rank == 0:
-                                    writer.fill(P_nI.data.reshape(proj_shape))
+                                    writer.fill(P_nI.data.reshape(
+                                        proj_shape).astype(proj_dtype))
                                 else:
                                     self.kpt_comm.send(P_nI.data, 0)
                     elif self.comm.rank == 0:
                         data = np.empty(proj_shape, self.dtype)
                         self.kpt_comm.receive(data, rank)
-                        writer.fill(data)
+                        writer.fill(data.astype(proj_dtype))
 
-        if not skip_wfs:
-            self._write_wave_functions(writer, spin_k_shape, precision)
+        if flags.include_wfs:
+            self._write_wave_functions(writer, spin_k_shape, flags)
 
-    def _write_wave_functions(self, writer, spin_k_shape, precision='double'):
-        from gpaw.new.gpw import as_single_precision
+    def _write_wave_functions(self, writer, spin_k_shape, flags):
         # We collect all bands to master.  This may have to be changed
         # to only one band at a time XXX
         xshape = self.get_max_shape(global_shape=True)
         shape = spin_k_shape + (self.nbands,) + xshape
         dtype = complex if self.mode == 'pw' else self.dtype
-        dtype_write = dtype
-        singlep = precision == 'single'
-        if singlep:
-            if dtype == complex:
-                dtype_write = np.complex64
-            else:
-                dtype_write = np.float32
+        dtype_write = flags.storage_dtype(dtype)
         c = 1.0 if self.mode == 'lcao' else Bohr**-1.5
 
         writer.add_array('coefficients', shape, dtype=dtype_write)
@@ -419,18 +404,12 @@ class IBZWaveFunctions(Generic[WFT]):
                                 buf_nX[..., x:] = 0.0
                                 coef_nX = buf_nX
                         if rank == 0:
-                            if singlep:
-                                writer.fill(as_single_precision(coef_nX * c))
-                            else:
-                                writer.fill(coef_nX * c)
+                            writer.fill(flags.to_storage_dtype(coef_nX * c))
                         else:
                             self.kpt_comm.send(coef_nX, 0)
                 elif self.comm.rank == 0:
                     self.kpt_comm.receive(buf_nX, rank)
-                    if singlep:
-                        writer.fill(as_single_precision(buf_nX * c))
-                    else:
-                        writer.fill(buf_nX * c)
+                    writer.fill(flags.to_storage_dtype(buf_nX * c))
 
     def write_summary(self, log):
         fl = self.fermi_levels * Ha
@@ -505,7 +484,7 @@ class IBZWaveFunctions(Generic[WFT]):
                 return
             if hasattr(psit_nX.data, 'fd'):  # fd=file-descriptor
                 self.read_from_file_init_wfs_dm = True
-                psit_nX.data = psit_nX.data[:]  # read
+                psit_nX.data = np.ascontiguousarray(psit_nX.data[:])  # read
 
     def get_homo_lumo(self, spin: int = None) -> Array1D:
         """Return HOMO and LUMO eigenvalues."""
