@@ -1,13 +1,13 @@
 import numpy as np
-from gpaw.core.matrix import Matrix
+from scipy import sparse
+from gpaw.core.matrix import Matrix, MatrixWithNoData
 from gpaw.lcao.tci import TCIExpansions
 from gpaw.new import zips
 from gpaw.new.fd.builder import FDDFTComponentsBuilder
 from gpaw.new.lcao.ibzwfs import LCAOIBZWaveFunctions
-from gpaw.new.lcao.eigensolver import LCAOEigensolver
 from gpaw.new.lcao.forces import TCIDerivatives
 from gpaw.new.lcao.hamiltonian import LCAOHamiltonian
-from gpaw.new.lcao.hybrids import HybridLCAOEigensolver, HybridXCFunctional
+from gpaw.new.lcao.hybrids import HybridXCFunctional
 from gpaw.new.lcao.wave_functions import LCAOWaveFunctions
 from gpaw.utilities.timing import NullTimer
 
@@ -18,11 +18,9 @@ class LCAODFTComponentsBuilder(FDDFTComponentsBuilder):
                  params,
                  *,
                  comm,
-                 distribution=None,
-                 interpolation=3):
-        super().__init__(atoms, params, comm=comm)
-        assert interpolation == 3
-        self.distribution = distribution
+                 log):
+        super().__init__(atoms, params, comm=comm, log=log)
+        self.distribution = params.mode.distribution
         self.basis = None
 
     def create_wf_description(self):
@@ -41,16 +39,18 @@ class LCAODFTComponentsBuilder(FDDFTComponentsBuilder):
         return LCAOHamiltonian(self.basis)
 
     def create_eigensolver(self, hamiltonian):
-        if self.params.xc['name'] in ['HSE06', 'PBE0', 'EXX']:
-            return HybridLCAOEigensolver(self.basis,
-                                         self.relpos_ac,
-                                         self.grid.cell_cv)
-        if self.params.eigensolver.get('name') == 'scissors':
-            from gpaw.lcao.scissors import ScissorsLCAOEigensolver
-            return ScissorsLCAOEigensolver(self.basis,
-                                           self.params.eigensolver['shifts'],
-                                           self.ibz.symmetries)
-        return LCAOEigensolver(self.basis)
+        from gpaw.dft import DefaultEigensolver
+        es = self.params.eigensolver
+        if isinstance(es, DefaultEigensolver):
+            if self.params.xc.name in ['HSE06', 'PBE0', 'EXX']:
+                name = 'hybrid'
+            else:
+                name = 'lcao'
+            es = es.from_param({'name': name, **es.params})
+        return es.build_lcao(self.basis,
+                             self.relpos_ac,
+                             self.grid.cell_cv,
+                             self.ibz.symmetries)
 
     def read_ibz_wave_functions(self, reader):
         c = 1
@@ -76,7 +76,6 @@ class LCAODFTComponentsBuilder(FDDFTComponentsBuilder):
                                   basis,
                                   potential,
                                   *,
-                                  log=None,
                                   coefficients=None):
         ibzwfs, _ = create_lcao_ibzwfs(
             basis,
@@ -105,18 +104,17 @@ def create_lcao_ibzwfs(basis,
     nao = setups.nao
 
     def create_wfs(spin, q, k, kpt_c, weight):
-        C_nM = Matrix(nbands, 2 * nao if ncomponents == 4 else nao,
-                      dtype,
-                      dist=(band_comm, band_comm.size, 1))
+        shape = (nbands, 2 * nao if ncomponents == 4 else nao)
         if coefficients is not None:
-            C_nM.data[:] = coefficients.proxy(spin, k)
+            C_nM = Matrix(*shape,
+                          dtype=dtype,
+                          dist=(band_comm, band_comm.size, 1))
+            n1, n2 = C_nM.dist.my_row_range()
+            C_nM.data[:] = coefficients.proxy(spin, k)[n1:n2]
         else:
-            # We set the first element to NaN as a hack so that the
-            # code can later tell that the data is not initialized.
-            # We could set /all/ the elements, but what we care about is
-            # only this piece of information.  Maybe we can find a better
-            # solution.
-            pass  # C_nM.data[:1, :1] = np.nan
+            C_nM = MatrixWithNoData(*shape,
+                                    dtype=dtype,
+                                    dist=(band_comm, band_comm.size, 1))
         return LCAOWaveFunctions(
             setups=setups,
             tci_derivatives=tci_derivatives,
@@ -137,7 +135,6 @@ def create_lcao_ibzwfs(basis,
 
     ibzwfs = LCAOIBZWaveFunctions.create(
         ibz=ibz,
-        nelectrons=nelectrons,
         ncomponents=ncomponents,
         create_wfs_func=create_wfs,
         kpt_comm=kpt_comm,
@@ -175,10 +172,11 @@ def tci_helper(basis,
     P_qaMi = [{a: P_aqMi[a][q] for a in my_atom_indices}
               for q in range(len(S0_qMM))]
 
-    for a, P_qMi in P_aqMi.items():
-        dO_ii = setups[a].dO_ii
-        for P_Mi, S_MM in zips(P_qMi, S0_qMM):
-            S_MM += P_Mi[M1:M2].conj() @ dO_ii @ P_Mi.T
+    add_atomic_overlap_corrections(P_qaMi=P_qaMi,
+                                   S0_qMM=S0_qMM,
+                                   setups=setups,
+                                   M1=M1,
+                                   M2=M2)
     domain_comm.sum(S0_qMM)
 
     # self.atomic_correction= self.atomic_correction_cls.new_from_wfs(self)
@@ -199,3 +197,23 @@ def tci_helper(basis,
     tci_derivatives = TCIDerivatives(manytci, atomdist, nao)
 
     return S_qMM, T_qMM, P_qaMi, tciexpansions, tci_derivatives
+
+
+def add_atomic_overlap_corrections(
+        P_qaMi,
+        S0_qMM,
+        setups,
+        M1: int,
+        M2: int):
+    for P_aMi, S_MM in zips(P_qaMi, S0_qMM):
+        # No atoms on this rank
+        if len(P_aMi) == 0:
+            continue
+
+        dO_II = sparse.block_diag(
+            [setups[a].dO_ii for a in P_aMi],
+            format='csr')
+        P_MI = sparse.hstack(
+            [sparse.coo_matrix(P_Mi) for P_Mi in P_aMi.values()],
+            format='csr')
+        S_MM += P_MI[M1:M2].conj().dot(dO_II.dot(P_MI.T)).todense()
