@@ -66,12 +66,16 @@ def add_force_contributions(wfs: LCAOWaveFunctions,
 
     setups = wfs.setups
 
+    my_row_range = wfs.T_MM.dist.my_row_range()
+    m1, m2 = my_row_range
     rhoT_MM = wfs.calculate_density_matrix(transposed=True)
     erhoT_MM = wfs.calculate_density_matrix(transposed=True, eigs=True)
-    add_kinetic_term(rhoT_MM, dTdR_vMM, F_av, indices, wfs.atomdist.indices)
+    add_kinetic_term(rhoT_MM, dTdR_vMM[:, m1:m2], F_av,
+                     indices, wfs.atomdist.indices, my_row_range)
     add_pot_term(potential.vt_sR[wfs.spin], wfs.basis, wfs.q, rhoT_MM, F_av)
-    add_den_mat_term(erhoT_MM, dThetadR_vMM, F_av, indices,
-                     wfs.atomdist.indices)
+    add_den_mat_term(erhoT_MM, dThetadR_vMM[:, m1:m2], F_av, indices,
+                     wfs.atomdist.indices, my_row_range)
+
     for b in wfs.atomdist.indices:
         add_den_mat_paw_term(b,
                              setups[b].dO_ii,
@@ -79,17 +83,20 @@ def add_force_contributions(wfs: LCAOWaveFunctions,
                              dPdR_avMi[b],
                              erhoT_MM,
                              indices,
-                             F_av)
+                             F_av,
+                             my_row_range)
+
         add_atomic_density_term(b,
                                 potential.dH_asii[b][wfs.spin],
                                 wfs.P_aMi[b],
                                 dPdR_avMi[b],
                                 rhoT_MM,
                                 indices,
-                                F_av)
+                                F_av,
+                                my_row_range)
 
 
-def add_kinetic_term(rhoT_MM, dTdR_vMM, F_av, indices, mya):
+def add_kinetic_term(rhoT_MM, dTdR_vMM, F_av, indices, mya, my_row_range):
     """Calculate Kinetic energy term in LCAO
 
     :::
@@ -103,10 +110,12 @@ def add_kinetic_term(rhoT_MM, dTdR_vMM, F_av, indices, mya):
             """
 
     for a, M1, M2 in indices:
-        if a in mya:
+        if a in mya and M2 >= my_row_range[0] and M1 <= my_row_range[1]:
+            m1 = max(M1, my_row_range[0]) - my_row_range[0]
+            m2 = min(M2, my_row_range[1]) - my_row_range[0]
             F_av[a, :] += 2 * np.einsum('vmM, mM -> v',
-                                        dTdR_vMM[:, M1:M2],
-                                        rhoT_MM[M1:M2]).real
+                                        dTdR_vMM[:, m1:m2],
+                                        rhoT_MM[m1:m2]).real
 
 
 def add_pot_term(vt_R: UGArray,
@@ -129,7 +138,7 @@ def add_pot_term(vt_R: UGArray,
                                                q)
 
 
-def add_den_mat_term(erhoT_MM, dThetadR_vMM, F_av, indices, mya):
+def add_den_mat_term(erhoT_MM, dThetadR_vMM, F_av, indices, mya, my_row_range):
     """Calculate density matrix term in LCAO"""
     # Density matrix contribution due to basis overlap
     #
@@ -141,13 +150,16 @@ def add_den_mat_term(erhoT_MM, dThetadR_vMM, F_av, indices, mya):
     #         mu in a; nu
     #
     for a, M1, M2 in indices:
-        if a in mya:
+        if a in mya and M2 >= my_row_range[0] and M1 <= my_row_range[1]:
+            m1 = max(M1, my_row_range[0]) - my_row_range[0]
+            m2 = min(M2, my_row_range[1]) - my_row_range[0]
             F_av[a, :] -= 2 * np.einsum('vmM, mM -> v',
-                                        dThetadR_vMM[:, M1:M2],
-                                        erhoT_MM[M1:M2]).real
+                                        dThetadR_vMM[:, m1:m2],
+                                        erhoT_MM[m1:m2]).real
 
 
-def add_den_mat_paw_term(b, dO_ii, P_Mi, dPdR_vMi, erhoT_MM, indices, F_av):
+def add_den_mat_paw_term(b, dO_ii, P_Mi, dPdR_vMi, erhoT_MM,
+                         indices, F_av, my_row_range):
     """Calcualte PAW correction"""
     # TO DO: split this function into
     # _get_den_mat_paw_term (which calculate Frho_av) and
@@ -170,23 +182,28 @@ def add_den_mat_paw_term(b, dO_ii, P_Mi, dPdR_vMi, erhoT_MM, indices, F_av):
     #         -----    b mu
     #           ij
     #
+    m1, m2 = my_row_range
     dtype = P_Mi.dtype
-    Z_MM = np.zeros((len(P_Mi), len(P_Mi)), dtype)
+    Z_MM = np.zeros((m2 - m1, len(P_Mi)), dtype)
     dOP_iM = np.zeros((len(dO_ii), len(P_Mi)), dtype)
     mmm(1.0, dO_ii.astype(dtype), 'N', P_Mi, 'C', 0.0, dOP_iM)
     for v in range(3):
         mmm(1.0,
-            dPdR_vMi[v], 'N',
+            dPdR_vMi[v, m1:m2], 'N',
             dOP_iM, 'N',
             0.0, Z_MM)
         ZE_M = np.einsum('MN, MN -> M', Z_MM, erhoT_MM).real
         for a, M1, M2 in indices:
-            dE = 2 * ZE_M[M1:M2].sum()
-            F_av[a, v] -= dE  # the "b; mu in a; nu" term
-            F_av[b, v] += dE  # the "mu nu" term
+            if M2 >= my_row_range[0] and M1 <= my_row_range[1]:
+                am1 = max(M1, my_row_range[0]) - my_row_range[0]
+                am2 = min(M2, my_row_range[1]) - my_row_range[0]
+                dE = 2 * ZE_M[am1:am2].sum()
+                F_av[a, v] -= dE  # the "b; mu in a; nu" term
+                F_av[b, v] += dE  # the "mu nu" term
 
 
-def add_atomic_density_term(b, dH_ii, P_Mi, dPdR_vMi, rhoT_MM, indices, F_av):
+def add_atomic_density_term(b, dH_ii, P_Mi, dPdR_vMi, rhoT_MM,
+                            indices, F_av, my_row_range):
     # Atomic density contribution
     #            -----                         -----
     #  a          \     a                       \     b
@@ -203,17 +220,21 @@ def add_atomic_density_term(b, dH_ii, P_Mi, dPdR_vMi, rhoT_MM, indices, F_av):
     #         -----    b mu
     #           ij
     #
+    m1, m2 = my_row_range
     dtype = P_Mi.dtype
-    A_MM = np.zeros((len(P_Mi), len(P_Mi)), dtype)
+    A_MM = np.zeros((m2 - m1, len(P_Mi)), dtype)
     dHP_iM = np.zeros((len(dH_ii), len(P_Mi)), dtype)
     mmm(1.0, dH_ii.astype(dtype), 'N', P_Mi, 'C', 0.0, dHP_iM)
     for v in range(3):
         mmm(1.0,
-            dPdR_vMi[v], 'N',
+            dPdR_vMi[v, m1:m2], 'N',
             dHP_iM, 'N',
             0.0, A_MM)
         AR_M = np.einsum('MN, MN -> M', A_MM, rhoT_MM).real
         for a, M1, M2 in indices:
-            dE = 2 * AR_M[M1:M2].sum()
-            F_av[a, v] += dE  # the "b; mu in a; nu" term
-            F_av[b, v] -= dE  # the "mu nu" term
+            if M2 >= my_row_range[0] and M1 <= my_row_range[1]:
+                am1 = max(M1, my_row_range[0]) - my_row_range[0]
+                am2 = min(M2, my_row_range[1]) - my_row_range[0]
+                dE = 2 * AR_M[am1:am2].sum()
+                F_av[a, v] += dE  # the "b; mu in a; nu" term
+                F_av[b, v] -= dE  # the "mu nu" term
