@@ -12,23 +12,27 @@ x   r or h
 
 from __future__ import annotations
 
+import functools
+import operator
 from collections import defaultdict
 from typing import DefaultDict
 
 import numpy as np
+from ase.units import Ha
 from gpaw.core.arrays import DistributedArrays
 from gpaw.core.atom_arrays import AtomArrays
 from gpaw.core.uniform_grid import UGArray
+from gpaw.mpi import MPIComm, serial_comm
 from gpaw.new import trace, zips
+from gpaw.new.energies import DFTEnergies
+from gpaw.new.environment import Environment
+from gpaw.new.logger import indent
 from gpaw.new.potential import Potential
 from gpaw.new.xc import Functional
 from gpaw.setup import Setup
 from gpaw.spinorbit import soc as soc_terms
 from gpaw.typing import Array1D, Array2D, Array3D
-from gpaw.utilities import pack_hermitian, pack_density, unpack_hermitian
-from gpaw.new.logger import indent
-from gpaw.mpi import MPIComm, serial_comm
-from gpaw.new.external_potential import ExternalPotential
+from gpaw.utilities import pack_density, pack_hermitian, unpack_hermitian
 
 
 class PotentialCalculator:
@@ -38,14 +42,16 @@ class PotentialCalculator:
                  setups: list[Setup],
                  *,
                  relpos_ac: Array2D,
-                 external_potential: ExternalPotential | None = None,
+                 environment: Environment,
+                 extensions: list | None = None,
                  soc: bool = False):
         self.poisson_solver = poisson_solver
         self.xc = xc
         self.setups = setups
-        self.external_potential = external_potential or ExternalPotential()
         self.relpos_ac = relpos_ac
         self.soc = soc
+        self.environment = environment or Environment(len(relpos_ac))
+        self.extensions: list = extensions or []
 
     def __str__(self):
         return (f'{self.poisson_solver}\n'
@@ -59,9 +65,28 @@ class PotentialCalculator:
                                               UGArray,
                                               UGArray,
                                               DistributedArrays,
-                                              AtomArrays]:
+                                              AtomArrays,
+                                              float]:
         raise NotImplementedError
 
+    def move(self, relpos_ac, atomdist):
+        for ext in self.extensions:
+            ext.move_atoms(relpos_ac)
+
+    @property
+    def extensions_force_av(self):
+        if not self.extensions:
+            return np.zeros((len(self.setups), 3))
+        return functools.reduce(operator.add, [ext.force_contribution()
+                                for ext in self.extensions])
+
+    @property
+    def extensions_stress_contribution(self):
+        if not self.extensions:
+            return np.zeros((3, 3))
+        return functools.reduce(operator.add, [ext.stress_contribution()
+                                for ext in self.extensions])
+
     def calculate_charges(self, vHt_x):
         raise NotImplementedError
 
@@ -73,23 +98,27 @@ class PotentialCalculator:
                                    ibzwfs=None,
                                    vHt_x: DistributedArrays | None = None,
                                    kpt_band_comm: MPIComm | None = None
-                                   ) -> tuple[Potential, AtomArrays]:
+                                   ) -> tuple[Potential,
+                                              DFTEnergies,
+                                              AtomArrays]:
         xc = self.xc
         if xc.exx_fraction != 0.0:
             from gpaw.new.xc import create_functional
             self.xc = create_functional('PBE', xc.grid)
-        potential, V_al = self.calculate(density, ibzwfs, vHt_x, kpt_band_comm)
+        potential, energies, V_al = self.calculate(
+            density, ibzwfs, vHt_x, kpt_band_comm)
         if xc.exx_fraction != 0.0:
             self.xc = xc
-        return potential, V_al
+        return potential, energies, V_al
 
+    @trace
     def calculate(self,
                   density,
                   ibzwfs=None,
                   vHt_x: DistributedArrays | None = None,
                   kpt_band_comm: MPIComm | None = None
-                  ) -> tuple[Potential, AtomArrays]:
-        energies, vt_sR, dedtaut_sr, vHt_x, V_aL = (
+                  ) -> tuple[Potential, DFTEnergies, AtomArrays]:
+        energies, vt_sR, dedtaut_sr, vHt_x, V_aL, e_stress = (
             self.calculate_pseudo_potential(density, ibzwfs, vHt_x))
         e_kinetic = 0.0
         for spin, (vt_R, nt_R) in enumerate(zips(vt_sR, density.nt_sR)):
@@ -106,7 +135,7 @@ class PotentialCalculator:
         else:
             dedtaut_sR = None
 
-        energies['kinetic'] = e_kinetic
+        energies['kinetic_correction'] = e_kinetic
 
         if kpt_band_comm is None:
             if ibzwfs is None:
@@ -117,27 +146,35 @@ class PotentialCalculator:
             self.setups,
             density,
             self.xc,
-            self.external_potential,
             V_aL,
             self.soc,
+            self.extensions,
             kpt_band_comm)
 
+        for ext in self.extensions:
+            dct = ext.get_energy_contributions()
+            for name, e in dct.items():
+                assert name not in energies
+                energies[name] = e
+
         energies['spinorbit'] = 0.0
         for key, e in corrections.items():
-            if 0:
-                print(f'{key:10} {energies[key]:15.9f} {e:15.9f}')
+            if False:
+                print(f'{key:10} {energies[key] * Ha:15.9f} {e * Ha:15.9f}')
             energies[key] += e
 
-        return Potential(vt_sR, dH_asii, dedtaut_sR, energies, vHt_x), V_aL
+        return (Potential(vt_sR, dH_asii, dedtaut_sR, vHt_x, e_stress),
+                DFTEnergies(**energies),
+                V_aL)
 
 
 @trace
 def calculate_non_local_potential(setups,
                                   density,
                                   xc,
-                                  ext_pot,
                                   V_aL,
                                   soc: bool,
+                                  extensions,
                                   kpt_band_comm: MPIComm
                                   ) -> tuple[AtomArrays,
                                              dict[str, float]]:
@@ -152,7 +189,7 @@ def calculate_non_local_potential(setups,
             V_L = V_aL[a]
             setup = setups[a]
             dH_sii, corrections = calculate_non_local_potential1(
-                setup, xc, ext_pot, D_sii, V_L, soc)
+                setup, xc, D_sii, V_L, soc, extensions, a)
             dH_asii[a][:] = dH_sii
             for key, e in corrections.items():
                 energy_corrections[key] += e
@@ -163,7 +200,8 @@ def calculate_non_local_potential(setups,
     kpt_band_comm.sum(dH_asii.data)
 
     # Sum over domain:
-    names = ['kinetic', 'coulomb', 'zero', 'xc', 'external', 'spinorbit']
+    names = ['kinetic_correction', 'coulomb', 'zero', 'xc', 'external',
+             'spinorbit']
     energies = np.array([energy_corrections[name] for name in names])
     density.D_asii.layout.atomdist.comm.sum(energies)
     kpt_band_comm.sum(energies)
@@ -174,11 +212,12 @@ def calculate_non_local_potential(setups,
 
 def calculate_non_local_potential1(setup: Setup,
                                    xc: Functional,
-                                   ext_pot,
                                    D_sii: Array3D,
                                    V_L: Array1D,
-                                   soc: bool) -> tuple[Array3D,
-                                                       dict[str, float]]:
+                                   soc: bool,
+                                   extensions,
+                                   atom_index) -> tuple[Array3D,
+                                                        dict[str, float]]:
     ncomponents = len(D_sii)
     ndensities = 2 if ncomponents == 2 else 1
     D_sp = np.array([pack_density(D_ii.real) for D_ii in D_sii])
@@ -203,7 +242,7 @@ def calculate_non_local_potential1(setup: Setup,
     dH_sp[:ndensities] = dH_p
     e_xc = xc.calculate_paw_correction(setup, D_sp, dH_sp)
 
-    e_external = ext_pot.add_paw_correction(setup.Delta_pL[:, 0], dH_sp)
+    # e_external = ext_pot.add_paw_correction(setup.Delta_pL[:, 0], dH_sp)
 
     dH_sii = unpack_hermitian(dH_sp)
 
@@ -212,11 +251,15 @@ def calculate_non_local_potential1(setup: Setup,
         e_xc += eU
         dH_sii += dHU_sii
 
+    for extension in extensions:
+        e_xc += extension.update_non_local_hamiltonian(
+            D_sii, setup, atom_index, dH_sii)
+
     e_kinetic -= (D_sii * dH_sii).sum().real
 
-    return dH_sii, {'kinetic': e_kinetic,
+    return dH_sii, {'kinetic_correction': e_kinetic,
                     'coulomb': e_coulomb,
                     'zero': e_zero,
                     'xc': e_xc,
-                    'external': e_external,
+                    'external': 0.0,  # e_external,
                     'spinorbit': e_soc}
