@@ -16,29 +16,22 @@ class Potential:
                  vt_sR: UGArray,
                  dH_asii: AtomArrays,
                  dedtaut_sR: UGArray | None,
-                 energies: dict[str, float],
-                 vHt_x: XArray | None = None):
+                 vHt_x: XArray | None = None,
+                 e_stress: float = np.nan):
         self.vt_sR = vt_sR
         self.dH_asii = dH_asii
         self.dedtaut_sR = dedtaut_sR
-        self.energies = energies
         self.vHt_x = vHt_x  # initial guess for Hartree potential
+        self.e_stress = e_stress  # idotropic contribution to stress tensor
 
     def __repr__(self):
         return (f'Potential({self.vt_sR}, {self.dH_asii}, '
-                f'{self.dedtaut_sR}, {self.energies})')
+                f'{self.dedtaut_sR})')
 
     def __str__(self) -> str:
         return (f'potential:\n'
                 f'  grid points: {self.vt_sR.desc.size}\n')
 
-    def update_from(self, potential):
-        self.vt_sR = potential.vt_sR
-        self.dH_asii = potential.dH_asii
-        self.dedtaut_sR = potential.dedtaut_sR
-        self.energies = potential.energies
-        self.vHt_x = potential.vHt_x
-
     def dH(self, P_ani, out_ani, spin):
         if len(P_ani.dims) == 1:  # collinear wave functions
             P_ani.block_diag_multiply(self.dH_asii, out_ani, spin)
@@ -72,16 +65,10 @@ class Potential:
             self.dH_asii.redist(atomdist, comm1, comm2),
             None if self.dedtaut_sR is None else self.dedtaut_sR.redist(
                 grid, comm1, comm2),
-            self.energies.copy(),
             None if self.vHt_x is None else self.vHt_x.redist(
                 desc, comm1, comm2))
 
-    def _write_gpw(self, writer, ibzwfs, precision='double'):
-        from gpaw.new.calculation import combine_energies
-        energies = combine_energies(self, ibzwfs)
-        energies['band'] = ibzwfs.energies['band']
-        if 'stress' in self.energies:
-            energies['stress'] = self.energies['stress']
+    def write_to_gpw(self, writer, flags):
         dH_asp = self.dH_asii.to_cpu().to_lower_triangle().gather()
         vt_sR = self.vt_sR.to_xp(np).gather()
         if self.dedtaut_sR is not None:
@@ -91,24 +78,15 @@ class Potential:
         if dH_asp is None:
             return
 
-        vt_sR_data = vt_sR.data
-        if precision == 'single':
-            from gpaw.new.gpw import as_single_precision
-            vt_sR_data = as_single_precision(vt_sR_data)
         writer.write(
-            potential=vt_sR_data * Ha,
-            atomic_hamiltonian_matrices=dH_asp.data * Ha,
-            **{f'e_{name}': val * Ha for name, val in energies.items()})
+            potential=flags.to_storage_dtype(vt_sR.data * Ha),
+            atomic_hamiltonian_matrices=dH_asp.data * Ha)
         if self.vHt_x is not None:
-            vHt_x_data = vHt_x.data
-            if precision == 'single':
-                vHt_x_data = as_single_precision(vHt_x_data)
-            writer.write(electrostatic_potential=vHt_x_data * Ha)
+            vHt_x_data = flags.to_storage_dtype(vHt_x.data * Ha)
+            writer.write(electrostatic_potential=vHt_x_data)
         if self.dedtaut_sR is not None:
-            dedtaut_sR_data = dedtaut_sR.data
-            if precision == 'single':
-                dedtaut_sR_data = as_single_precision(dedtaut_sR_data)
-            writer.write(mgga_potential=dedtaut_sR_data * Bohr**3)
+            dedtaut_sR_data = flags.to_storage_dtype(dedtaut_sR.data * Bohr**3)
+            writer.write(mgga_potential=dedtaut_sR_data)
 
     def get_vacuum_level(self) -> float:
         grid = self.vt_sR.desc
