@@ -1,19 +1,23 @@
 from __future__ import annotations
 
+import os
+import warnings
 from functools import cached_property
 
+import numpy as np
 from ase.units import Ha
+
 from gpaw.core import PWDesc, UGDesc
 from gpaw.core.domain import Domain
 from gpaw.core.matrix import Matrix
 from gpaw.core.plane_waves import PWArray
 from gpaw.new import zips
 from gpaw.new.builder import create_uniform_grid
-from gpaw.new.external_potential import create_external_potential
 from gpaw.new.gpw import as_double_precision
 from gpaw.new.pw.bloechl_poisson import BloechlPAWPoissonSolver
 from gpaw.new.pw.hamiltonian import PWHamiltonian, SpinorPWHamiltonian
 from gpaw.new.pw.hybrids import PWHybridHamiltonian
+from gpaw.new.pw.hybridsk import PWHybridHamiltonianK
 from gpaw.new.pw.paw_poisson import SlowPAWPoissonSolver
 from gpaw.new.pw.poisson import make_poisson_solver
 from gpaw.new.pw.pot_calc import PlaneWavePotentialCalculator
@@ -23,22 +27,24 @@ from gpaw.typing import Array1D
 
 
 class PWDFTComponentsBuilder(PWFDDFTComponentsBuilder):
-    interpolation = 'fft'
-
     def __init__(self,
                  atoms,
                  params,
                  *,
-                 comm,
-                 ecut=340,
-                 qspiral=None,
-                 dedecut=None):
-        self.ecut = ecut / Ha
-        super().__init__(atoms, params, comm=comm, qspiral=qspiral)
+                 comm=None,
+                 log=None):
+        mode = params.mode
+        self.ecut = mode.ecut / Ha
+        # mode.dedecut ???
+        super().__init__(atoms, params, comm=comm, log=log)
 
         self._nct_ag = None
         self._tauct_ag = None
 
+        nthreads = int(os.environ.get('OMP_NUM_THREADS', '') or '1')
+        if nthreads > 1:
+            warnings.warn(
+                'Using OMP_NUM_THREADS>1 in PW-mode is not useful!')
         # We should just distribute the atom evenly, but that is not compatible
         # with LCAO initialization!
         # return AtomDistribution.from_number_of_atoms(len(self.relpos_ac),
@@ -52,7 +58,7 @@ class PWDFTComponentsBuilder(PWFDDFTComponentsBuilder):
             self.atoms.pbc,
             self.ibz.symmetries,
             h=self.params.h,
-            interpolation='fft',
+            interpolation=self.params.interpolation or 'fft',
             ecut=self.ecut,
             comm=self.communicators['d'])
         fine_grid = grid.new(size=grid.size_c * 2)
@@ -87,8 +93,8 @@ class PWDFTComponentsBuilder(PWFDDFTComponentsBuilder):
         return self.interpolation_desc.new(ecut=8 * self.ecut)
 
     @cached_property
-    def fast_poisson_solver(self):
-        fast = self.params.poissonsolver.get('fast', False)
+    def fast_poisson_solver(self) -> bool:
+        fast = self.params.poissonsolver.params.get('fast', False)
         if fast:
             # Only works for gaussian compensation charges at the moment:
             fast = False
@@ -114,8 +120,8 @@ class PWDFTComponentsBuilder(PWFDDFTComponentsBuilder):
                 self.interpolation_desc, self.relpos_ac, self.atomdist)
         return self._tauct_ag
 
-    def create_poisson_solver(self):
-        psparams = self.params.poissonsolver.copy() or {'strength': 1.0}
+    def create_poisson_solver(self, env):
+        psparams = self.params.poissonsolver.params.copy() or {'strength': 1.0}
         psparams.pop('fast', False)
 
         if self.fast_poisson_solver:
@@ -126,7 +132,8 @@ class PWDFTComponentsBuilder(PWFDDFTComponentsBuilder):
         pw = self.electrostatic_potential_desc
         ps = make_poisson_solver(pw,
                                  grid,
-                                 self.params.charge,
+                                 self.charge,
+                                 env,
                                  **psparams)
 
         if self.fast_poisson_solver:
@@ -140,17 +147,19 @@ class PWDFTComponentsBuilder(PWFDDFTComponentsBuilder):
             ps, self.relpos_ac, self.atomdist, self.xp)
 
     def create_potential_calculator(self):
+        env = self.create_environment(self.fine_grid)
         return PlaneWavePotentialCalculator(
             self.grid, self.fine_grid,
             self.interpolation_desc,
             self.setups,
             self.xc,
-            self.create_poisson_solver(),
-            external_potential=create_external_potential(self.params.external),
+            self.create_poisson_solver(env),
             relpos_ac=self.relpos_ac,
             atomdist=self.atomdist,
             soc=self.soc,
-            xp=self.xp)
+            xp=self.xp,
+            environment=env,
+            extensions=self.get_extensions())
 
     def create_hamiltonian_operator(self, blocksize=10):
         if self.ncomponents < 4:
@@ -159,11 +168,18 @@ class PWDFTComponentsBuilder(PWFDDFTComponentsBuilder):
             assert self.communicators['d'].size == 1
             assert self.communicators['k'].size == 1
             assert self.nbands % self.communicators['b'].size == 0
-            return PWHybridHamiltonian(
-                self.grid, self.wf_desc, self.xc, self.setups,
-                self.relpos_ac, self.atomdist,
-                comp_charge_in_real_space=self.params.experimental.get(
-                    'ccirs'))
+            if self.dtype is float:
+                return PWHybridHamiltonian(
+                    self.grid, self.wf_desc, self.xc, self.setups,
+                    self.relpos_ac, self.atomdist,
+                    comp_charge_in_real_space=self.params.experimental.get(
+                        'ccirs'))
+            else:
+                return PWHybridHamiltonianK(
+                    self.grid, self.wf_desc, self.xc, self.setups,
+                    self.relpos_ac, self.atomdist, self.log,
+                    self.communicators['k'],
+                    self.communicators['w'])
         return SpinorPWHamiltonian(self.qspiral_v)
 
     def convert_wave_functions_from_uniform_grid(self,
@@ -171,14 +187,33 @@ class PWDFTComponentsBuilder(PWFDDFTComponentsBuilder):
                                                  basis_set,
                                                  kpt_c,
                                                  q):
-        # Replace this with code that goes directly from C_nM to
-        # psit_nG via PWAtomCenteredFunctions.
-        # XXX
+        if self.params.experimental.get('fast_pw_init', True):
+            if self.ncomponents < 4:
+                from gpaw.core.pwacf import PWAtomCenteredFunctions
+                pw = self.wf_desc.new(kpt=kpt_c)
+                phit_aJG = PWAtomCenteredFunctions(
+                    [setup.basis_functions_J for setup in self.setups],
+                    self.relpos_ac,
+                    pw,
+                    atomdist=self.atomdist,
+                    xp=self.xp)
+                psit_nG = pw.empty(self.nbands,
+                                   comm=self.communicators['b'],
+                                   xp=self.xp)
+                mynbands, M = C_nM.dist.shape
+                phit_aJG.multiply(C_nM.to_xp(self.xp).to_dtype(pw.dtype),
+                                  out_nG=psit_nG[:mynbands])
+                return psit_nG
 
-        grid = self.grid.new(kpt=kpt_c, dtype=self.dtype)
-        pw = self.wf_desc.new(kpt=kpt_c)
+        lcao_dtype = complex if \
+            np.issubdtype(self.dtype, np.complexfloating) else float
 
-        if self.dtype == complex:
+        grid = self.grid.new(kpt=kpt_c, dtype=lcao_dtype)
+        pw = self.wf_desc.new(kpt=kpt_c, dtype=lcao_dtype)
+        if self.dtype != lcao_dtype:
+            pw_correct = self.wf_desc.new(kpt=kpt_c, dtype=self.dtype)
+
+        if np.issubdtype(self.dtype, np.complexfloating):
             emikr_R = grid.eikr(-kpt_c)
 
         mynbands, M = C_nM.dist.shape
@@ -188,9 +223,15 @@ class PWDFTComponentsBuilder(PWFDDFTComponentsBuilder):
             basis_set.lcao_to_grid(C_nM.data, psit_nR.data, q)
 
             for psit_R, psit_G in zips(psit_nR, psit_nG, strict=False):
-                if self.dtype == complex:
+                if np.issubdtype(self.dtype, np.complexfloating):
                     psit_R.data *= emikr_R
                 psit_R.fft(out=psit_G)
+
+            if self.dtype != lcao_dtype:
+                psit2_nG = pw_correct.empty(self.nbands,
+                                            self.communicators['b'])
+                psit2_nG.data[:] = psit_nG.data
+                return psit2_nG.to_xp(self.xp)
             return psit_nG.to_xp(self.xp)
         else:
             psit_nsG = pw.empty((self.nbands, 2), self.communicators['b'])
@@ -264,7 +305,7 @@ def check_g_vector_ordering(grid: UGDesc,
                             pw: PWDesc,
                             index_G: Array1D) -> None:
     size = tuple(grid.size)
-    if pw.dtype == float:
+    if np.issubdtype(pw.dtype, np.floating):
         size = (size[0], size[1], size[2] // 2 + 1)
     index0_G = pw.indices(size)
     nG = len(index0_G)
