@@ -1,15 +1,13 @@
-from types import SimpleNamespace
 from math import pi
 
 import numpy as np
 
 from gpaw.new.builder import DFTComponentsBuilder
-from gpaw.new.pwfd.ibzwfs import PWFDIBZWaveFunction
+from gpaw.new.pwfd.ibzwfs import PWFDIBZWaveFunctions
 from gpaw.new.lcao.eigensolver import LCAOEigensolver
 from gpaw.new.lcao.hamiltonian import LCAOHamiltonian
-from gpaw.new.pwfd.davidson import Davidson
-from gpaw.new.pwfd.etdm import ETDMPWFD
 from gpaw.new.pwfd.wave_functions import PWFDWaveFunctions
+from gpaw.core.arrays import XArrayWithNoData
 
 
 class PWFDDFTComponentsBuilder(DFTComponentsBuilder):
@@ -17,41 +15,37 @@ class PWFDDFTComponentsBuilder(DFTComponentsBuilder):
                  atoms,
                  params,
                  *,
-                 comm,
-                 qspiral=None):
-        super().__init__(atoms, params, comm=comm)
+                 comm=None,
+                 log=None):
+        super().__init__(atoms, params, comm=comm, log=log)
+        qspiral = params.mode.qspiral
         self.qspiral_v = (None if qspiral is None else
                           qspiral @ self.grid.icell * (2 * pi))
 
     def create_eigensolver(self, hamiltonian):
-        eigsolv_params = self.params.eigensolver.copy()
-        name = eigsolv_params.pop('name', 'dav')
-        if name == 'dav':
-            return Davidson(
-                self.nbands,
-                self.wf_desc,
-                self.communicators['b'],
-                hamiltonian.create_preconditioner,
-                converge_bands=self.params.convergence.get('bands',
-                                                           'occupied'),
-                **eigsolv_params)
-        from gpaw.directmin.etdm_fdpw import FDPWETDM
-        return ETDMPWFD(self.setups,
-                        self.communicators['w'],
-                        self.atoms,
-                        FDPWETDM(**eigsolv_params))
+        from gpaw.dft import DefaultEigensolver
+        es = self.params.eigensolver
+        if isinstance(es, DefaultEigensolver):
+            es = es.from_param({'name': 'davidson', **es.params})
+        return es.build(
+            self.nbands,
+            self.wf_desc,
+            self.communicators['b'],
+            hamiltonian,
+            self.params.convergence.get('bands', 'occupied'),
+            self.setups,
+            self.atoms)
 
     def read_ibz_wave_functions(self, reader):
         kpt_comm, band_comm, domain_comm = (self.communicators[x]
                                             for x in 'kbd')
 
         def create_wfs(spin: int, q: int, k: int, kpt_c, weight: float):
-            psit_nG = SimpleNamespace(
+            psit_nG = XArrayWithNoData(
                 comm=band_comm,
                 dims=(self.nbands,),
                 desc=self.wf_desc.new(kpt=kpt_c),
-                data=None,
-                xp=np)
+                xp=self.xp)
             wfs = PWFDWaveFunctions(
                 spin=spin,
                 q=q,
@@ -66,9 +60,8 @@ class PWFDDFTComponentsBuilder(DFTComponentsBuilder):
 
             return wfs
 
-        ibzwfs = PWFDIBZWaveFunction.create(
+        ibzwfs = PWFDIBZWaveFunctions.create(
             ibz=self.ibz,
-            nelectrons=self.nelectrons,
             ncomponents=self.ncomponents,
             create_wfs_func=create_wfs,
             kpt_comm=self.communicators['k'],
@@ -80,27 +73,34 @@ class PWFDDFTComponentsBuilder(DFTComponentsBuilder):
 
         return ibzwfs
 
-    def create_ibz_wave_functions(self, basis, potential, *, log):
+    def create_ibz_wave_functions(self, basis, potential):
         from gpaw.new.lcao.builder import create_lcao_ibzwfs
 
         if self.params.random:
-            return self.create_random_ibz_wave_functions(log)
+            return self.create_random_ibz_wave_functions()
 
         # sl_default = self.params.parallel['sl_default']
         # sl_lcao = self.params.parallel['sl_lcao'] or sl_default
 
+        lcao_dtype = complex if \
+            np.issubdtype(self.dtype, np.complexfloating) else float
+
         lcaonbands = min(self.nbands,
                          basis.Mmax * (2 if self.ncomponents == 4 else 1))
         lcao_ibzwfs, _ = create_lcao_ibzwfs(
             basis,
             self.ibz, self.communicators, self.setups,
-            self.relpos_ac, self.grid, self.dtype,
+            self.relpos_ac, self.grid, lcao_dtype,
             lcaonbands, self.ncomponents, self.atomdist, self.nelectrons)
 
+        self.log('\nDiagonalizing LCAO Hamiltonian', flush=True)
+
         hamiltonian = LCAOHamiltonian(basis)
         LCAOEigensolver(basis).iterate(
             lcao_ibzwfs, None, potential, hamiltonian)
 
+        self.log('Converting LCAO to grid', flush=True)
+
         def create_wfs(spin, q, k, kpt_c, weight):
             lcaowfs = lcao_ibzwfs.wfs_qs[q][spin]
             assert lcaowfs.spin == spin
@@ -113,11 +113,10 @@ class PWFDDFTComponentsBuilder(DFTComponentsBuilder):
             mynbands = len(psit_nX.data)
             eig_n = np.empty(self.nbands)
             eig_n[:lcaonbands] = lcaowfs._eig_n
-            eig_n[lcaonbands:] = 1e10
+            eig_n[lcaonbands:] = 100.0  # set high value for random wfs.
             if mylcaonbands < mynbands:
                 psit_nX[mylcaonbands:].randomize(
                     seed=self.communicators['w'].rank)
-
             wfs = PWFDWaveFunctions(
                 psit_nX=psit_nX,
                 spin=spin,
@@ -132,17 +131,16 @@ class PWFDDFTComponentsBuilder(DFTComponentsBuilder):
             wfs._eig_n = eig_n
             return wfs
 
-        return PWFDIBZWaveFunction.create(
+        return PWFDIBZWaveFunctions.create(
             ibz=self.ibz,
-            nelectrons=self.nelectrons,
             ncomponents=self.ncomponents,
             create_wfs_func=create_wfs,
             kpt_comm=self.communicators['k'],
             kpt_band_comm=self.communicators['D'],
             comm=self.communicators['w'])
 
-    def create_random_ibz_wave_functions(self, log):
-        log('Initializing wave functions with random numbers')
+    def create_random_ibz_wave_functions(self):
+        self.log('Initializing wave functions with random numbers')
 
         def create_wfs(spin, q, k, kpt_c, weight):
             desc = self.wf_desc.new(kpt=kpt_c)
@@ -166,9 +164,8 @@ class PWFDDFTComponentsBuilder(DFTComponentsBuilder):
 
             return wfs
 
-        return PWFDIBZWaveFunction.create(
+        return PWFDIBZWaveFunctions.create(
             ibz=self.ibz,
-            nelectrons=self.nelectrons,
             ncomponents=self.ncomponents,
             create_wfs_func=create_wfs,
             kpt_comm=self.communicators['k'],
