@@ -17,7 +17,8 @@ from gpaw.new import prod, trace, zips
 from gpaw.new.potential import Potential
 from gpaw.new.wave_functions import WaveFunctions
 from gpaw.setup import Setups
-from gpaw.typing import Array2D, Array3D, ArrayND, Vector
+from gpaw.typing import Array2D, Array3D, Vector
+from gpaw.utilities import as_real_dtype
 
 
 class PWFDWaveFunctions(WaveFunctions, XP):
@@ -33,7 +34,7 @@ class PWFDWaveFunctions(WaveFunctions, XP):
                  weight: float = 1.0,
                  ncomponents: int = 1,
                  qspiral_v: Vector | None = None):
-        assert isinstance(atomdist, AtomDistribution)
+        # assert isinstance(atomdist, AtomDistribution)
         self.psit_nX = psit_nX
         nbands = psit_nX.dims[0]
         super().__init__(setups=setups,
@@ -177,7 +178,8 @@ class PWFDWaveFunctions(WaveFunctions, XP):
         occ_n = self.weight * self.spin_degeneracy * self.myocc_n
         self.psit_nX.add_ked(occ_n, taut_sR[self.spin])
 
-    def orthonormalize(self, work_array_nX: ArrayND = None):
+    @trace
+    def orthonormalize(self, psit2_nX):
         r"""Orthonormalize wave functions.
 
         Computes the overlap matrix:::
@@ -210,10 +212,12 @@ class PWFDWaveFunctions(WaveFunctions, XP):
         P_ani = self.P_ani
 
         P2_ani = P_ani.new()
-        psit2_nX = psit_nX.new(data=work_array_nX)
-
-        dS_aii = self.setups.get_overlap_corrections(P_ani.layout.atomdist,
-                                                     self.xp)
+        if psit2_nX is None:
+            psit2_nX = psit_nX.new()
+        dS_aii = self.setups.get_overlap_corrections(
+            P_ani.layout.atomdist,
+            self.xp,
+            dtype=as_real_dtype(P_ani.data.dtype))
 
         # We are actually calculating S^*:
         S = psit_nX.matrix_elements(psit_nX, domain_sum=False, cc=True)
@@ -236,10 +240,12 @@ class PWFDWaveFunctions(WaveFunctions, XP):
     def subspace_diagonalize(self,
                              Ht,
                              dH,
-                             work_array: ArrayND = None,
-                             Htpsit_nX=None,
+                             psit2_nX,
+                             data_buffer=None,
                              scalapack_parameters=(None, 1, 1, None)):
         """
+        If data_buffer is None, psit2_nX will be used as a buffer
+        for the wave functions.
 
         Ht(in, out):::
 
@@ -252,14 +258,13 @@ class PWFDWaveFunctions(WaveFunctions, XP):
           <𝜓 |p> ΔH  <p |𝜓>
             m  i   ij  j  n
         """
-        self.orthonormalize(work_array)
+        self.orthonormalize(psit2_nX)
         psit_nX = self.psit_nX
         P_ani = self.P_ani
-        psit2_nX = psit_nX.new(data=work_array)
         P2_ani = P_ani.new()
         domain_comm = psit_nX.desc.comm
 
-        Ht = partial(Ht, out=psit2_nX, spin=self.spin)
+        Ht = partial(Ht, out=psit2_nX, spin=self.spin, calculate_energy=True)
         H = psit_nX.matrix_elements(psit_nX,
                                     function=Ht,
                                     domain_sum=False,
@@ -268,12 +273,13 @@ class PWFDWaveFunctions(WaveFunctions, XP):
         P_ani.matrix.multiply(P2_ani, opb='C', symmetric=True,
                               out=H, beta=1.0)
         domain_comm.sum(H.data, 0)
-
         if domain_comm.rank == 0:
             slcomm, r, c, b = scalapack_parameters
             if r == c == 1:
                 slcomm = None
+            # print(H.data)
             self._eig_n = as_np(H.eigh(scalapack=(slcomm, r, c, b)))
+            # print(self._eig_n)
             H.complex_conjugate()
             # H.data[n, :] now contains the nth eigenvector and eps_n[n]
             # the nth eigenvalue
@@ -282,13 +288,16 @@ class PWFDWaveFunctions(WaveFunctions, XP):
 
         domain_comm.broadcast(H.data, 0)
         domain_comm.broadcast(self._eig_n, 0)
-        if Htpsit_nX is not None:
-            H.multiply(psit2_nX, out=Htpsit_nX)
-
-        H.multiply(psit_nX, out=psit2_nX)
-        psit_nX.data[:] = psit2_nX.data
-        H.multiply(P_ani, out=P2_ani)
-        P_ani.data[:] = P2_ani.data
+        if data_buffer is None:
+            H.multiply(psit_nX, out=psit2_nX)
+            psit_nX.data[:] = psit2_nX.data
+            H.multiply(P_ani, out=P2_ani)
+            P_ani.data[:] = P2_ani.data
+        else:
+            H.multiply(psit_nX, out=psit_nX, data_buffer=data_buffer)
+            H.multiply(psit2_nX, out=psit2_nX, data_buffer=data_buffer)
+            H.multiply(P_ani, out=P2_ani)
+            P_ani.data[:] = P2_ani.data
 
     def force_contribution(self,
                            potential: Potential,
@@ -339,8 +348,7 @@ class PWFDWaveFunctions(WaveFunctions, XP):
     def collect(self,
                 n1: int = 0,
                 n2: int = 0) -> PWFDWaveFunctions | None:
-        """Collect range of bands to master of band and domain
-        communicators."""
+        """Collect range of bands to master of band and domain comms."""
         # Also collect projections instead of recomputing XXX
         n2 = n2 if n2 > 0 else self.nbands + n2
         spinors = (2,) if self.ncomponents == 4 else ()
@@ -375,10 +383,12 @@ class PWFDWaveFunctions(WaveFunctions, XP):
                 ba = 0
                 na = nb
             if domain_comm.rank == 0:
-                return PWFDWaveFunctions.from_wfs(
+                wfs = PWFDWaveFunctions.from_wfs(
                     self,
                     psit_nX,
                     atomdist=self.atomdist.gather())
+                wfs._eig_n = self.eig_n[n1:n2]
+                return wfs
         else:
             rank = band_comm.rank
             ranka, ba = max((rank1, b1), (rank, 0))
@@ -417,10 +427,10 @@ class PWFDWaveFunctions(WaveFunctions, XP):
 
         :::
 
-           _    /  _ ~ ~ _   ---  a  a  _a
-           μ  = | dr 𝜓 𝜓 r + >   P  P  Δμ
-            mn  /     m n    ---  im jn  ij
-                             aij
+           _    / ~ ~ _ _   ---  a  a  _a
+           μ  = | 𝜓 𝜓 rdr + >   P  P  Δμ
+            mn  /  m n      ---  im jn  ij
+                            aij
 
         Returns
         -------
