@@ -7,16 +7,19 @@ from types import SimpleNamespace
 from typing import Any, Callable
 
 import numpy as np
+
+from gpaw import KohnShamConvergenceError
 from gpaw.convergence_criteria import (Criterion, check_convergence,
                                        dict2criterion)
+from gpaw.new.energies import DFTEnergies
+from gpaw.new.logger import indent
+from gpaw.new.ibzwfs import IBZWaveFunctions
 from gpaw.scf import write_iteration
 from gpaw.typing import Array2D
-from gpaw.new.logger import indent
-from gpaw import KohnShamConvergenceError
 
 
 class TooFewBandsError(KohnShamConvergenceError):
-    """Not enough bands for CBM+x convergence cfriterium."""
+    """Not enough bands for CBM+x convergence criterion."""
 
 
 class SCFLoop:
@@ -48,9 +51,10 @@ class SCFLoop:
                 f'occupation numbers:\n{indent(self.occ_calc)}\n')
 
     def iterate(self,
-                ibzwfs,
+                ibzwfs: IBZWaveFunctions,
                 density,
                 potential,
+                energies: DFTEnergies,
                 pot_calc,
                 *,
                 maxiter=None,
@@ -59,11 +63,6 @@ class SCFLoop:
         cc = self.convergence
         maxiter = maxiter or self.maxiter
 
-        self.eigensolver.initialize_etdm(
-            ibzwfs, density, potential,
-            pot_calc, self.occ_calc,
-            self.hamiltonian, self.mixer, log)
-
         if log:
             log('convergence criteria:')
             for criterion in cc.values():
@@ -81,18 +80,25 @@ class SCFLoop:
             dens_error = 0.0
 
         for self.niter in itertools.count(start=1):
-            wfs_error = self.eigensolver.iterate(
-                ibzwfs, density, potential, self.hamiltonian)
-            ibzwfs.calculate_occs(
+            eig_error, wfs_error, energies = self.eigensolver.iterate(
+                ibzwfs, density, potential,
+                self.hamiltonian, pot_calc, energies)
+            nelectrons = (density.nvalence - density.charge +
+                          pot_calc.environment.charge)
+            e_band, e_entropy, e_extrapolation = ibzwfs.calculate_occs(
                 self.occ_calc,
+                nelectrons,
                 fix_fermi_level=self.fix_fermi_level)
-            if self.eigensolver.direct:
-                ibzwfs.energies['band'] = 0.0
+
+            energies.set(**pot_calc.xc.energies,
+                         band=e_band,
+                         entropy=e_entropy,
+                         extrapolation=e_extrapolation)
 
             ctx = SCFContext(
-                log, self.niter,
+                log, self.niter, energies,
                 ibzwfs, density, potential,
-                wfs_error, dens_error,
+                wfs_error, dens_error, eig_error,
                 self.comm, calculate_forces,
                 pot_calc, self.update_density_and_potential)
 
@@ -104,8 +110,13 @@ class SCFLoop:
 
             if log:
                 write_iteration(cc, converged_items, entries, ctx, log)
+
             if converged:
-                break
+                converged = pot_calc.environment.post_scf_convergence(
+                    ibzwfs, nelectrons, self.occ_calc, self.mixer, log)
+                if converged:
+                    break
+
             if self.niter == maxiter:
                 if wfs_error < inf:
                     raise KohnShamConvergenceError
@@ -114,15 +125,8 @@ class SCFLoop:
             if self.update_density_and_potential:
                 density.update(ibzwfs, ked=pot_calc.xc.type == 'MGGA')
                 dens_error = self.mixer.mix(density)
-                new_potential, _ = pot_calc.calculate(
+                potential, energies, _ = pot_calc.calculate(
                     density, ibzwfs, potential.vHt_x)
-                # Because of the way direct-optimization works at the moment,
-                # we need to update the potential in-place!
-                potential.update_from(new_potential)
-                if self.eigensolver.direct:
-                    ekin = ibzwfs.calculate_kinetic_energy(
-                        self.hamiltonian, density)
-                    potential.energies['kinetic'] = ekin
 
         self.eigensolver.postprocess(
             ibzwfs, density, potential, self.hamiltonian)
@@ -132,28 +136,28 @@ class SCFContext:
     def __init__(self,
                  log,
                  niter: int,
+                 energies: DFTEnergies,
                  ibzwfs,
                  density,
                  potential,
                  wfs_error: float,
                  dens_error: float,
+                 eig_error: float,
                  comm,
                  calculate_forces: Callable[[], Array2D],
                  pot_calc,
                  update_density_and_potential):
         self.log = log
         self.niter = niter
+        self.energies = energies
         self.ibzwfs = ibzwfs
         self.density = density
         self.potential = potential
-        energy = np.array([sum(e
-                               for name, e in potential.energies.items()
-                               if name != 'stress') +
-                           sum(ibzwfs.energies.values())])
-        comm.broadcast(energy, 0)
-        self.ham = SimpleNamespace(e_total_extrapolated=energy[0],
+        energy = energies.total_extrapolated
+        self.ham = SimpleNamespace(e_total_extrapolated=energy,
                                    get_workfunctions=self._get_workfunctions)
-        self.wfs = SimpleNamespace(nvalence=ibzwfs.nelectrons,
+        self.wfs = SimpleNamespace(nvalence=density.nvalence +
+                                   pot_calc.environment.charge,
                                    world=comm,
                                    eigensolver=SimpleNamespace(
                                        error=wfs_error),
@@ -163,6 +167,7 @@ class SCFContext:
             calculate_magnetic_moments=density.calculate_magnetic_moments,
             fixed=not update_density_and_potential,
             error=dens_error)
+        self.eig_error = eig_error
         self.calculate_forces = calculate_forces
         self.poisson_solver = pot_calc.poisson_solver
 
