@@ -70,16 +70,17 @@ def create_poisson_solver(name='fast', **kwargs):
         raise ValueError('Unknown poisson solver: %s' % name)
 
 
-def PoissonSolver(name='fast', dipolelayer=None, **kwargs):
+def PoissonSolver(name='fast', dipolelayer=None, zero_vacuum=False, **kwargs):
     p = create_poisson_solver(name=name, **kwargs)
     if dipolelayer is not None:
-        p = DipoleCorrection(p, dipolelayer)
+        p = DipoleCorrection(p, dipolelayer, zero_vacuum=zero_vacuum)
     return p
 
 
-def FDPoissonSolverWrapper(dipolelayer=None, **kwargs):
+def FDPoissonSolverWrapper(dipolelayer=None, zero_vacuum=False, **kwargs):
     if dipolelayer is not None:
-        return DipoleCorrection(FDPoissonSolver(**kwargs), dipolelayer)
+        return DipoleCorrection(FDPoissonSolver(**kwargs), dipolelayer,
+                                zero_vacuum=zero_vacuum)
     return FDPoissonSolver(**kwargs)
 
 
@@ -106,6 +107,12 @@ class _PoissonSolver:
     def estimate_memory(self, mem):
         raise NotImplementedError()
 
+    def build(self, grid, xp):
+        from gpaw.new.poisson import PoissonSolverWrapper
+        self.xp = xp
+        self.set_grid_descriptor(grid._gd)
+        return PoissonSolverWrapper(self)
+
 
 class BasePoissonSolver(_PoissonSolver):
     def __init__(self, *, remove_moment=None,
