@@ -150,13 +150,13 @@ class PlasmaFrequencyIntegrand(Integrand):
         K0 = kptpair_factory.gs.kpoints.kptfinder.find(k_c)  # XXX
 
         kpt1 = kptpair_factory.get_k_point(point.spin, K0, n1, n2)
-        n_n = range(n1, n2)
+        n_n = np.arange(n1, n2)
 
         pair_calc = kptpair_factory.pair_calculator(
             blockcomm=self._drude.blockcomm)
         vel_nv = pair_calc.intraband_pair_density(kpt1, n_n)
 
-        if self._drude.integrationmode is None:
+        if self._drude.integrationmode == 'point integration':
             f_n = kpt1.f_n
             width = self._drude.gs.get_occupations_width()
             if width > 1e-15:
