@@ -5,7 +5,7 @@ from dataclasses import dataclass
 import numpy as np
 
 from gpaw.response import ResponseGroundStateAdapter, ResponseContext
-from gpaw.response.pair_functions import SingleQPWDescriptor
+from gpaw.response.qpd import SingleQPWDescriptor
 from gpaw.response.localft import LocalFTCalculator
 from gpaw.response.fxc_kernels import AdiabaticFXCCalculator
 
