@@ -32,14 +32,11 @@ class KPointPair:
         self.kpt2 = kpt2
         self.Q_G = Q_G
 
-    def get_transition_energies(self, n_n, m_m):
+    def get_transition_energies(self):
         """Return the energy difference for specified bands."""
-        n_n = np.array(n_n)
-        m_m = np.array(m_m)
         kpt1 = self.kpt1
         kpt2 = self.kpt2
-        deps_nm = (kpt1.eps_n[n_n - self.kpt1.n1][:, np.newaxis] -
-                   kpt2.eps_n[m_m - self.kpt2.n1])
+        deps_nm = kpt1.eps_n[:, np.newaxis] - kpt2.eps_n
         return deps_nm
 
     def get_occupation_differences(self):
@@ -116,7 +113,8 @@ class KPointPairFactory:
                       ut_nR, eps_n, f_n, P_ani, k_c)
 
     @timer('Get kpoint pair')
-    def get_kpoint_pair(self, qpd, s, K, n1, n2, m1, m2, blockcomm=None):
+    def get_kpoint_pair(self, qpd, s, K, n1, n2, m1, m2,
+                        blockcomm=None, flipspin=False):
         assert m1 <= m2
         assert n1 <= n2
 
@@ -125,10 +123,12 @@ class KPointPairFactory:
         k_c = self.gs.kd.bzk_kc[K]
         K1 = kptfinder.find(k_c)
         K2 = kptfinder.find(k_c + qpd.q_c)
+        s1 = s
+        s2 = (s + flipspin) % 2
 
         with self.context.timer('get k-points'):
-            kpt1 = self.get_k_point(s, K1, n1, n2)
-            kpt2 = self.get_k_point(s, K2, m1, m2, blockcomm=blockcomm)
+            kpt1 = self.get_k_point(s1, K1, n1, n2)
+            kpt2 = self.get_k_point(s2, K2, m1, m2, blockcomm=blockcomm)
 
         with self.context.timer('fft indices'):
             Q_G = phase_shifted_fft_indices(kpt1.k_c, kpt2.k_c, qpd)
