@@ -249,7 +249,6 @@ class WCalculator(WBaseCalculator):
     def calculate_W_wGG(self, chi0, fxc_mode='GW',
                         only_correlation=False):
         """In-place calculation of the screened interaction."""
-        chi0_wGG = chi0.body.copy_array_with_distribution('wGG')
         dfc = DielectricFunctionCalculator(chi0, self.coulomb,
                                            self.xckernel, fxc_mode)
         self.context.timer.start('Dyson eq.')
@@ -263,11 +262,9 @@ class WCalculator(WBaseCalculator):
             sqrtV_G = dfc.sqrtV_G
             V0, sqrtV0 = self.get_V0sqrtV0(chi0)
 
-        for iw, chi0_GG in enumerate(chi0_wGG):
-            # Note, at q=0 get_epsinv_GG modifies chi0_GG
-            einv_GG = dfc.get_epsinv_GG(chi0_GG, iw)
-            # Renaming the chi0_GG buffer since it will be used to store W
-            W_GG = chi0_GG
+        einv_wGG = dfc.get_epsinv_wGG(only_correlation=False)
+        W_wGG = np.empty_like(einv_wGG)
+        for iw, (einv_GG, W_GG) in enumerate(zip(einv_wGG, W_wGG)):
             # If only_correlation = True function spits out
             # W^c = sqrt(V)(epsinv - delta_GG')sqrt(V). However, full epsinv
             # is still needed for q0_corrector.
@@ -275,7 +272,7 @@ class WCalculator(WBaseCalculator):
             W_GG[:] = einvt_GG * (sqrtV_G *
                                   sqrtV_G[:, np.newaxis])
             if self.q0_corrector is not None and chi0.optical_limit:
-                W = dfc.wblocks1d.a + iw
+                W = dfc.wblocks.a + iw
                 self.q0_corrector.add_q0_correction(chi0.qpd, W_GG,
                                                     einv_GG,
                                                     chi0.chi0_WxvG[W],
@@ -287,7 +284,7 @@ class WCalculator(WBaseCalculator):
                                             V0, sqrtV0, dfc.sqrtV_G)
 
         self.context.timer.stop('Dyson eq.')
-        return chi0_wGG
+        return W_wGG
 
     def dyson_and_W_new(self, iq, q_c, chi0, ecut, coulomb):
         # assert not self.do_GW_too
