@@ -8,8 +8,8 @@ from ase.units import Hartree
 
 from gpaw.response.frequencies import ComplexFrequencyDescriptor
 from gpaw.response.pw_parallelization import Blocks1D
-from gpaw.response.pair_functions import (SingleQPWDescriptor, Chi,
-                                          get_pw_coordinates)
+from gpaw.response.pair_functions import Chi, get_pw_coordinates
+from gpaw.response.qpd import SingleQPWDescriptor
 from gpaw.response.chiks import ChiKSCalculator
 from gpaw.response.coulomb_kernels import NewCoulombKernel
 from gpaw.response.fxc_kernels import FXCKernel, AdiabaticFXCCalculator
