@@ -77,9 +77,11 @@ def create_setup(symbol, xc='LDA', lmax=0,
                                  'functional.  This calculation would use '
                                  'the %s functional.' % xc.get_setup_name())
         else:
-            setupdata = SetupData(symbol, xc.get_setup_name(),
-                                  type, True,
-                                  world=world)
+            setupdata = SetupData.find_and_read_path(symbol,
+                                                     xc.get_setup_name(),
+                                                     setuptype=type,
+                                                     world=world)
+
     if hasattr(setupdata, 'build'):
         # It is not so nice that we have hubbard_u floating around here.
         # For example, none of the other setup types are aware
@@ -615,8 +617,6 @@ class LeanSetup(BaseSetup):
 
         # XAS stuff
         self.phicorehole_g = s.phicorehole_g  # should be optional
-        if s.phicorehole_g is not None:
-            self.A_ci = s.A_ci  # oscillator strengths
 
         # Required to get all electron density
         self.rgd = s.rgd
@@ -832,11 +832,6 @@ class Setup(BaseSetup):
 
         self.fcorehole = data.fcorehole
         self.lcorehole = data.lcorehole
-        if data.phicorehole_g is not None:
-            if self.lcorehole == 0:
-                self.calculate_oscillator_strengths(phi_jg)
-            else:
-                self.A_ci = None
 
         # Construct splines:
         self.vbar = rgd.spline(vbar_g, rcutfilter)
@@ -1223,33 +1218,14 @@ class Setup(BaseSetup):
         basis = PartialWaveBasis(self.symbol, basis_functions_J, n_J)
         return basis
 
-    def calculate_oscillator_strengths(self, phi_jg):
-        # XXX implement oscillator strengths for lcorehole != 0
-        assert self.lcorehole == 0
-        self.A_ci = np.zeros((3, self.ni))
-        nj = len(phi_jg)
-        i = 0
-        for j in range(nj):
-            l = self.l_j[j]
-            if l == 1:
-                a = self.rgd.integrate(phi_jg[j] * self.data.phicorehole_g,
-                                       n=1) / (4 * pi)
-
-                for m in range(3):
-                    c = (m + 1) % 3
-                    self.A_ci[c, i] = a
-                    i += 1
-            else:
-                i += 2 * l + 1
-        assert i == self.ni
-
 
 class PartialWaveBasis(Basis):  # yuckkk
     def __init__(self, symbol, phit_J, n_J):
-        Basis.__init__(self, symbol, 'partial-waves', readxml=False)
         self._basis_functions_J = phit_J
-        self.bf_j = [BasisFunction(n, phit.get_angular_momentum_number())
-                     for n, phit in zip(n_J, phit_J)]
+        super().__init__(
+            symbol, 'partial-waves',
+            bf_j=[BasisFunction(n, phit.get_angular_momentum_number())
+                  for n, phit in zip(n_J, phit_J)])
 
     def tosplines(self):
         return self._basis_functions_J
@@ -1339,7 +1315,7 @@ class Setups(list):
                 # (meaning we load the basis set now from a file) or an actual
                 # pre-created Basis object (meaning we just pass it along)
                 if isinstance(basis, str):
-                    basis = Basis(symbol, basis, world=world)
+                    basis = Basis.find(symbol, basis, world=world)
                 setup = create_setup(symbol, xc, 2, type,
                                      basis, setupdata=setupdata,
                                      filter=filter, world=world,
@@ -1453,7 +1429,7 @@ class Setups(list):
             atomdist=atomdist,
             xp=xp)
 
-    def create_compensation_charges(self, domain, positions, atomdist,
+    def create_compensation_charges(self, domain, positions, atomdist=None,
                                     xp=np):
         if self.backwards_compatible and hasattr(domain, 'ecut'):
             integral = None
@@ -1465,12 +1441,13 @@ class Setups(list):
             integrals=integral,
             xp=xp)
 
-    def get_overlap_corrections(self, atomdist, xp):
+    def get_overlap_corrections(self, atomdist, xp, dtype=np.float64):
         if atomdist is getattr(self, '_atomdist', None):
-            return self.dS_aii
+            if self.dS_aii.data.dtype == dtype:
+                return self.dS_aii
         self._atomdist = atomdist
         dS_aii = AtomArraysLayout([setup.dO_ii.shape for setup in self],
-                                  atomdist=atomdist).empty()
+                                  atomdist=atomdist, dtype=dtype).empty()
         for a, dS_ii in dS_aii.items():
             dS_ii[:] = self[a].dO_ii
         self.dS_aii = dS_aii.to_xp(xp)
@@ -1539,7 +1516,7 @@ def types2atomtypes(symbols, types, default):
     if isinstance(types, str):
         return [types] * natoms
 
-    # If present, None will map to the default type,
+    # If present, 'default' will map to the default type,
     # else use the input default
     type_a = [types.get('default', default)] * natoms
 
