@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import hashlib
 import os
 import re
@@ -31,12 +33,6 @@ class SetupData:
         self.zero_reference = zero_reference
         self.generator_version = generator_version
 
-        # Default filename if this setup is written
-        if name is None or name == 'paw':
-            self.stdfilename = f'{symbol}.{self.setupname}'
-        else:
-            self.stdfilename = f'{symbol}.{name}.{self.setupname}'
-
         self.filename = None  # full path if this setup was loaded from file
         self.fingerprint = None  # hash value of file data if applicable
 
@@ -85,7 +81,8 @@ class SetupData:
         self.e_total = 0.0
         self.e_kinetic_core = 0.0
 
-        # Generator may store description of setup in this string
+        # Generator may store description of setup in these
+        self.type = None
         self.generatorattrs = []
         self.generatordata = ''
 
@@ -118,6 +115,34 @@ class SetupData:
         if readxml:
             self.read_xml(world=world)
 
+    @classmethod
+    def find_and_read_path(cls, symbol, xctype,
+                           setuptype='paw', world=None):
+
+        setupdata = SetupData(symbol, xctype,
+                              name=setuptype,
+                              readxml=False,
+                              world=world)
+
+        setupdata.filename, source = search_for_file(setupdata.stdfilename,
+                                                     world=world)
+        PAWXMLParser(setupdata).parse(source=source, world=world)
+
+        nj = len(setupdata.l_j)
+        setupdata.e_kin_jj.shape = (nj, nj)
+
+        return setupdata
+
+    @property
+    def stdfilename(self):
+        """Default filename if this setup is written."""
+        assert self.symbol is not None
+        assert self.setupname is not None
+        if self.name is None or self.name == 'paw':
+            return f'{self.symbol}.{self.setupname}'
+        else:
+            return f'{self.symbol}.{self.name}.{self.setupname}'
+
     def __repr__(self):
         return ('{0}({symbol!r}, {setupname!r}, name={name!r}, '
                 'generator_version={generator_version!r}, ...)'
@@ -133,6 +158,7 @@ class SetupData:
         self.phit_jg.append(phit_g)
         self.pt_jg.append(pt_g)
 
+    # XXX delete me
     def read_xml(self, source=None, world=None):
         PAWXMLParser(self).parse(source=source, world=world)
         nj = len(self.l_j)
@@ -395,6 +421,16 @@ class SetupData:
         return setup
 
 
+def read_maybe_unzipping(path: Path | str) -> bytes:
+    import gzip
+    if Path(path).suffix == '.gz':
+        with gzip.open(path) as fd:
+            return fd.read()
+
+    with open(path, 'rb') as fd:
+        return fd.read()
+
+
 def search_for_file(name: str, world=None) -> Tuple[str, bytes]:
     """Traverse gpaw setup paths to find file.
 
@@ -413,12 +449,7 @@ def search_for_file(name: str, world=None) -> Tuple[str, bytes]:
                 # last/newest of the results (used with SG15).  (User must
                 # instantiate (UPF)SetupData directly to override.)
                 filename = max(filenames)
-                import gzip
-                if filename.endswith('.gz'):
-                    with gzip.open(filename) as fd:
-                        source = fd.read()
-                else:
-                    source = Path(filename).read_bytes()
+                source = read_maybe_unzipping(filename)
                 break
 
     if world is not None:
@@ -476,6 +507,8 @@ class PAWXMLParser(xml.sax.handler.ContentHandler):
         if name == 'atom':
             Z = float(attrs['Z'])
             setup.Z = Z
+            assert setup.symbol is None or setup.symbol == attrs['symbol']
+            setup.symbol = attrs['symbol']
             assert setup.Z == Z
             setup.Nc = float(attrs['core'])
             Nv = float(attrs['valence'])
@@ -490,6 +523,8 @@ class PAWXMLParser(xml.sax.handler.ContentHandler):
                     setup.orbital_free = True
                 else:
                     assert attrs['type'] == 'GGA'
+            assert setup.setupname is None or setup.setupname == setup.xcname
+            setup.setupname = setup.xcname
         elif name == 'ae_energy':
             setup.e_total = float(attrs['total'])
             setup.e_kinetic = float(attrs['kinetic'])
@@ -558,7 +593,13 @@ class PAWXMLParser(xml.sax.handler.ContentHandler):
         elif name == 'core_hole_state':
             setup.has_corehole = True
             setup.fcorehole = float(attrs['removed'])
-            setup.lcorehole = 'spdf'.find(attrs['state'][1])
+            full_state = attrs['state']
+            state_l = full_state.lstrip('0123456789')
+            assert state_l
+            state_n = full_state[:-len(state_l)]
+            assert state_n
+            setup.ncorehole = int(state_n)
+            setup.lcorehole = 'spdf'.find(state_l)
             setup.core_hole_e = float(attrs['eig'])
             setup.core_hole_e_kin = float(attrs['ekin'])
             self.data = []
