@@ -23,18 +23,21 @@ import numpy as np
 
 from math import pi
 from collections import defaultdict
-from gpaw.cgpaw import spherical_harmonics
-from gpaw import debug
+from gpaw import debug, GPAW_NO_C_EXTENSION
 
 __all__ = ['Y', 'YL', 'nablarlYL', 'Yl']
 
-if debug:
-    def Yl(l: int, R_v: np.ndarray, rlY_m: np.ndarray) -> None:
-        assert R_v.dtype == float and R_v.shape == (3,)
-        assert rlY_m.dtype == float and rlY_m.shape == (2 * l + 1,)
-        spherical_harmonics(l, R_v, rlY_m)
+if GPAW_NO_C_EXTENSION:
+    Yl = None
 else:
-    Yl = spherical_harmonics
+    from gpaw.cgpaw import spherical_harmonics
+    if debug:
+        def Yl(l: int, R_v: np.ndarray, rlY_m: np.ndarray) -> None:
+            assert R_v.dtype == float and R_v.shape == (3,)
+            assert rlY_m.dtype == float and rlY_m.shape == (2 * l + 1,)
+            spherical_harmonics(l, R_v, rlY_m)
+    else:
+        Yl = spherical_harmonics
 
 names = [['1'],
          ['y', 'z', 'x'],
