@@ -1,6 +1,7 @@
 import pytest
 from ase import Atoms
 from ase.io import read
+from ase.units import Ha
 
 from gpaw import GPAW
 
@@ -15,6 +16,11 @@ def test_no_cell():
 @pytest.mark.parametrize('name', ['h2_pw', 'bcc_li_lcao'])
 def test_read_txt(in_tmp_dir, gpw_files, name):
     gpw = gpw_files[name]
-    e0 = GPAW(gpw).get_atoms().get_potential_energy()
-    e = read(gpw.with_suffix('.txt')).get_potential_energy()
+    calc = GPAW(gpw)
+    e0 = calc.get_atoms().get_potential_energy()
+    atoms = read(gpw.with_suffix('.txt'))
+    e = atoms.get_potential_energy()
     assert e == pytest.approx(e0)
+    if not calc.old:
+        assert atoms.calc.energy_contributions['kinetic'] == pytest.approx(
+            calc.dft.energies.kinetic * Ha)
