@@ -19,8 +19,7 @@ for name in molecule_names + atom_names:
                       xc='PBE',
                       txt=name + '.txt')
     if name in ['Na2', 'NaCl', 'NO', 'ClO', 'Cl', 'Si']:
-        atoms.calc = atoms.calc.new(eigensolver='dav',
-                                    mixer=Mixer(0.05, 2))
+        atoms.calc = atoms.calc.new(mixer=Mixer(0.05, 2))
     atoms.get_forces()
     c.write(atoms, id=id, name=name, relaxed=False)
     if len(atoms) > 1:
