@@ -2,27 +2,19 @@ import pytest
 
 from gpaw import GPAW
 from gpaw.directmin.derivatives import Davidson
-from ase import Atoms
 
 
+@pytest.mark.old_gpaw_only
 @pytest.mark.do
-def test_break_instability_lcao(in_tmp_dir):
-    atoms = Atoms('H2', positions=[(0, 0, 0), (0, 0, 2.0)])
-    atoms.center(vacuum=2.0)
-    atoms.set_pbc(False)
-
-    calc = GPAW(xc='PBE',
-                mode='lcao',
-                h=0.24,
-                basis='sz(dzp)',
-                spinpol=True,
-                eigensolver='etdm-lcao',
-                convergence={'density': 1.0e-2,
-                             'eigenstates': 1.0e-2},
-                occupations={'name': 'fixed-uniform'},
-                mixer={'backend': 'no-mixing'},
-                nbands='nao',
-                symmetry='off')
+def test_break_instability_lcao(in_tmp_dir, gpw_files):
+    calc = GPAW(gpw_files['h2_break_ilcao'])
+    # XXX(rg): Remove hack after tchem-gl-13
+    calc.set_positions()
+    calc.wfs.eigensolver.initialize_dm_helper(calc.wfs,
+                                              calc.hamiltonian,
+                                              calc.density,
+                                              calc.log)
+    atoms = calc.atoms
     atoms.calc = calc
     e_symm = atoms.get_potential_energy()
     assert e_symm == pytest.approx(-2.035632, abs=1.0e-3)
