@@ -4,10 +4,16 @@ from pathlib import Path
 import numpy as np
 
 from ase import Atom, Atoms
+from ase.units import Bohr
 from ase.build import bulk, molecule
 from ase.lattice.compounds import L1_2
 from ase.lattice.hexagonal import Graphene
-from gpaw import Davidson, FermiDirac, GPAW, Mixer, PW
+from gpaw import Davidson, FermiDirac, GPAW, Mixer, PW, FD, LCAO
+from gpaw.directmin.etdm_fdpw import FDPWETDM
+from gpaw.directmin.etdm_lcao import LCAOETDM
+from gpaw.directmin.tools import excite
+from gpaw.directmin.derivatives import Davidson as SICDavidson
+from gpaw.mom import prepare_mom_calculation
 from gpaw.mpi import world, serial_comm
 from gpaw.new.ase_interface import GPAW as GPAWNew
 from gpaw.poisson import FDPoissonSolver, PoissonSolver
@@ -137,14 +143,19 @@ class GPWFiles(CachedFilesHandler):
         magmoms = None if calc_type == 'col' else [mm * easy_axis]
         soc = True if calc_type == 'ncolsoc' else False
 
-        Ni.calc = GPAWNew(mode={'name': 'pw', 'ecut': 280}, xc='LDA',
+        Ni.calc = GPAWNew(mode={'name': 'pw', 'ecut': 380},
+                          xc='LDA',
+                          nbands='200%',
                           kpts={'size': (4, 4, 4), 'gamma': True},
                           parallel={'domain': 1, 'band': 1},
                           mixer={'beta': 0.5},
                           symmetry=symmetry,
                           occupations={'name': 'fermi-dirac', 'width': 0.05},
-                          convergence={'density': 1e-4},
-                          magmoms=magmoms, soc=soc,
+                          convergence={'density': 1e-8,
+                                       'bands': 'CBM+10',
+                                       'eigenstates': 1e-12},
+                          magmoms=magmoms,
+                          soc=soc,
                           txt=self.folder / f'fcc_Ni_{calc_type}.txt')
         Ni.get_potential_energy()
         return Ni.calc
@@ -195,6 +206,711 @@ class GPWFiles(CachedFilesHandler):
         atoms.get_potential_energy()
         return atoms.calc
 
+    @gpwfile
+    def c2h4_do_fd(self):
+        atm = Atoms('CCHHHH',
+                    positions=[
+                        [-0.66874198, -0.00001714, -0.00001504],
+                        [0.66874210, 0.00001699, 0.00001504],
+                        [-1.24409879, 0.00000108, -0.93244784],
+                        [-1.24406253, 0.00000112, 0.93242153],
+                        [1.24406282, -0.93242148, 0.00000108],
+                        [1.24409838, 0.93244792, 0.00000112]
+                    ]
+                    )
+        atm.center(vacuum=4.0)
+        atm.set_pbc(False)
+        atm.calc = GPAW(mode=FD(), h=0.3,
+                        xc='PBE',
+                        occupations={'name': 'fixed-uniform'},
+                        eigensolver={'name': 'etdm-fdpw',
+                                     'converge_unocc': True},
+                        mixer={'backend': 'no-mixing'},
+                        spinpol=True,
+                        symmetry='off',
+                        nbands=-5,
+                        convergence={'eigenstates': 4.0e-6},
+                        )
+        atm.get_potential_energy()
+        return atm.calc
+
+    @gpwfile
+    def c2h4_do_pw(self):
+        atm = Atoms(
+            "CCHHHH",
+            positions=[
+                [-0.66874198, -0.00001714, -0.00001504],
+                [0.66874210, 0.00001699, 0.00001504],
+                [-1.24409879, 0.00000108, -0.93244784],
+                [-1.24406253, 0.00000112, 0.93242153],
+                [1.24406282, -0.93242148, 0.00000108],
+                [1.24409838, 0.93244792, 0.00000112],
+            ],
+        )
+        atm.center(vacuum=4.0)
+        atm.set_pbc(False)
+        atm.calc = GPAW(
+            mode=PW(300, force_complex_dtype=True),
+            xc="PBE",
+            occupations={"name": "fixed-uniform"},
+            eigensolver={"name": "etdm-fdpw", "converge_unocc": True},
+            mixer={"backend": "no-mixing"},
+            spinpol=True,
+            symmetry="off",
+            nbands=-5,
+            convergence={"eigenstates": 4.0e-6},
+        )
+        atm.get_potential_energy()
+        return atm.calc
+
+    def h3_maker(self, vacuum=2.0, pbc=True, **kwargs):
+        atoms = Atoms('H3', positions=[(0, 0, 0),
+                                       (0.59, 0, 0),
+                                       (1.1, 0, 0)])
+        atoms.center(vacuum=vacuum)
+        atoms.set_pbc(pbc)
+        return atoms
+
+    @gpwfile
+    def h3_do_num_pw_complex(self):
+        atoms = self.h3_maker()
+        calc = GPAW(mode=PW(300, force_complex_dtype=True),
+                    basis='sz(dzp)',
+                    h=0.3,
+                    spinpol=False,
+                    convergence={'energy': np.inf,
+                                 'eigenstates': np.inf,
+                                 'density': np.inf,
+                                 'minimum iterations': 1},
+                    eigensolver=FDPWETDM(converge_unocc=True),
+                    occupations={'name': 'fixed-uniform'},
+                    mixer={'backend': 'no-mixing'},
+                    nbands='nao',
+                    symmetry='off')
+        atoms.calc = calc
+        atoms.get_potential_energy()
+        return atoms.calc
+
+    @gpwfile
+    def h3_do_num_pw(self):
+        atoms = self.h3_maker()
+        calc = GPAW(mode=PW(300, force_complex_dtype=False),
+                    basis='sz(dzp)',
+                    h=0.3,
+                    spinpol=False,
+                    convergence={'energy': np.inf,
+                                 'eigenstates': np.inf,
+                                 'density': np.inf,
+                                 'minimum iterations': 1},
+                    eigensolver=FDPWETDM(converge_unocc=True),
+                    occupations={'name': 'fixed-uniform'},
+                    mixer={'backend': 'no-mixing'},
+                    nbands='nao',
+                    symmetry='off')
+        atoms.calc = calc
+        atoms.get_potential_energy()
+        return atoms.calc
+
+    @gpwfile
+    def h3_do_sd_lcao(self):
+        atoms = self.h3_maker(pbc=False)
+        atoms.set_initial_magnetic_moments([1, 0, 0])
+        calc = GPAW(mode='lcao',
+                    h=0.3,
+                    spinpol=True,
+                    convergence={'energy': 0.1,
+                                 'eigenstates': 1e-4,
+                                 'density': 1e-4},
+                    eigensolver={'name': 'etdm-lcao'},
+                    occupations={'name': 'fixed-uniform'},
+                    mixer={'backend': 'no-mixing'},
+                    nbands='nao',
+                    symmetry='off')
+        atoms.calc = calc
+        atoms.get_potential_energy()
+        return atoms.calc
+
+    @gpwfile
+    def h3_do_num_lcao(self):
+        atm = self.h3_maker()
+        atm.calc = GPAW(mode=LCAO(force_complex_dtype=True),
+                        basis='sz(dzp)',
+                        h=0.3,
+                        spinpol=False,
+                        convergence={'eigenstates': 10.0,
+                                     'density': 10.0,
+                                     'energy': 10.0},
+                        occupations={'name': 'fixed-uniform'},
+                        eigensolver={'name': 'etdm-lcao',
+                                     'matrix_exp': 'egdecomp'},
+                        mixer={'backend': 'no-mixing'},
+                        nbands='nao',
+                        symmetry='off',
+                        txt=None)
+        atm.get_potential_energy()
+        return atm.calc
+
+    def h2o_maker(self, vacuum, t=np.pi / 180 * 104.51, eps=0, **kwargs):
+        d = 0.9575
+        H2O = Atoms('OH2',
+                    positions=[(0, 0, 0),
+                               (d + eps, 0, 0),
+                               (d * np.cos(t), d * np.sin(t), 0)],
+                    **kwargs)
+        H2O.center(vacuum=vacuum)
+        return H2O
+
+    @gpwfile
+    def h2o_do_gmf_lcao(self):
+        atm = self.h2o_maker(vacuum=4.0)
+        atm.calc = GPAW(
+            mode=LCAO(),
+            basis="dzp",
+            h=0.22,
+            occupations={"name": "fixed-uniform"},
+            eigensolver="etdm-lcao",
+            mixer={"backend": "no-mixing"},
+            nbands="nao",
+            symmetry="off",
+            spinpol=True,
+            convergence={"density": 1.0e-4, "eigenstates": 4.0e-8},
+        )
+        atm.get_potential_energy()
+        return atm.calc
+
+    @gpwfile
+    def h2o_do_lcao(self):
+        atm = self.h2o_maker(vacuum=5.0)
+        atm.calc = GPAW(
+            mode=LCAO(),
+            basis="dzp",
+            occupations={"name": "fixed-uniform"},
+            eigensolver="etdm",
+            mixer={"backend": "no-mixing"},
+            nbands="nao",
+            symmetry="off",
+        )
+        atm.get_potential_energy()
+        return atm.calc
+
+    @gpwfile
+    def h2o_cdo_lcao(self):
+        atm = self.h2o_maker(vacuum=4.0)
+        atm.calc = GPAW(
+            mode=LCAO(),
+            basis="dzp",
+            h=0.22,
+            occupations={"name": "fixed-uniform"},
+            eigensolver={"name": "etdm-lcao"},
+            mixer={"backend": "no-mixing"},
+            nbands="nao",
+            symmetry="off",
+            spinpol=True,
+            convergence={"density": 1.0e-4, "eigenstates": 4.0e-8},
+        )
+        atm.get_potential_energy()
+        return atm.calc
+
+    @gpwfile
+    def h2o_cdo_lcao_sic(self):
+        atm = self.h2o_maker(vacuum=4.0)
+        atm.calc = GPAW(
+            mode=LCAO(force_complex_dtype=True),
+            h=0.22,
+            occupations={"name": "fixed-uniform"},
+            eigensolver={
+                "name": "etdm-lcao",
+                "localizationtype": "PM_PZ",
+                "localizationseed": 42,
+                "subspace_convergence": 1e-3,
+                "functional": {"name": "PZ-SIC", "scaling_factor": (0.5, 0.5)},
+            },
+            convergence={"eigenstates": 1e-4},
+            mixer={"backend": "no-mixing"},
+            nbands="nao",
+            symmetry="off",
+        )
+        atm.get_potential_energy()
+        return atm.calc
+
+    @gpwfile
+    def h2o_fdsic(self):
+        atm = self.h2o_maker(vacuum=4.0,
+                             t=np.pi / 180 * (104.51 + 2.0),
+                             eps=0.02)
+        atm.calc = GPAW(
+            mode=FD(force_complex_dtype=True),
+            h=0.25,
+            occupations={"name": "fixed-uniform"},
+            eigensolver=FDPWETDM(
+                functional={"name": "PZ-SIC", "scaling_factor": (0.5, 0.5)},
+                localizationseed=42,
+                localizationtype="FB_ER",
+                grad_tol_pz_localization=1.0e-3,
+                maxiter_pz_localization=200,
+                converge_unocc=True,
+            ),
+            convergence={"eigenstates": 1e-4},
+            mixer={"backend": "no-mixing"},
+            symmetry="off",
+            spinpol=True,
+        )
+        atm.get_potential_energy()
+        return atm.calc
+
+    @gpwfile
+    def h2o_lcaosic(self):
+        atm = self.h2o_maker(vacuum=4.0)
+        atm.calc = GPAW(
+            mode=LCAO(force_complex_dtype=True),
+            h=0.22,
+            occupations={"name": "fixed-uniform"},
+            eigensolver=LCAOETDM(
+                localizationtype="PM_PZ",
+                localizationseed=42,
+                functional={"name": "PZ-SIC", "scaling_factor": (0.5, 0.5)},
+            ),
+            convergence={"eigenstates": 1e-4},
+            mixer={"backend": "no-mixing"},
+            nbands="nao",
+            symmetry="off",
+        )
+        atm.get_potential_energy()
+        return atm.calc
+
+    @gpwfile
+    def h2o_mom_lcaosic(self):
+        atm = self.h2o_maker(vacuum=3.0)
+        calc = GPAW(
+            mode=LCAO(force_complex_dtype=True),
+            h=0.24,
+            basis="sz(dzp)",
+            spinpol=True,
+            symmetry="off",
+            eigensolver="etdm-lcao",
+            mixer={"backend": "no-mixing"},
+            occupations={"name": "fixed-uniform"},
+            convergence={"eigenstates": 1e-4},
+        )
+        atm.calc = calc
+        atm.get_potential_energy()
+        atm.calc.set(eigensolver=LCAOETDM(excited_state=True))
+        f_sn = excite(atm.calc, 0, 0, (0, 0))
+        prepare_mom_calculation(atm.calc, atm, f_sn)
+        atm.get_potential_energy()
+        atm.calc.set(
+            eigensolver=LCAOETDM(
+                searchdir_algo={"name": "l-sr1p"},
+                linesearch_algo={"name": "max-step"},
+                need_init_orbs=False,
+                localizationtype="PM_PZ",
+                localizationseed=42,
+                functional={"name": "pz-sic", "scaling_factor": (0.5, 0.5)},
+            ),
+            convergence={"eigenstates": 1e-2},
+        )
+        atm.get_potential_energy()
+        return atm.calc
+
+    @gpwfile
+    def h2o_gmf_lcaosic(self):
+        atm = self.h2o_maker(vacuum=3.0)
+        calc = GPAW(
+            mode=LCAO(),
+            basis="sz(dzp)",
+            h=0.24,
+            occupations={"name": "fixed-uniform"},
+            eigensolver="etdm-lcao",
+            convergence={"eigenstates": 1e-4},
+            mixer={"backend": "no-mixing"},
+            nbands="nao",
+            spinpol=True,
+            symmetry="off",
+        )
+        atm.calc = calc
+        atm.get_potential_energy()
+        atm.calc.set(eigensolver=LCAOETDM(excited_state=True))
+        f_sn = excite(atm.calc, 0, 0, (0, 0))
+        prepare_mom_calculation(atm.calc, atm, f_sn)
+        atm.get_potential_energy()
+        dave = SICDavidson(atm.calc.wfs.eigensolver, None)
+        appr_sp_order = dave.estimate_sp_order(atm.calc)
+
+        for kpt in atm.calc.wfs.kpt_u:
+            f_sn[kpt.s] = kpt.f_n
+        atm.calc.set(
+            eigensolver=LCAOETDM(
+                partial_diagonalizer={
+                    "name": "Davidson",
+                    "seed": 42,
+                    "m": 20,
+                    "eps": 5e-3,
+                    "remember_sp_order": True,
+                    "sp_order": appr_sp_order,
+                },
+                linesearch_algo={"name": "max-step"},
+                searchdir_algo={"name": "LBFGS-P_GMF"},
+                localizationtype="PM",
+                functional={"name": "PZ-SIC", "scaling_factor": (0.5, 0.5)},
+                need_init_orbs=False,
+            ),
+            occupations={"name": "mom",
+                         "numbers": f_sn,
+                         "use_fixed_occupations": True},
+        )
+        atm.get_potential_energy()
+        return atm.calc
+
+    @gpwfile
+    def h2o_mom_pwsic(self):
+        atm = self.h2o_maker(vacuum=3.0)
+        calc = GPAW(
+            mode=PW(300, force_complex_dtype=True),
+            spinpol=True,
+            symmetry="off",
+            eigensolver=FDPWETDM(converge_unocc=True),
+            mixer={"backend": "no-mixing"},
+            occupations={"name": "fixed-uniform"},
+            convergence={"eigenstates": 1e-4},
+        )
+        atm.calc = calc
+        atm.get_potential_energy()
+        atm.calc.set(eigensolver=FDPWETDM(excited_state=True))
+        f_sn = excite(atm.calc, 0, 0, (0, 0))
+        prepare_mom_calculation(atm.calc, atm, f_sn)
+        atm.get_potential_energy()
+        atm.calc.set(
+            eigensolver=FDPWETDM(
+                excited_state=True,
+                need_init_orbs=False,
+                functional={"name": "PZ-SIC",
+                            "scaling_factor": (0.5, 0.5)},  # SIC/2
+                localizationseed=42,
+                localizationtype="PM",
+                grad_tol_pz_localization=1.0e-2,
+                printinnerloop=False,
+                grad_tol_inner_loop=1.0e-2,
+            ),
+            convergence={"eigenstates": 1e-3, "density": 1e-3},
+        )
+        atm.get_potential_energy()
+        return atm.calc
+
+    @gpwfile
+    def h2o_pwsic(self):
+        atm = self.h2o_maker(vacuum=4.0,
+                             t=np.pi / 180 * (104.51 + 2.0),
+                             eps=0.02)
+        atm.calc = GPAW(
+            mode=PW(300, force_complex_dtype=True),
+            occupations={"name": "fixed-uniform"},
+            eigensolver=FDPWETDM(
+                functional={"name": "pz-sic", "scaling_factor": (0.5, 0.5)},
+                localizationseed=42,
+                localizationtype="FB_ER",
+                grad_tol_pz_localization=5.0e-3,
+                maxiter_pz_localization=200,
+                converge_unocc=True,
+            ),
+            convergence={"eigenstates": 1e-4},
+            mixer={"backend": "no-mixing"},
+            symmetry="off",
+            spinpol=True,
+        )
+        atm.get_potential_energy()
+        return atm.calc
+
+    @gpwfile
+    def h2o_mom_do_pw(self):
+        atm = self.h2o_maker(vacuum=4.0)
+        calc = GPAW(
+            mode=PW(300),
+            spinpol=True,
+            symmetry="off",
+            eigensolver={"name": "etdm-fdpw", "converge_unocc": True},
+            mixer={"backend": "no-mixing"},
+            occupations={"name": "fixed-uniform"},
+            convergence={"eigenstates": 1e-4},
+            txt=None,
+        )
+        atm.calc = calc
+        atm.get_potential_energy()
+        return atm.calc
+
+    @gpwfile
+    def co_mom_do_lcao_forces(self):
+        L = 4.0
+        d = 1.13
+        atoms = Atoms(
+            "CO",
+            [[0.5 * L, 0.5 * L, 0.5 * L - 0.5 * d],
+             [0.5 * L, 0.5 * L, 0.5 * L + 0.5 * d]],
+        )
+        atoms.set_cell([L, L, L])
+        atoms.set_pbc(True)
+        calc = GPAW(
+            mode="lcao",
+            basis="dzp",
+            h=0.22,
+            xc="PBE",
+            spinpol=True,
+            symmetry="off",
+            occupations={"name": "fixed-uniform"},
+            eigensolver={"name": "etdm-lcao", "linesearch_algo": "max-step"},
+            mixer={"backend": "no-mixing"},
+            nbands="nao",
+            convergence={"density": 1.0e-4, "eigenstates": 4.0e-8},
+        )
+        atoms.calc = calc
+        atoms.get_potential_energy()
+        return atoms.calc
+
+    @gpwfile
+    def h2o_mom_do_lcao(self):
+        atm = self.h2o_maker(vacuum=4.0)
+        atm.calc = GPAW(
+            mode=LCAO(),
+            basis="dzp",
+            h=0.22,
+            occupations={"name": "fixed-uniform"},
+            eigensolver="etdm-lcao",
+            mixer={"backend": "no-mixing"},
+            nbands="nao",
+            symmetry="off",
+            spinpol=True,
+            convergence={"density": 1.0e-4, "eigenstates": 4.0e-8},
+        )
+        atm.get_potential_energy()
+        return atm.calc
+
+    @gpwfile
+    def h2o_pz_localization_pw(self):
+        atm = self.h2o_maker(vacuum=3.0,
+                             t=np.pi / 180 * (104.51 + 2.0),
+                             eps=0.02)
+        atm.calc = GPAW(
+            mode=PW(300, force_complex_dtype=True),
+            occupations={"name": "fixed-uniform"},
+            convergence={
+                "energy": np.inf,
+                "eigenstates": np.inf,
+                "density": np.inf,
+                "minimum iterations": 0,
+            },
+            eigensolver=FDPWETDM(converge_unocc=False),
+            mixer={"backend": "no-mixing"},
+            symmetry="off",
+            spinpol=True,
+        )
+        atm.get_potential_energy()
+        atm.calc.set(
+            eigensolver=FDPWETDM(
+                functional={"name": "PZ-SIC", "scaling_factor": (0.5, 0.5)},
+                localizationseed=42,
+                localizationtype="KS_PZ",
+                localization_tol=5.0e-2,
+                converge_unocc=False,
+            )
+        )
+        atm.get_potential_energy()
+        return atm.calc
+
+    @gpwfile
+    def c2h4_do_lcao(self):
+        atoms = Atoms(
+            "C2H4",
+            [
+                [6.68748500e-01, 2.00680000e-04, 5.55800000e-05],
+                [-6.68748570e-01, -2.00860000e-04, -5.51500000e-05],
+                [4.48890600e-01, -5.30146300e-01, 9.32670330e-01],
+                [4.48878120e-01, -5.30176640e-01, -9.32674730e-01],
+                [-1.24289513e00, 1.46164400e-02, 9.32559990e-01],
+                [-1.24286000e00, -1.46832100e-02, -9.32554970e-01],
+            ],
+        )
+        atoms.center(vacuum=4)
+        eigensolver = LCAOETDM(
+            searchdir_algo={"name": "l-sr1p"},
+            linesearch_algo={"name": "max-step"}
+        )
+        calc = GPAW(
+            mode="lcao",
+            basis="dzp",
+            h=0.24,
+            xc="PBE",
+            symmetry="off",
+            occupations={"name": "fixed-uniform"},
+            eigensolver=eigensolver,
+            mixer={"backend": "no-mixing"},
+            nbands="nao",
+            convergence={"density": 1.0e-4, "eigenstates": 4.0e-8},
+        )
+        atoms.calc = calc
+        atoms.get_potential_energy()
+        return atoms.calc
+
+    @gpwfile
+    def h3_orthonorm_lcao(self):
+        atoms = Atoms("H3", positions=[(0, 0, 0), (0.59, 0, 0), (1.1, 0, 0)])
+        atoms.set_initial_magnetic_moments([1, 0, 0])
+
+        atoms.center(vacuum=2.0)
+        atoms.set_pbc(False)
+        calc = GPAW(
+            mode="lcao",
+            basis="sz(dzp)",
+            h=0.3,
+            spinpol=True,
+            convergence={
+                "energy": np.inf,
+                "eigenstates": np.inf,
+                "density": np.inf,
+                "minimum iterations": 1,
+            },
+            eigensolver={"name": "etdm-lcao"},
+            occupations={"name": "fixed-uniform"},
+            mixer={"backend": "no-mixing"},
+            nbands="nao",
+            symmetry="off",
+            txt=None,
+        )
+        atoms.calc = calc
+        atoms.get_potential_energy()
+        return atoms.calc
+
+    @gpwfile
+    def h2_sic_scfsic(self):
+        a = 6.0
+        atm = Atoms("H2", positions=[(0, 0, 0), (0, 0, 0.737)], cell=(a, a, a))
+        atm.center()
+        calc = GPAW(mode="fd", xc="LDA-PZ-SIC",
+                    eigensolver="rmm-diis", setups="hgh")
+        atm.calc = calc
+        atm.get_potential_energy()
+        return atm.calc
+
+    @gpwfile
+    def h_magmom(self):
+        a = 6.0
+        atm = Atoms("H", magmoms=[1.0], cell=(a, a, a))
+        atm.center()
+        calc = GPAW(mode="fd", xc="LDA-PZ-SIC",
+                    eigensolver="rmm-diis", setups="hgh")
+        atm.calc = calc
+        atm.get_potential_energy()
+        return atm.calc
+
+    @gpwfile
+    def h_hess_num_pw(self):
+        calc = GPAW(
+            xc="PBE",
+            mode=PW(300, force_complex_dtype=False),
+            h=0.25,
+            convergence={
+                "energy": np.inf,
+                "eigenstates": np.inf,
+                "density": np.inf,
+                "minimum iterations": 1,
+            },
+            spinpol=False,
+            eigensolver=FDPWETDM(converge_unocc=True),
+            occupations={"name": "fixed-uniform"},
+            mixer={"backend": "no-mixing"},
+            nbands=2,
+            symmetry="off",
+        )
+        atoms = Atoms("H", positions=[[0, 0, 0]])
+        atoms.center(vacuum=5.0)
+        atoms.set_pbc(False)
+        atoms.calc = calc
+        atoms.get_potential_energy()
+        return atoms.calc
+
+    @gpwfile
+    def h2_break_ilcao(self):
+        atoms = Atoms('H2', positions=[(0, 0, 0), (0, 0, 2.0)])
+        atoms.center(vacuum=2.0)
+        atoms.set_pbc(False)
+        calc = GPAW(xc='PBE',
+                    mode='lcao',
+                    h=0.24,
+                    basis='sz(dzp)',
+                    spinpol=True,
+                    eigensolver='etdm-lcao',
+                    convergence={'density': 1.0e-2,
+                                 'eigenstates': 1.0e-2},
+                    occupations={'name': 'fixed-uniform'},
+                    mixer={'backend': 'no-mixing'},
+                    nbands='nao',
+                    symmetry='off')
+        atoms.calc = calc
+        atoms.get_potential_energy()
+        return atoms.calc
+
+    @gpwfile
+    def h_do_gdavid_lcao(self):
+        calc = GPAW(xc='PBE',
+                    mode=LCAO(force_complex_dtype=True),
+                    h=0.25,
+                    basis='dz(dzp)',
+                    spinpol=False,
+                    eigensolver={'name': 'etdm-lcao',
+                                 'representation': 'u-invar'},
+                    occupations={'name': 'fixed-uniform'},
+                    mixer={'backend': 'no-mixing'},
+                    nbands='nao',
+                    symmetry='off',
+                    )
+
+        atoms = Atoms('H', positions=[[0, 0, 0]])
+        atoms.center(vacuum=5.0)
+        atoms.set_pbc(False)
+        atoms.calc = calc
+        atoms.get_potential_energy()
+        return atoms.calc
+
+    @gpwfile
+    def h2_mom_do_pwh(self):
+        d = 1.4 * Bohr
+        h2 = Atoms('H2',
+                   positions=[[-d / 2, 0, 0],
+                              [d / 2, 0, 0]])
+        h2.center(vacuum=3)
+        calc = GPAW(mode=PW(300),
+                    # h=0.3,
+                    xc={'name': 'HSE06', 'backend': 'pw'},
+                    eigensolver={'name': 'etdm-fdpw',
+                                 'converge_unocc': True},
+                    mixer={'backend': 'no-mixing'},
+                    occupations={'name': 'fixed-uniform'},
+                    symmetry='off',
+                    nbands=2,
+                    convergence={'eigenstates': 4.0e-6})
+        h2.calc = calc
+        h2.get_potential_energy()
+        return h2.calc
+
+    @gpwfile
+    def h_hess_num_lcao(self):
+        calc = GPAW(xc='PBE',
+                    mode=LCAO(force_complex_dtype=True),
+                    h=0.25,
+                    basis='dz(dzp)',
+                    spinpol=False,
+                    eigensolver={'name': 'etdm-lcao',
+                                 'representation': 'u-invar'},
+                    occupations={'name': 'fixed-uniform'},
+                    mixer={'backend': 'no-mixing'},
+                    nbands='nao',
+                    symmetry='off',
+                    )
+        atoms = Atoms('H', positions=[[0, 0, 0]])
+        atoms.center(vacuum=5.0)
+        atoms.set_pbc(False)
+        atoms.calc = calc
+        atoms.get_potential_energy()
+        return atoms.calc
+
     @gpwfile
     def silicon_pdens_tool(self):
         # used by response code's pdens tool test
@@ -455,8 +1171,6 @@ class GPWFiles(CachedFilesHandler):
 
     @gpwfile
     def h2o_xas(self):
-        from math import cos, pi, sin
-
         setupname = 'h2o_xas_hch1s'
         self.generator2_setup(
             'O', 8, '2s,s,2p,p,d', [1.2], 1.0, None, 2,
@@ -464,18 +1178,7 @@ class GPWFiles(CachedFilesHandler):
             name=setupname)
 
         a = 5.0
-        d = 0.9575
-        t = pi / 180 * 104.51
-        H2O = Atoms(
-            [
-                Atom('O', (0, 0, 0)),
-                Atom('H', (d, 0, 0)),
-                Atom('H', (d * cos(t), d * sin(t), 0)),
-            ],
-            cell=(a, a, a),
-            pbc=False,
-        )
-        H2O.center()
+        H2O = self.h2o_maker(vacuum=None, cell=[a, a, a], pbc=False)
         calc = GPAW(
             txt=self.folder / 'h2o_xas.txt',
             mode='fd',
@@ -541,18 +1244,37 @@ class GPWFiles(CachedFilesHandler):
         return si.calc
 
     @gpwfile
-    def si_qpoint_rounding_bug(self):
+    def si_noisy_kpoints(self):
         # Test system for guarding against inconsistent kpoints as in #1178.
         from ase.calculators.calculator import kpts2kpts
         atoms = bulk('Si')
-        kpts = kpts2kpts(kpts={'gamma': True, 'size': (6, 1, 1)}, atoms=atoms)
+        kpts = kpts2kpts(kpts={'size': (2, 2, 2), 'gamma': True}, atoms=atoms)
 
         # Error happened when qpoint was ~1e-17 yet was not considered gamma.
         # Add a bit of noise on purpose so we are sure to hit such a case,
         # even if the underlying implementation changes:
-        kpts.kpts += np.linspace(1e-16, 1e-15, 18).reshape(6, 3)
+        kpts.kpts += np.linspace(1e-16, 1e-15, 24).reshape(8, 3)
 
-        calc = GPAW(mode=PW(340), kpts=kpts)
+        calc = GPAW(mode='pw',
+                    xc='LDA',
+                    occupations=FermiDirac(width=0.001),
+                    kpts=kpts.kpts,
+                    txt=self.folder / 'si_noisy_kpoints.txt')
+        atoms.calc = calc
+        atoms.get_potential_energy()
+        return calc
+
+    @gpwfile
+    def si_pw_nbands10_converged(self):
+        calc = GPAW(mode='pw',
+                    kpts={'size': (2, 2, 2), 'gamma': True},
+                    occupations=FermiDirac(0.01),
+                    nbands=10,
+                    symmetry='off',
+                    convergence={'bands': -4, 'density': 1e-7,
+                                 'eigenstates': 1e-10})
+
+        atoms = bulk('Si', 'diamond', a=5.431)
         atoms.calc = calc
         atoms.get_potential_energy()
         return calc
@@ -597,7 +1319,6 @@ class GPWFiles(CachedFilesHandler):
 
         calc = GPAW(mode='fd',
                     txt=self.folder / 'si_corehole_pw.txt',
-                    nbands=None,
                     h=0.25,
                     occupations=FermiDirac(width=0.05),
                     setups='si_corehole_pw_hch1s',
@@ -1024,7 +1745,7 @@ class GPWFiles(CachedFilesHandler):
 
         gs_calc = GPAW(
             txt=self.folder / 'nacl_fd.txt',
-            mode='fd', nbands=4, eigensolver='cg',
+            mode='fd', nbands=4,  # eigensolver='cg',
             gpts=(32, 32, 44), xc='LDA', symmetry={'point_group': False},
             setups={'Na': '1'})
         atoms.calc = gs_calc
@@ -1043,8 +1764,7 @@ class GPWFiles(CachedFilesHandler):
         atoms.get_potential_energy()
         return atoms.calc
 
-    @gpwfile
-    def hbn_pw(self):
+    def _hbn_pw(self, symmetry={}):
         atoms = Graphene(symbol='B',
                          latticeconstant={'a': 2.5, 'c': 1.0},
                          size=(1, 1, 1))
@@ -1058,10 +1778,19 @@ class GPWFiles(CachedFilesHandler):
                           occupations=FermiDirac(0.001),
                           parallel={'domain': 1},
                           convergence={'bands': 26},
-                          kpts={'size': (3, 3, 1), 'gamma': True})
+                          kpts={'size': (3, 3, 1), 'gamma': True},
+                          symmetry=symmetry)
         atoms.get_potential_energy()
         return atoms.calc
 
+    @gpwfile
+    def hbn_pw(self):
+        return self._hbn_pw()
+
+    @gpwfile
+    def hbn_pw_nosym(self):
+        return self._hbn_pw(symmetry='off')
+
     @gpwfile
     def graphene_pw(self):
         from ase.lattice.hexagonal import Graphene
@@ -1226,8 +1955,7 @@ class GPWFiles(CachedFilesHandler):
     def p4_pw_spinpol(self):
         return self._p4(spinpol=True)
 
-    @gpwfile
-    def ni_pw_kpts333(self):
+    def _ni_pw_kpts333(self, setups={'Ni': '10'}):
         from ase.dft.kpoints import monkhorst_pack
         # from gpaw.mpi import serial_comm
         Ni = bulk('Ni', 'fcc')
@@ -1239,7 +1967,7 @@ class GPWFiles(CachedFilesHandler):
                     txt=self.folder / 'ni_pw_kpts333.txt',
                     kpts=kpts,
                     occupations=FermiDirac(0.001),
-                    setups={'Ni': '10'},
+                    setups=setups,
                     parallel=dict(domain=1),  # >1 fails on 8 cores
                     # communicator=serial_comm
                     )
@@ -1249,6 +1977,14 @@ class GPWFiles(CachedFilesHandler):
         calc.diagonalize_full_hamiltonian()
         return calc
 
+    @gpwfile
+    def ni_pw(self):
+        return self._ni_pw_kpts333(setups={})
+
+    @gpwfile
+    def ni_pw_kpts333(self):
+        return self._ni_pw_kpts333()
+
     @gpwfile
     def c_pw(self):
         atoms = bulk('C')
@@ -1314,12 +2050,16 @@ class GPWFiles(CachedFilesHandler):
                    scaled_positions=[[0.5, 0.5, 0.5]],
                    pbc=False)
 
-        Tl.calc = GPAWNew(mode={'name': 'pw', 'ecut': 300}, xc='LDA',
-                          occupations={'name': 'fermi-dirac', 'width': 0.01},
-                          symmetry='off',
-                          convergence={'density': 1e-6},
-                          magmoms=[[0, 0, 0.5]], soc=True,
-                          txt=self.folder / 'Tl_box_pw.txt')
+        Tl.calc = GPAWNew(
+            mode={'name': 'pw', 'ecut': 300},
+            xc='LDA',
+            occupations={'name': 'fermi-dirac', 'width': 0.01},
+            symmetry='off',
+            convergence={'density': 1e-6},
+            parallel={'domain': 1, 'band': 1},
+            magmoms=[[0, 0, 0.5]],
+            soc=True,
+            txt=self.folder / 'Tl_box_pw.txt')
         Tl.get_potential_energy()
         return Tl.calc
 
@@ -1395,6 +2135,9 @@ class GPWFiles(CachedFilesHandler):
         a = 2.867
         mm = 2.21
         atoms = bulk('Fe', 'bcc', a=a)
+        # It is necessary to rattle the atoms to make sure that all tests pass
+        # on all machines - see https://gitlab.com/gpaw/gpaw/-/issues/1397
+        atoms.rattle(0.01, seed=42)
         atoms.set_initial_magnetic_moments([mm])
         atoms.center()
         tag = '_nosym' if symmetry == 'off' else ''
