@@ -12,7 +12,7 @@ pytestmark = pytest.mark.skipif(not LibElpa.have_elpa(),
 
 def test_lcao_lcao_elpa_kpts(gpaw_new):
     if gpaw_new and world.size == 8:
-        pytest.skip(msg='Not implementted')
+        pytest.skip('Not implementted')
 
     energies = []
 
