@@ -8,10 +8,9 @@ from gpaw.lcao.tools import get_lcao_hamiltonian
 from gpaw.mpi import world
 
 
-@pytest.mark.old_gpaw_only
 def test_lcao_lcao_hamiltonian(in_tmp_dir, add_cwd_to_setup_paths):
     if world.rank == 0:
-        basis = BasisMaker('Li').generate(1, 1)
+        basis = BasisMaker.from_symbol('Li').generate(1, 1)
         basis.write_xml()
     world.barrier()
 
