@@ -3,10 +3,9 @@ import pytest
 import numpy as np
 
 from gpaw.nlopt.matrixel import make_nlodata
-from gpaw.mpi import world
 
 
-@pytest.mark.skipif(world.size > 1, reason='Only serial')
+@pytest.mark.serial
 def test_mme_Ni(gpw_files):
 
     # Collinear calculation
@@ -27,13 +26,7 @@ def test_mme_Ni(gpw_files):
     E_ncol_n = data[2]
     p_ncol_vnn = np.abs(data[3])
 
-    # print(np.abs(p1_col_vnn[0]))
-    # print(' ')
-    # print(np.abs(p2_col_vnn[0]))
-    # print(' ')
-    # print(np.abs(p_ncol_vnn[0].round(10)))
-
-    assert E_ncol_n[0:3] == pytest.approx(E1_col_n, abs=1.e-9)
-    assert E_ncol_n[3:6] == pytest.approx(E2_col_n, abs=1.e-9)
-    assert p_ncol_vnn[:, 0:3, 0:3] == pytest.approx(p1_col_vnn, abs=1.e-9)
-    assert p_ncol_vnn[:, 3:6, 3:6] == pytest.approx(p2_col_vnn, abs=1.e-9)
+    assert E_ncol_n[0:3] == pytest.approx(E1_col_n, abs=3.0e-6)
+    assert E_ncol_n[3:6] == pytest.approx(E2_col_n, abs=5.0e-6)
+    assert p_ncol_vnn[:, 0:3, 0:3] == pytest.approx(p1_col_vnn, abs=2.0e-6)
+    assert p_ncol_vnn[:, 3:6, 3:6] == pytest.approx(p2_col_vnn, abs=2.0e-6)
