@@ -24,7 +24,7 @@ def test_pw_slab(gpu, GPAW):
     if gpu:
         parallel['gpu'] = True
     calc = GPAW(mode=PW(200),
-                eigensolver='dav' if GPAW is NewGPAW else 'rmm-diis',
+                eigensolver='rmm-diis',
                 parallel=parallel,
                 kpts=(k, k, 1))
     slab.calc = calc
