@@ -3,7 +3,7 @@ from gpaw.grid_descriptor import GridDescriptor
 from gpaw import PoissonSolver
 from gpaw.spline import Spline
 from gpaw.lfc import LocalizedFunctionsCollection as LFC
-from gpaw.response.pair_functions import SingleQPWDescriptor
+from gpaw.response.qpd import SingleQPWDescriptor
 import numpy as np
 from gpaw.response.coulomb_kernels import (get_coulomb_kernel,
                                            get_integrated_kernel)
