@@ -1,15 +1,10 @@
 import pytest
 from gpaw.response.g0w0 import G0W0
 import numpy as np
-from gpaw.mpi import world
 
 
 @pytest.mark.response
-def test_gw_anisotropic(in_tmp_dir, gpw_files, gpaw_new):
-    if gpaw_new and world.size > 1:
-        pytest.skip('Hybrids not working in parallel with GPAW_NEW=1')
-    print(gpw_files)
-
+def test_gw_anisotropic(in_tmp_dir, gpw_files):
     gw = G0W0(gpw_files['sic_pw'],
               'gw-test',
               nbands=5,
