@@ -16,7 +16,9 @@ def test_gw_sg15(in_tmp_dir, add_cwd_to_setup_paths):
     sys.center(vacuum=2.5)
 
     calc = GPAW(setups='sg15',
-                xc='PBE', mode=PW(ecut=300), convergence={'bands': 40},
+                xc='PBE', mode=PW(ecut=300),
+                convergence={'bands': 40,
+                             'density': 1e-6},
                 nbands=50, kpts=(2, 2, 2))
     sys.calc = calc
     sys.get_potential_energy()
@@ -29,5 +31,5 @@ def test_gw_sg15(in_tmp_dir, add_cwd_to_setup_paths):
               filename='H2_g0w0_b11-15')
     results = gw.calculate()
     refs = pytest.approx([1.58177132, 3.33595453, 3.87312694, 4.95358262,
-                         6.09555254], abs=1e-5)
+                         6.09555254], abs=2e-5)
     assert results['qp'][0][0] == refs
