@@ -6,11 +6,8 @@ from gpaw.xc.rpa import RPACorrelation
 
 @pytest.mark.rpa
 @pytest.mark.response
-@pytest.mark.parametrize('gpwfilename, eref', [
-    ('si_qpoint_rounding_bug', -18.814),
-    ('si_pw', -12.61),
-])
-def test_rpa_energy_si(in_tmp_dir, gpw_files, gpwfilename, eref):
+@pytest.mark.parametrize('gpwfilename', ['si_pw', 'si_noisy_kpoints'])
+def test_rpa_energy_si(in_tmp_dir, gpw_files, gpwfilename):
     calc = GPAW(gpw_files[gpwfilename], communicator=serial_comm)
     calc.diagonalize_full_hamiltonian(nbands=50)
 
@@ -21,5 +18,5 @@ def test_rpa_energy_si(in_tmp_dir, gpw_files, gpwfilename, eref):
     rpa = RPACorrelation(calc, qsym=True, nfrequencies=8, ecut=[ecut])
     E_rpa_qsym = rpa.calculate()
 
-    assert E_rpa_qsym == pytest.approx(eref, abs=0.01)
+    assert E_rpa_qsym == pytest.approx(-12.61, abs=0.01)
     assert E_rpa_qsym == pytest.approx(E_rpa_noqsym)
