@@ -8,7 +8,7 @@ from gpaw.analyse.multipole import Multipole
 from gpaw.utilities.adjust_cell import adjust_cell
 
 
-@pytest.mark.later
+@pytest.mark.old_gpaw_only
 def test_multipoleH2O(in_tmp_dir):
     h = 0.3
 
