@@ -50,22 +50,22 @@ def test_orbmag_Ni(gpw_files):
     dif_orbmag = np.linalg.norm(orbmag_ncol_v - orbmag_col_v)
 
     assert dif_energy == pytest.approx(0, abs=1.0e-6)
-    assert dif_magmom == pytest.approx(0, abs=1.0e-6)
-    assert dif_orbmag == pytest.approx(0, abs=1.0e-5)
+    assert dif_magmom == pytest.approx(0, abs=2.0e-6)
+    assert dif_orbmag == pytest.approx(0, abs=1.0e-3)
 
     # Non-collinear calculation with self-consistent spin–orbit
     calc_ncolsoc = GPAW(gpw_files['fcc_Ni_ncolsoc'],
                         parallel={'domain': 1, 'band': 1})
 
     energy_ncolsoc = calc_ncolsoc.get_potential_energy(calc_ncolsoc.atoms)
-    assert energy_ncolsoc == pytest.approx(-8.33016, abs=1.0e-5)
+    assert energy_ncolsoc == pytest.approx(-8.478, abs=1.0e-3)
     orbmag_ncolsoc_v = calc_ncolsoc.get_orbital_magnetic_moments()[0]
 
     # Assert direction and magnitude of orbital magnetic moment
     assert np.linalg.norm(orbmag_ncolsoc_v) == pytest.approx(
-        0.044320880619364636, abs=2e-6)
+        0.045, abs=2e-3)
     assert np.dot(orbmag_ncolsoc_v, easy_axis) == pytest.approx(
-        0.044320880619364636, abs=2e-6)
+        0.045, abs=2e-3)
 
     # Get difference between orbital magnetic moments when soc is included
     # self-consistently. Assert that this difference doesn't change.
@@ -73,5 +73,5 @@ def test_orbmag_Ni(gpw_files):
     dif_orbmag2 = np.linalg.norm(orbmag_ncolsoc_v - orbmag_col_v)
     dif_orbmag3 = np.linalg.norm(orbmag_ncolsoc_v - orbmag_ncol_v)
 
-    assert dif_orbmag2 == pytest.approx(0.002409330194910108, abs=1e-5)
-    assert dif_orbmag3 == pytest.approx(0.002409330194910108, abs=1e-5)
+    assert dif_orbmag2 == pytest.approx(0, abs=2e-3)
+    assert dif_orbmag3 == pytest.approx(0, abs=2e-3)
