@@ -74,6 +74,89 @@ class DebugTimer(Timer):
         Timer.stop(self, name)
 
 
+class GPUEvent:
+    def __init__(self, name):
+        self.name = name
+        import cupy
+        default = dict(block=False,
+                       disable_timing=False,
+                       interprocess=False)
+        self.stop_event = cupy.cuda.Event(**default)
+        self.start_event = cupy.cuda.Event(**default)
+        self.start_event.record()
+
+    def stop(self):
+        self.stop_event.record()
+
+    def get_time(self):
+        import cupy
+        return cupy.cuda.get_elapsed_time(self.start_event,
+                                          self.stop_event) / 1000
+
+
+class GPUTimerBase:
+
+    def __init__(self, max_stack=10):
+        self.event_queue = []
+        self.event_stack = []
+        self.max_stack = max_stack
+        from collections import defaultdict
+        self.gpu_timers = defaultdict(float)
+
+    def gpu_start(self, key):
+        self.event_stack.append(GPUEvent(key))
+
+    def gpu_stop(self):
+        gpu_event = self.event_stack.pop()
+        gpu_event.stop()
+        self.event_queue.append(gpu_event)
+        if len(self.event_queue) > self.max_stack:
+            self.handle_events()
+
+    def handle_events(self):
+        while len(self.event_queue):
+            event = self.event_queue[0]
+            if not event.stop_event.done:
+                break
+            del self.event_queue[0]
+            time = event.get_time()
+            self.gpu_timers[event.name] += time
+            self.handle_event_hook(event)
+
+    def handle_event_hook(self, event):
+        pass
+
+    def gpu_write(self, out=sys.stdout):
+        import cupy
+        event = cupy.cuda.Event(block=True)
+        event.synchronize()
+        self.handle_events()
+
+        timers, self.timers = self.timers, self.gpu_timers
+        Timer.write(self, out)
+        self.timers = timers
+
+
+class GPUTimer(Timer, GPUTimerBase):
+    def __init__(self, *args, **kwargs):
+        Timer.__init__(self, *args, **kwargs)
+        GPUTimerBase.__init__(self)
+
+    def start(self, name):
+        Timer.start(self, name)
+        GPUTimerBase.gpu_start(self, name)
+
+    def stop(self, name=None):
+        Timer.stop(self, name)
+        GPUTimerBase.gpu_stop(self)
+
+    def write(self, out=sys.stdout):
+        print('CPU event timings:', file=out)
+        Timer.write(self, out)
+        print('GPU event timings:', file=out)
+        GPUTimerBase.gpu_write(self, out)
+
+
 def ranktxt(comm, rank=None):
     rank = comm.rank if rank is None else rank
     ndigits = len(str(comm.size - 1))
@@ -128,13 +211,16 @@ class Profiler(Timer):
         # legacy json format for perfetto always assumes microseconds
         self.u = 1_000_000
 
+        self.synchronize()
+        Timer.__init__(self, 1000)
+
+    def synchronize(self):
         # Synchronize in order to have same time reference
         ref = np.zeros(1)
-        if comm.rank == 0:
+        if self.comm.rank == 0:
             ref[0] = time.time()
-        comm.broadcast(ref, 0)
+        self.comm.broadcast(ref, 0)
         self.ref = ref[0]
-        Timer.__init__(self, 1000)
 
     def finish_trace(self):
         self.txt.close()
@@ -170,6 +256,63 @@ class Profiler(Timer):
         Timer.stop(self, name)
 
 
+class GPUProfiler(Profiler, GPUTimerBase):
+    def __init__(self, prefix, comm=mpi.world):
+        Profiler.__init__(self, prefix, comm=comm)
+        GPUTimerBase.__init__(self)
+
+    def synchronize(self):
+        from cupy.cuda import Event
+        # Make sure GPU gets here
+        event = Event(block=True)
+        event.record()
+        event.synchronize()
+
+        # Wait all CPUs
+        self.comm.barrier()
+
+        # Now all GPUs and CPUs are somewhat simultaneous
+        # So, record the reference event and time
+        event = Event(block=True)
+        event.record()
+        self.ref = time.time()
+        self.ref_event = event
+
+        # Broadcast CPU time reference (possibly problematic)
+        buf = np.zeros(1)
+        buf[0] = self.ref
+        self.comm.broadcast(buf, 0)
+        self.ref = buf[0]
+
+    def start(self, name, gpu=False):
+        Profiler.start(self, name)
+        if gpu:
+            GPUTimerBase.gpu_start(self, name)
+
+    def stop(self, name=None, gpu=False):
+        Profiler.stop(self, name)
+        if gpu:
+            GPUTimerBase.gpu_stop(self)
+
+    def handle_event_hook(self, event):
+        import cupy
+
+        def get_time(e):
+            t = cupy.cuda.get_elapsed_time
+            return t(self.ref_event, e)
+
+        ms_start = get_time(event.start_event)
+        ms_stop = get_time(event.stop_event)
+        self.txt.write(
+            f"""{{"name": "{event.name}", "cat": "PERF", "ph": "B","""
+            f""" "pid": {self.pid}, "tid": "GPU {self.ranktxt}", """
+            f""""ts": {int(ms_start * 1000)} }},\n""")
+        self.txt.write(
+            f"""{{"name": "{event.name}", "cat": "PERF", "ph": "E", """
+            f""""pid": {self.pid}, "tid": "GPU {self.ranktxt}", """
+            f""""ts": {int(ms_stop * 1000)} }},\n""")
+
+
 class HPMTimer(Timer):
     """HPMTimer requires installation of the IBM BlueGene/P HPM
     middleware interface to the low-level UPC library. This will
