@@ -1,4 +1,3 @@
-import numpy as np
 from gpaw import get_libraries
 from gpaw.xc.functional import XCFunctional
 from gpaw.xc.gga import GGA
@@ -35,8 +34,7 @@ def xc_string_to_dict(string):
 def XC(kernel,
        parameters=None,
        atoms=None,
-       collinear=True,
-       xp=np) -> XCFunctional:
+       collinear=True) -> XCFunctional:
     """Create XCFunctional object.
 
     kernel: XCKernel object, dict or str
@@ -74,7 +72,7 @@ def XC(kernel,
             # HSExx, since otherwise PWHybrid would hijack the control flow.
             from gpaw.xc.ri import RI
             return RI(name, **kwargs)
-        elif backend == 'pw' or name in ['HSE03', 'HSE06']:
+        elif backend == 'pw' or name in ['HSE03', 'HSE06', 'YS-PBE0']:
             from gpaw.hybrids import HybridXC
             return HybridXC(name, **kwargs)  # type: ignore
         elif backend:
@@ -150,6 +148,6 @@ def XC(kernel,
         return LDA(kernel, **kwargs)
 
     elif kernel.type == 'GGA':
-        return GGA(kernel, xp=xp, **kwargs)
+        return GGA(kernel, **kwargs)
     else:
         return MGGA(kernel, **kwargs)
