@@ -2,7 +2,8 @@ from math import pi
 
 import numpy as np
 
-from gpaw.xc.lda import calculate_paw_correction
+from gpaw.xc.lda import (calculate_paw_correction, stress_integral,
+                         stress_lda_term)
 from gpaw.utilities.blas import axpy
 from gpaw.fd_operators import Gradient
 from gpaw.sphere.lebedev import Y_nL, weight_n
@@ -10,6 +11,27 @@ from gpaw.xc.pawcorrection import rnablaY_nLv
 from gpaw.xc.functional import XCFunctional
 
 
+def stress_gga_term(gd, sigma_xg, gradn_svg, dedsigma_xg):
+    P = 0
+    nspins = gradn_svg.shape[0]
+    for sigma_g, dedsigma_g in zip(sigma_xg, dedsigma_xg):
+        P -= 2 * stress_integral(gd, sigma_g, dedsigma_g)
+    stress_vv = P * np.eye(3)
+    for v1 in range(3):
+        for v2 in range(3):
+            stress_vv[v1, v2] -= 2 * stress_integral(
+                gd, gradn_svg[0, v1] * gradn_svg[0, v2], dedsigma_xg[0]
+            )
+            if nspins == 2:
+                stress_vv[v1, v2] -= 2 * stress_integral(
+                    gd, gradn_svg[0, v1] * gradn_svg[1, v2], dedsigma_xg[1]
+                )
+                stress_vv[v1, v2] -= 2 * stress_integral(
+                    gd, gradn_svg[1, v1] * gradn_svg[1, v2], dedsigma_xg[2]
+                )
+    return stress_vv
+
+
 class GGARadialExpansion:
     def __init__(self, rcalc, *args):
         self.rcalc = rcalc
@@ -174,11 +196,10 @@ def get_gradient_ops(gd, nn, xp):
 
 
 class GGA(XCFunctional):
-    def __init__(self, kernel, stencil=2, xp=np):
+    def __init__(self, kernel, stencil=2):
         XCFunctional.__init__(self, kernel.name, kernel.type)
         self.kernel = kernel
         self.stencil_range = stencil
-        self.xp = xp
 
     def set_grid_descriptor(self, gd):
         XCFunctional.set_grid_descriptor(self, gd)
@@ -215,28 +236,10 @@ class GGA(XCFunctional):
         e_g = self.gd.empty()
         self.kernel.calculate(e_g, n_sg, v_sg, sigma_xg, dedsigma_xg)
 
-        def integrate(a1_g, a2_g=None):
-            return self.gd.integrate(a1_g, a2_g, global_integral=False)
+        stress_vv = stress_lda_term(self.gd, e_g, n_sg, v_sg)
+        stress_vv[:] += stress_gga_term(self.gd, sigma_xg, gradn_svg,
+                                        dedsigma_xg)
 
-        P = integrate(e_g)
-        for v_g, n_g in zip(v_sg, n_sg):
-            P -= integrate(v_g, n_g)
-        for sigma_g, dedsigma_g in zip(sigma_xg, dedsigma_xg):
-            P -= 2 * integrate(sigma_g, dedsigma_g)
-
-        stress_vv = P * np.eye(3)
-        for v1 in range(3):
-            for v2 in range(3):
-                stress_vv[v1, v2] -= integrate(gradn_svg[0, v1] *
-                                               gradn_svg[0, v2],
-                                               dedsigma_xg[0]) * 2
-                if nspins == 2:
-                    stress_vv[v1, v2] -= integrate(gradn_svg[0, v1] *
-                                                   gradn_svg[1, v2],
-                                                   dedsigma_xg[1]) * 2
-                    stress_vv[v1, v2] -= integrate(gradn_svg[1, v1] *
-                                                   gradn_svg[1, v2],
-                                                   dedsigma_xg[2]) * 2
         if not skip_sum:
             self.gd.comm.sum(stress_vv)
         return stress_vv
