@@ -1,6 +1,6 @@
 from __future__ import annotations
+from dataclasses import dataclass
 import functools
-import os
 from time import ctime
 
 import numpy as np
@@ -19,28 +19,6 @@ def default_ecut_extrapolation(ecut, extrapolate):
     return ecut * (1 + 0.5 * np.arange(extrapolate))**(-2 / 3)
 
 
-def rpa(filename, ecut=200.0, blocks=1, extrapolate=4):
-    """Calculate RPA energy.
-
-    filename: str
-        Name of restart-file.
-    ecut: float
-        Plane-wave cutoff.
-    blocks: int
-        Split polarizability matrix in this many blocks.
-    extrapolate: int
-        Number of cutoff energies to use for extrapolation.
-    """
-    name, ext = filename.rsplit('.', 1)
-    assert ext == 'gpw'
-    from gpaw.xc.rpa import RPACorrelation
-    rpa = RPACorrelation(name, name + '-rpa.dat',
-                         nblocks=blocks,
-                         ecut=default_ecut_extrapolation(ecut, extrapolate),
-                         txt=name + '-rpa.txt')
-    rpa.calculate()
-
-
 class GCut:
     def __init__(self, cut_G):
         self._cut_G = cut_G
@@ -79,162 +57,183 @@ def initialize_q_points(kd, qsym):
         U_scc = kd.symmetry.op_scc
         ibzq_qc = kd.get_ibz_q_points(bzq_qc, U_scc)[0]
         weight_q = kd.q_weights
-    return bzq_qc, ibzq_qc, weight_q
+    return ibzq_qc, weight_q
+
+
+@dataclass
+class RPAIntegral:
+    omega_w: np.ndarray
+    weight_w: np.ndarray
+    ibzq_qc: np.ndarray
+    weight_q: np.ndarray
+    ecut_i: np.ndarray
+
+    @property
+    def nq(self):
+        """Number of q-points."""
+        return len(self.weight_q)
+
+    @property
+    def nw(self):
+        """Number of frequencies."""
+        nw = len(self.omega_w)
+        assert len(self.weight_w) == nw
+        return nw
+
+    @property
+    def ni(self):
+        """Number of plane-wave cutoffs to extrapolate over."""
+        return len(self.ecut_i)
+
+    def integrate_frequencies(self, in_xwi):
+        out_xi = self.weight_w @ in_xwi
+        return out_xi
+
+    def integrate_qpoints(self, in_xqi):
+        out_xi = self.weight_q @ in_xqi
+        return out_xi
+
+
+@dataclass
+class RPAData:
+    integral: RPAIntegral
+
+    def __post_init__(self):
+        self.energy_qwi = np.zeros(self.shape)
+
+    @property
+    def shape(self):
+        return (self.integral.nq, self.integral.nw, self.integral.ni)
+
+    def contribution_from_qpoint(self, q, i=-1):
+        return self.integral.integrate_frequencies(self.energy_qwi[q, :, i])
+
+    @property
+    def energy_wqi(self):
+        return np.swapaxes(self.energy_qwi, 0, 1)
+
+    @property
+    def energy_qi(self):
+        """Correlation energy contribution, E_c(q)."""
+        return self.integral.integrate_frequencies(self.energy_qwi)
+
+    @property
+    def energy_wi(self):
+        """Correlation energy contribution, E_c(ω)."""
+        return self.integral.integrate_qpoints(self.energy_wqi)
+
+    @property
+    def energy_i(self):
+        """Correlation energy E_c as a function of the plane-wave cutoff."""
+        return self.integral.integrate_qpoints(self.energy_qi)
 
 
 class RPACalculator:
-    def __init__(self, gs, *, context, filename=None,
-                 ecut,
-                 skip_gamma=False, qsym=True,
-                 frequencies, weights, truncation=None,
-                 nblocks=1, calculate_q=None):
+    def __init__(
+            self,
+            gs,
+            *,
+            context,
+            ecut,
+            frequencies,
+            weights,
+            qsym=True,
+            skip_gamma=False,
+            truncation=None,
+            nblocks=1,
+            calculate_q=None
+    ):
         self.gs = gs
         self.context = context
 
-        self.omega_w = frequencies / Hartree
-        self.weight_w = weights / Hartree
-
+        # Normalize RPA integral inputs
+        if isinstance(ecut, (float, int)):
+            ecut = default_ecut_extrapolation(ecut, extrapolate=6)
+        ecut_i = np.asarray(np.sort(ecut)) / Hartree
         # TODO: We should avoid this requirement.
-        assert len(self.omega_w) % nblocks == 0
-
-        self.nblocks = nblocks
-
-        self.coulomb = CoulombKernel.from_gs(gs, truncation=truncation)
-        self.skip_gamma = skip_gamma
-
+        # thosk notes: it might work now (after some extensive clean-up of the
+        # frequency integration), but I have not tested it
+        assert len(frequencies) % nblocks == 0
         # We should actually have a kpoint descriptor for the qpoints.
         # We are badly failing at making use of the existing tools by reducing
         # the qpoints to dumb arrays.
-        self.bzq_qc, self.ibzq_qc, self.weight_q = initialize_q_points(
-            gs.kd, qsym)
+        ibzq_qc, weight_q = initialize_q_points(gs.kd, qsym)
+        # Collect information about the RPA integral on a single object (a.u.)
+        self.integral = RPAIntegral(
+            omega_w=frequencies / Hartree,
+            weight_w=weights / Hartree / (2 * np.pi),
+            ibzq_qc=ibzq_qc,
+            weight_q=weight_q,
+            ecut_i=ecut_i,
+        )
 
-        self.filename = filename
+        self.chi0calc = Chi0Calculator(
+            self.gs, self.context.with_txt('chi0.txt'),
+            nblocks=nblocks,
+            wd=FrequencyDescriptor(1j * self.integral.omega_w),
+            eta=0.0,
+            intraband=False,
+            hilbert=False,
+            ecut=max(self.integral.ecut_i) * Hartree)
+        self.coulomb = CoulombKernel.from_gs(gs, truncation=truncation)
 
+        self.skip_gamma = skip_gamma
+        # This is a super weird way of achieving inheritance...
         if calculate_q is None:
             calculate_q = self.calculate_q_rpa
         self.calculate_q = calculate_q
 
-        if isinstance(ecut, (float, int)):
-            ecut = default_ecut_extrapolation(ecut, extrapolate=6)
-        self.ecut_i = np.asarray(np.sort(ecut)) / Hartree
+    def calculate(self, *, nbands=None) -> np.ndarray:
+        """Calculate the RPA correlation energy as a function of cutoff."""
+        data = self.calculate_all_contributions(nbands=nbands)
+        return data.energy_i * Hartree  # energies in eV
 
-    def read(self, ecut_i, filename):
-        with open(filename) as fd:
-            lines = fd.readlines()[1:]
+    def calculate_all_contributions(
+            self, *, nbands=None, spin=False) -> RPAData:
+        """Calculate RPA correlation energy contributions.
 
-        n = 0
-        energy_qi = []
-        nq = len(lines) // len(ecut_i)
-        for q_c in self.ibzq_qc[:nq]:
-            energy_qi.append([])
-            for ecut in ecut_i:
-                current_inputs = np.array([*q_c, ecut * Hartree])
-                numbers_from_file = [float(x) for x in lines[n].split()]
-                previous_inputs = numbers_from_file[:-1]
-
-                if not np.allclose(current_inputs, previous_inputs):
-                    # Energies are not reusable since input parameters
-                    # have changed
-                    return []
-
-                energy = numbers_from_file[-1]
-                energy_qi[-1].append(energy / Hartree)
-                n += 1
-
-        return energy_qi
-
-    def energies_to_string(self, energy_qi, ecut_i):
-        lines = []
-        app = lines.append
-        app('q1 q2 q3 E_cut E_c(q)')
-        for energy_i, q_c in zip(energy_qi, self.ibzq_qc):
-            for energy, ecut in zip(energy_i, ecut_i):
-                tokens = [repr(num) for num in
-                          (*q_c, ecut * Hartree, energy * Hartree)]
-                app(' '.join(tokens))
-
-    def write(self, energy_qi, ecut_i):
-        txt = self.energies_to_string(energy_qi, ecut_i)
-        if self.context.comm.rank == 0 and self.filename:
-            with open(self.filename, 'w') as fd:
-                print(txt, file=fd)
-
-    def calculate(self, *, nbands=None, spin=False, txt=''):
-        """Calculate RPA correlation energy for one or several cutoffs.
-
-        ecut: float or list of floats
-            Plane-wave cutoff(s) in eV.
         nbands: int
             Number of bands (defaults to number of plane-waves).
         spin: bool
             Separate spin in response function.
             (Only needed for beyond RPA methods that inherit this function).
-        txt:
-            Prefix for the chi0.txt file. Added as {txt}_chi0.txt
         """
 
         p = functools.partial(self.context.print, flush=False)
 
-        ecut_i = self.ecut_i
-        ecutmax = max(ecut_i)
-
         if nbands is None:
             p('Response function bands : Equal to number of plane waves')
         else:
             p('Response function bands : %s' % nbands)
         p('Plane wave cutoffs (eV) :', end='')
-        for e in ecut_i:
+        for e in self.integral.ecut_i:
             p(f' {e * Hartree:.3f}', end='')
         p()
         p(self.coulomb.description())
-        self.context.print('')
-
-        if self.filename and os.path.isfile(self.filename):
-            energy_qi = self.read(ecut_i, self.filename)
-            self.context.print(
-                'Read %d q-points from file: %s\n' % (len(energy_qi)))
-
-            self.context.comm.barrier()
-
-        chi0calc = Chi0Calculator(
-            self.gs, self.context.with_txt(
-                f'{txt + "_" if txt else ""}chi0.txt'),
-            nblocks=self.nblocks,
-            wd=FrequencyDescriptor(1j * self.omega_w),
-            eta=0.0,
-            intraband=False,
-            hilbert=False,
-            ecut=ecutmax * Hartree)
-
-        self.blockcomm = chi0calc.chi0_body_calc.blockcomm
-
-        energy_qi = []
-        nq = len(energy_qi)
+        p('', flush=True)
 
         self.context.timer.start('RPA')
 
-        for q_c in self.ibzq_qc[nq:]:
+        data = RPAData(self.integral)
+        ecutmax = max(self.integral.ecut_i)
+        for q, q_c in enumerate(self.integral.ibzq_qc):
             if np.allclose(q_c, 0.0) and self.skip_gamma:
-                energy_qi.append(len(ecut_i) * [0.0])
-                self.write(energy_qi, ecut_i)
-                p('Not calculating E_c(q) at Gamma')
-                p()
+                p('Not calculating E_c(q) at Gamma', end='\n')
                 continue
 
-            chi0_s = [chi0calc.create_chi0(q_c)]
+            chi0_s = [self.chi0calc.create_chi0(q_c)]
             if spin:
-                chi0_s.append(chi0calc.create_chi0(q_c))
+                chi0_s.append(self.chi0calc.create_chi0(q_c))
 
             qpd = chi0_s[0].qpd
             nG = qpd.ngmax
 
             # First not completely filled band:
             m1 = self.gs.nocc1
-            p(f'# {len(energy_qi)}  -  {ctime().split()[-2]}')
+            p(f'# {q}  -  {ctime().split()[-2]}')
             p('q = [%1.3f %1.3f %1.3f]' % tuple(q_c))
 
-            energy_i = []
-            for ecut in ecut_i:
+            for i, ecut in enumerate(self.integral.ecut_i):
                 if ecut == ecutmax:
                     # Nothing to cut away:
                     gcut = GCut(None)
@@ -245,26 +244,26 @@ class RPACalculator:
 
                 p('E_cut = %d eV / Bands = %d:' % (ecut * Hartree, m2),
                   end='\n', flush=True)
-
-                energy = self.calculate_q(chi0calc, chi0_s, m1, m2, gcut)
-
-                energy_i.append(energy)
+                p('E_c(q) = ', end='', flush=False)
+                data.energy_qwi[q, :, i] = self.calculate_q(
+                    chi0_s, m1, m2, gcut
+                )
+                energy = data.contribution_from_qpoint(q, i=i)
+                p('%.3f eV' % (energy * Hartree), flush=True)
                 m1 = m2
 
-            energy_qi.append(energy_i)
-            self.write(energy_qi, ecut_i)
             p()
 
-        e_i = np.dot(self.weight_q, np.array(energy_qi))
+        e_i = data.energy_i
         p('==========================================================')
         p()
         p('Total correlation energy:')
-        for e_cut, e in zip(ecut_i, e_i):
+        for e_cut, e in zip(self.integral.ecut_i, e_i):
             p(f'{e_cut * Hartree:6.0f}:   {e * Hartree:6.4f} eV')
         p()
 
         if len(e_i) > 1:
-            self.extrapolate(e_i, ecut_i)
+            self.extrapolate(e_i, self.integral.ecut_i)
 
         p('Calculation completed at: ', ctime())
         p()
@@ -272,71 +271,51 @@ class RPACalculator:
         self.context.timer.stop('RPA')
         self.context.write_timer()
 
-        return e_i * Hartree
+        return data
 
     @timer('chi0(q)')
-    def calculate_q_rpa(self, chi0calc, chi0_s,
-                        m1, m2, gcut):
+    def calculate_q_rpa(self, chi0_s, m1, m2, gcut):
         chi0 = chi0_s[0]
-        chi0calc.update_chi0(
-            chi0, m1, m2, spins=range(chi0calc.gs.nspins))
-
-        self.context.print('E_c(q) = ', end='', flush=False)
-
-        chi0_wGG = chi0.body.copy_array_with_distribution('wGG')
-
-        kd = self.gs.kd
-        if not chi0.qpd.optical_limit:
-            e = self.calculate_energy_rpa(chi0.qpd, chi0_wGG, gcut)
-            self.context.print('%.3f eV' % (e * Hartree))
+        self.chi0calc.update_chi0(
+            chi0, m1=m1, m2=m2, spins=range(self.chi0calc.gs.nspins))
+        qpd = chi0.qpd
+        chi0_wGG = chi0.body.get_distributed_frequencies_array()
+        wblocks = chi0.body.get_distributed_frequencies_blocks1d()
+        # Calculate RPA energy contributions (as a function of w)
+        if chi0.qpd.optical_limit:
+            chi0_wvv = chi0.chi0_Wvv[wblocks.myslice]
+            chi0_wxvG = chi0.chi0_WxvG[wblocks.myslice]
+            energy_w = self.calculate_optical_limit_rpa_energies(
+                qpd, chi0_wGG, chi0_wvv, chi0_wxvG, gcut
+            )
         else:
-            from gpaw.response.gamma_int import GammaIntegrator
-            from gpaw.response.pw_parallelization import Blocks1D
+            energy_w = self.calculate_rpa_energies(qpd, chi0_wGG, gcut)
+        return wblocks.all_gather(energy_w)
 
-            wblocks = Blocks1D(self.blockcomm, len(self.omega_w))
-            gamma_int = GammaIntegrator(
-                truncation=self.coulomb.truncation,
-                kd=kd,
-                qpd=chi0.qpd,
-                chi0_wvv=chi0.chi0_Wvv[wblocks.myslice],
-                chi0_wxvG=chi0.chi0_WxvG[wblocks.myslice])
+    def calculate_optical_limit_rpa_energies(
+            self, qpd, chi0_wGG, chi0_wvv, chi0_wxvG, gcut):
+        """Calculate correlation energy from chi0 in the optical limit."""
+        from gpaw.response.gamma_int import GammaIntegral
 
-            e = 0
-            for iqf in range(len(gamma_int.qf_qv)):
-                for iw in range(wblocks.nlocal):
-                    gamma_int.set_appendages(chi0_wGG[iw], iw, iqf)
-                ev = self.calculate_energy_rpa(chi0.qpd, chi0_wGG, gcut,
-                                               q_v=gamma_int.qf_qv[iqf])
-                e += ev * gamma_int.weight_q
-            self.context.print('%.3f eV' % (e * Hartree))
+        gamma_int = GammaIntegral(self.coulomb, qpd=qpd)
 
-        return e
+        energy_w = []
+        for chi0_GG, chi0_vv, chi0_xvG in zip(chi0_wGG, chi0_wvv, chi0_wxvG):
+            # Integrate over the optical wave vector volume
+            energy = 0.
+            for qweight, sqrtV_G, chi0_mapping in gamma_int:
+                chi0p_GG = chi0_mapping(chi0_GG, chi0_vv, chi0_xvG)
+                energy += qweight * single_rpa_energy(
+                    chi0p_GG, gcut.cut(sqrtV_G), gcut)
+            energy_w.append(energy)
+        return np.array(energy_w)
 
-    @timer('Energy')
-    def calculate_energy_rpa(self, qpd, chi0_wGG, gcut, q_v=None):
-        """Evaluate correlation energy from chi0."""
-
-        sqrtV_G = gcut.cut(self.coulomb.sqrtV(qpd, q_v))
-
-        nG = len(sqrtV_G)
-
-        e_w = []
-        for chi0_GG in chi0_wGG:
-            chi0_GG = gcut.cut(chi0_GG, [0, 1])
-
-            e_GG = np.eye(nG) - chi0_GG * sqrtV_G * sqrtV_G[:, np.newaxis]
-            e = np.log(np.linalg.det(e_GG)) + nG - np.trace(e_GG)
-            e_w.append(e.real)
-
-        self.E_w, energy = self.gather_energies(e_w)
-        return energy
-
-    def gather_energies(self, e_w):
-        E_w = np.zeros_like(self.omega_w)
-        # XXX This requires all cores to the same number of w doesn't it?
-        self.blockcomm.all_gather(np.array(e_w), E_w)
-        energy = E_w @ self.weight_w / (2 * np.pi)
-        return E_w, energy
+    def calculate_rpa_energies(self, qpd, chi0_wGG, gcut):
+        """Evaluate correlation energy from chi0 at finite q."""
+        sqrtV_G = gcut.cut(self.coulomb.sqrtV(qpd, q_v=None))
+        return np.array([
+            single_rpa_energy(chi0_GG, sqrtV_G, gcut) for chi0_GG in chi0_wGG
+        ])
 
     def extrapolate(self, e_i, ecut_i):
         self.context.print('Extrapolated energies:', flush=False)
@@ -355,6 +334,14 @@ class RPACalculator:
         return e_i * Hartree
 
 
+def single_rpa_energy(chi0_GG, sqrtV_G, gcut):
+    nG = len(sqrtV_G)
+    chi0_GG = gcut.cut(chi0_GG, [0, 1])
+    e_GG = np.eye(nG) - chi0_GG * sqrtV_G * sqrtV_G[:, np.newaxis]
+    e = np.log(np.linalg.det(e_GG)) + nG - np.trace(e_GG)
+    return e.real
+
+
 def get_gauss_legendre_points(nw=16, frequency_max=800.0, frequency_scale=2.0):
     y_w, weights_w = p_roots(nw)
     y_w = y_w.real
@@ -384,9 +371,7 @@ class RPACorrelation(RPACalculator):
         xc: str
             Exchange-correlation kernel. This is only different from RPA when
             this object is constructed from a different module - e.g. fxc.py
-        filename: str
-            txt output
-        skip_gamme: bool
+        skip_gamma: bool
             If True, skip q = [0,0,0] from the calculation
         qsym: bool
             Use symmetry to reduce q-points
@@ -402,7 +387,7 @@ class RPACorrelation(RPACalculator):
             increase to e.g. 2.5 or 3.0 improves convergence wth respect to
             frequency points for metals
         frequencies: list
-            List of frequancies for user-specified frequency integration
+            List of frequencies for user-specified frequency integration
         weights: list
             list of weights (integration measure) for a user specified
             frequency grid. Must be specified and have the same length as
@@ -415,6 +400,8 @@ class RPACorrelation(RPACalculator):
             Number of parallelization blocks. Frequency parallelization
             can be specified by setting nblocks=nfrequencies and is useful
             for memory consuming calculations
+        ecut: float or list of floats
+            Plane-wave cutoff(s) in eV.
         txt: str
             txt file for saving and loading contributions to the correlation
             energy from different q-points
@@ -456,10 +443,9 @@ class RPACorrelation(RPACalculator):
         p('Number of spins                :', self.gs.nspins)
         p('Number of k-points             :', len(self.gs.kd.bzk_kc))
         p('Number of irreducible k-points :', len(self.gs.kd.ibzk_kc))
-        p('Number of q-points             :', len(self.bzq_qc))
-        p('Number of irreducible q-points :', len(self.ibzq_qc))
+        p('Number of irreducible q-points :', len(self.integral.ibzq_qc))
         p()
-        for q, weight in zip(self.ibzq_qc, self.weight_q):
+        for q, weight in zip(self.integral.ibzq_qc, self.integral.weight_q):
             p('    q: [%1.4f %1.4f %1.4f] - weight: %1.3f' %
               (q[0], q[1], q[2], weight))
         p()
@@ -475,17 +461,18 @@ class RPACorrelation(RPACalculator):
         p('Frequencies')
         if not user_spec:
             p('    Gauss-Legendre integration with %s frequency points' %
-              len(self.omega_w))
+              len(self.integral.omega_w))
             p('    Transformed from [0,oo] to [0,1] using e^[-aw^(1/B)]')
             p('    Highest frequency point at %5.1f eV and B=%1.1f' %
-              (self.omega_w[-1] * Hartree, frequency_scale))
+              (self.integral.omega_w[-1] * Hartree, frequency_scale))
         else:
             p('    User specified frequency integration with',
-              len(self.omega_w), 'frequency points')
+              len(self.integral.omega_w), 'frequency points')
         p()
         p('Parallelization')
         p('    Total number of CPUs          : % s' % self.context.comm.size)
-        p('    G-vector decomposition        : % s' % self.nblocks)
-        p('    K-point/band decomposition    : % s' %
-          (self.context.comm.size // self.nblocks))
+        blockcomm = self.chi0calc.chi0_body_calc.blockcomm
+        p('    G-vector decomposition        : % s' % blockcomm.size)
+        kncomm = self.chi0calc.chi0_body_calc.kncomm
+        p('    K-point/band decomposition    : % s' % kncomm.size)
         self.context.print('')
